Package: toxpiR
Type: Package
Title: Create ToxPi Prioritization Models
Version: 1.3.2
Authors@R: 
    c(person("Jonathon F", "Fleming",
             role = c("aut","cre"),
             email = "jffleming0129@gmail.com",
             comment = c(ORCID = "0000-0003-2447-3139")),
      person("Dayne L", "Filer", 
             role = c("aut", "fnd"), 
             email = "dayne.filer@gmail.com",
             comment = c(ORCID = "0000-0002-3443-5315")),
      person("Dillon T", "Lloyd",
             role = "aut"),
      person("Preethi", "Thunga",
             role = "aut",
             comment = c(ORCID = "0000-0001-5447-0129")),
      person("Skylar W", "Marvel",
             role = "aut"),
      person("Alison A", "Motsinger-Reif",
             role = c("fnd"),
             comment = c(ORCID = "0000-0003-1346-2493")),
      person("David M", "Reif",
             role = c("aut", "fnd"),
             email = "reif.david@gmail.com",
             comment = c(ORCID = "0000-0001-7815-6767")))
Description: 
  Enables users to build 'ToxPi' prioritization models and provides 
  functionality within the grid framework for plotting ToxPi graphs.
  'toxpiR' allows for more customization than the 'ToxPi GUI' 
  (<https://toxpi.github.io/>) and integration into existing workflows for greater 
  ease-of-use, reproducibility, and transparency.
  toxpiR package behaves nearly identically to the GUI; the package 
  documentation includes notes about all differences.
  The vignettes download example files from 
  <https://github.com/ToxPi/ToxPi-example-files>.
Imports: grDevices, methods, S4Vectors, grid, rlang, stats,
        BiocGenerics, tidyr, utils, ggplot2
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.3
Depends: R (>= 4.0)
Suggests: rmarkdown, knitr, testthat (>= 3.0.0), covr, DBI
Config/testthat/edition: 3
Config/testthat/parallel: true
VignetteBuilder: knitr
License: GPL (>= 3)
URL: https://github.com/ToxPi/toxpiR, https://toxpi.github.io/toxpiR/
BugReports: https://github.com/ToxPi/toxpiR/issues
NeedsCompilation: no
Packaged: 2026-05-01 15:59:04 UTC; flemingjf
Author: Jonathon F Fleming [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-2447-3139>),
  Dayne L Filer [aut, fnd] (ORCID:
    <https://orcid.org/0000-0002-3443-5315>),
  Dillon T Lloyd [aut],
  Preethi Thunga [aut] (ORCID: <https://orcid.org/0000-0001-5447-0129>),
  Skylar W Marvel [aut],
  Alison A Motsinger-Reif [fnd] (ORCID:
    <https://orcid.org/0000-0003-1346-2493>),
  David M Reif [aut, fnd] (ORCID:
    <https://orcid.org/0000-0001-7815-6767>)
Maintainer: Jonathon F Fleming <jffleming0129@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-01 16:30:02 UTC
Built: R 4.5.2; ; 2026-05-01 19:11:58 UTC; unix

Package: toxpiR
Type: Package
Title: Create ToxPi Prioritization Models
Version: 1.3.2
Authors@R: 
    c(person("Jonathon F", "Fleming",
             role = c("aut","cre"),
             email = "jffleming0129@gmail.com",
             comment = c(ORCID = "0000-0003-2447-3139")),
      person("Dayne L", "Filer", 
             role = c("aut", "fnd"), 
             email = "dayne.filer@gmail.com",
             comment = c(ORCID = "0000-0002-3443-5315")),
      person("Dillon T", "Lloyd",
             role = "aut"),
      person("Preethi", "Thunga",
             role = "aut",
             comment = c(ORCID = "0000-0001-5447-0129")),
      person("Skylar W", "Marvel",
             role = "aut"),
      person("Alison A", "Motsinger-Reif",
             role = c("fnd"),
             comment = c(ORCID = "0000-0003-1346-2493")),
      person("David M", "Reif",
             role = c("aut", "fnd"),
             email = "reif.david@gmail.com",
             comment = c(ORCID = "0000-0001-7815-6767")))
Description: 
  Enables users to build 'ToxPi' prioritization models and provides 
  functionality within the grid framework for plotting ToxPi graphs.
  'toxpiR' allows for more customization than the 'ToxPi GUI' 
  (<https://toxpi.github.io/>) and integration into existing workflows for greater 
  ease-of-use, reproducibility, and transparency.
  toxpiR package behaves nearly identically to the GUI; the package 
  documentation includes notes about all differences.
  The vignettes download example files from 
  <https://github.com/ToxPi/ToxPi-example-files>.
Imports: grDevices, methods, S4Vectors, grid, rlang, stats,
        BiocGenerics, tidyr, utils, ggplot2
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.3
Depends: R (>= 4.0)
Suggests: rmarkdown, knitr, testthat (>= 3.0.0), covr, DBI
Config/testthat/edition: 3
Config/testthat/parallel: true
VignetteBuilder: knitr
License: GPL (>= 3)
URL: https://github.com/ToxPi/toxpiR, https://toxpi.github.io/toxpiR/
BugReports: https://github.com/ToxPi/toxpiR/issues
NeedsCompilation: no
Packaged: 2026-05-01 15:59:04 UTC; flemingjf
Author: Jonathon F Fleming [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-2447-3139>),
  Dayne L Filer [aut, fnd] (ORCID:
    <https://orcid.org/0000-0002-3443-5315>),
  Dillon T Lloyd [aut],
  Preethi Thunga [aut] (ORCID: <https://orcid.org/0000-0001-5447-0129>),
  Skylar W Marvel [aut],
  Alison A Motsinger-Reif [fnd] (ORCID:
    <https://orcid.org/0000-0003-1346-2493>),
  David M Reif [aut, fnd] (ORCID:
    <https://orcid.org/0000-0001-7815-6767>)
Maintainer: Jonathon F Fleming <jffleming0129@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-01 16:30:02 UTC
Built: R 4.6.0; ; 2026-05-01 20:21:21 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(editDetails,pieGrob)
export(TxpModel)
export(TxpModelList)
export(TxpResultList)
export(TxpSlice)
export(TxpSliceList)
export(TxpTransFunc)
export(TxpTransFuncList)
export(as.TxpModelList)
export(as.TxpResultList)
export(as.TxpSliceList)
export(as.TxpTransFuncList)
export(grid.pieGridGrob)
export(grid.pieGrob)
export(pieGridGrob)
export(pieGrob)
export(txpCalculateScores)
export(txpExportGui)
export(txpImportGui)
exportClasses(TxpModel)
exportClasses(TxpModelList)
exportClasses(TxpResult)
exportClasses(TxpResultList)
exportClasses(TxpResultParam)
exportClasses(TxpSlice)
exportClasses(TxpSliceList)
exportClasses(TxpTransFunc)
exportClasses(TxpTransFuncList)
exportMethods("[")
exportMethods("names<-")
exportMethods("txpIDs<-")
exportMethods("txpSlices<-")
exportMethods("txpTransFuncs<-")
exportMethods("txpValueNames<-")
exportMethods("txpWeights<-")
exportMethods(as.data.frame)
exportMethods(duplicated)
exportMethods(length)
exportMethods(merge)
exportMethods(names)
exportMethods(plot)
exportMethods(sort)
exportMethods(txpCalculateScores)
exportMethods(txpIDs)
exportMethods(txpMissing)
exportMethods(txpModel)
exportMethods(txpRanks)
exportMethods(txpResultParam)
exportMethods(txpScores)
exportMethods(txpSliceScores)
exportMethods(txpSlices)
exportMethods(txpTransFuncs)
exportMethods(txpValueNames)
exportMethods(txpWeights)
import(ggplot2)
import(grid)
import(methods)
importClassesFrom(S4Vectors,List)
importClassesFrom(S4Vectors,SimpleList)
importClassesFrom(S4Vectors,character_OR_NULL)
importFrom(BiocGenerics,as.data.frame)
importFrom(BiocGenerics,duplicated)
importFrom(BiocGenerics,sort)
importFrom(BiocGenerics,updateObject)
importFrom(S4Vectors,List)
importFrom(S4Vectors,classNameForDisplay)
importFrom(S4Vectors,coolcat)
importFrom(S4Vectors,new2)
importFrom(S4Vectors,setValidity2)
importFrom(grDevices,col2rgb)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,extendrange)
importFrom(grDevices,rgb)
importFrom(rlang,is_integerish)
importFrom(rlang,is_named)
importFrom(rlang,is_scalar_character)
importFrom(rlang,is_scalar_integerish)
importFrom(rlang,is_scalar_logical)
importFrom(stats,sd)
importFrom(tidyr,separate)
importFrom(utils,data)
importFrom(utils,read.csv)
importFrom(utils,type.convert)
importFrom(utils,write.table)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----setup--------------------------------------------------------------------
library(toxpiR)

## Create a tempfile and download 'format_C.csv'
fmtc <- tempfile()
ghuc <- "https://raw.githubusercontent.com"
fmtcUrl <- file.path(ghuc, "ToxPi", "ToxPi-example-files", "main", "format_C.csv")
download.file(url = fmtcUrl, destfile = fmtc, quiet = TRUE)

## -----------------------------------------------------------------------------
## Import file into R
gui <- txpImportGui(fmtc)


## -----------------------------------------------------------------------------
gui$model
gui$input
gui$fills

## -----------------------------------------------------------------------------
## Calculate ToxPi scores
res <- txpCalculateScores(model = gui$model, input = gui$input, id.var = "Name",negative.value.handling = "missing")

## Overall ToxPi scores
txpScores(res)

## Slice scores
txpSliceScores(res, adjusted = FALSE)

## -----------------------------------------------------------------------------
out <- as.data.frame(res, adjusted = FALSE)
out <- out[order(out$score, decreasing = TRUE), ]
out

## ----fig.width = 7------------------------------------------------------------
plot(sort(res), fills = gui$fills)

## ----fig.width = 7, fig.height = 4--------------------------------------------
plot(res, txpRanks(res))
plot(res, txpRanks(res), labels = 1:10, pch = 16, size = grid::unit(0.75, "char"))

## ----fig.width = 7, fig.height = 5--------------------------------------------
## Hierarchical Clustering
hc <- hclust(dist(txpSliceScores(res)), method = 'complete')
plot(hc, hang = -1, labels = txpIDs(res), xlab = 'Name', sub = '')

## ----fig.width = 7, fig.height = 5--------------------------------------------
## K-Means Clustering, plotted using principal components
nClusters <- 3
km <- kmeans(txpSliceScores(res), nClusters)
pc <- prcomp(txpSliceScores(res))
coord <- predict(pc) * -sum(txpWeights(res))
plot(coord[,1], coord[,2], col = km$cluster, 
     xlab = 'PC1', ylab = 'PC2', pch = 16)

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.retina = 6
)

## ----eval = FALSE-------------------------------------------------------------
# install.packages("toxpiR")

## ----eval = FALSE-------------------------------------------------------------
# remotes::install_github("ToxPi/toxpiR")

## ----packages, warning=FALSE--------------------------------------------------
library(toxpiR)

## -----------------------------------------------------------------------------
data(txp_example_input, package = "toxpiR")
head(txp_example_input)

## -----------------------------------------------------------------------------

## Goal - Create two slices with transformation functions 
# Slice 1: Metric 1, No transformation 
# Slice 2: Metric 2 (square) and Metric 3 (no transformation)

slice2.trans <- TxpTransFuncList(func1 = function(x) x^2, func2 = NULL)

f.slices <- TxpSliceList(Slice1 = TxpSlice("metric1"), 
                         Slice2 = TxpSlice(c("metric2", "metric3"), 
                                           txpTransFuncs = slice2.trans ))


## -----------------------------------------------------------------------------

## Goal - Create ToxPi model.
# Slice 1, weight = 2
# Slice 2, weight = 1, apply log transform to final value. 

# Object storing list of transformation functions.
final.trans <- TxpTransFuncList(f1 = NULL, f2 = function(x) log10(x)) 

f.model <- TxpModel(txpSlices = f.slices, 
                    txpWeights = c(2,1),
                    txpTransFuncs = final.trans)


## -----------------------------------------------------------------------------
f.model <- txp_example_model #Load a more complex 4 slice model
f.results <- txpCalculateScores(model = f.model,
                                input = txp_example_input,
                                id.var = 'name' ) 

txpSliceScores(f.results) #ToxPi scores
txpWeights(f.results) #Print weights
txpMissing(f.results) #Proportion of missing data within each slice

## ----eval=FALSE---------------------------------------------------------------
# library("ggplot2")
# 
# # Default plot
# plot(f.results, package = "gg") # Using ggplot package

## ----fig.show='hold', fig.width=4, fig.height=4, echo=FALSE-------------------
library("ggplot2")

# Default plot
plot(f.results, package = "gg") # Using ggplot package

## ----fig.show = "hide"--------------------------------------------------------
## Optional code to make smoother lines

# Plot before updating coord_munch
plot(f.results["chem01"], package = "gg") 

# Save the original version of coord_munch
coord_munch_orig <- ggplot2:::coord_munch

# Make a wrapper function that has a different default for segment_length
if (length(formals(coord_munch_orig)) == 5) {
  coord_munch_new <- function(coord, data, range, segment_length = 1/1000,
                              is_closed = FALSE) {
    coord_munch_orig(coord, data, range, segment_length, is_closed)
  }
} else {
  coord_munch_new <- function(coord, data, range, segment_length = 1/1000) {
    coord_munch_orig(coord, data, range, segment_length)
  }
}

# The environment may need to be set
#environment(coord_munch_new) <- environment(coord_munch_orig)

# Replace ggplot2:::coord_munch with coord_munch_new
assignInNamespace("coord_munch", coord_munch_new, ns = "ggplot2")

# Plot after updating coord_munch
plot(f.results["chem01"], package = "gg") 

# Revert to original coord_munch ater plotting if desired 
#assignInNamespace("coord_munch", coord_munch_orig, ns = "ggplot2")

## ----eval=FALSE---------------------------------------------------------------
# # Changing the slice colors
# colors <- c("orange", "green", "magenta", "lightblue")
# plot(f.results["chem02"], package = "gg", fills = colors)
# plot(f.results["chem02"], package = "gg", fills = NULL)

## ----fig.show="hold", echo=FALSE----------------------------------------------
# Changing the slice colors
colors <- c("orange", "green", "magenta", "lightblue")
plot(f.results["chem02"], package = "gg", fills = colors)
plot(f.results["chem02"], package = "gg", fills = NULL)

## ----eval=FALSE---------------------------------------------------------------
# # Changing the background color
# plot(f.results["chem02"], package = "gg", bgColor = "lightskyblue")
# plot(f.results["chem02"], package = "gg", bgColor = NULL)

## ----fig.show="hold", echo=FALSE----------------------------------------------
# Changing the background color
plot(f.results["chem02"], package = "gg", bgColor = "lightskyblue")
plot(f.results["chem02"], package = "gg", bgColor = NULL)

## ----eval=FALSE---------------------------------------------------------------
# # Changing the max radius ring color
# plot(f.results["chem02"], package = "gg", borderColor = "black")
# plot(f.results["chem02"], package = "gg", borderColor = NULL)

## ----fig.show="hold", echo=FALSE----------------------------------------------
# Changing the max radius ring color
plot(f.results["chem02"], package = "gg", borderColor = "black")
plot(f.results["chem02"], package = "gg", borderColor = NULL)

## ----eval=FALSE---------------------------------------------------------------
# # Changing the slice border color
# plot(f.results["chem02"], package = "gg", sliceBorderColor = "magenta")
# plot(f.results["chem02"], package = "gg", sliceBorderColor = NULL)

## ----fig.show="hold", echo=FALSE----------------------------------------------
# Changing the slice border color
plot(f.results["chem02"], package = "gg", sliceBorderColor = "magenta")
plot(f.results["chem02"], package = "gg", sliceBorderColor = NULL)

## ----eval=FALSE---------------------------------------------------------------
# # Adding slice guidelines
# plot(f.results["chem02"], package = "gg", sliceLineColor = "red")
# plot(f.results["chem02"], package = "gg", sliceLineColor = NULL)

## ----fig.show="hold", echo=FALSE----------------------------------------------
# Adding slice guidelines 
plot(f.results["chem02"], package = "gg", sliceLineColor = "red")
plot(f.results["chem02"], package = "gg", sliceLineColor = NULL) 

## ----eval=FALSE---------------------------------------------------------------
# # Adding visible slice scores
# plot(f.results["chem02"], package = "gg", sliceValueColor = "brown")
# plot(f.results["chem02"], package = "gg", sliceValueColor = NULL)

## ----fig.show="hold", echo=FALSE----------------------------------------------
# Adding visible slice scores
plot(f.results["chem02"], package = "gg", sliceValueColor = "brown")
plot(f.results["chem02"], package = "gg", sliceValueColor = NULL)

## ----eval=FALSE---------------------------------------------------------------
# # Hiding inner circle
# plot(f.results["chem02"], package = "gg", showCenter = TRUE)
# plot(f.results["chem02"], package = "gg", showCenter = FALSE)

## ----fig.show="hold", echo=FALSE----------------------------------------------
# Hiding inner circle
plot(f.results["chem02"], package = "gg", showCenter = TRUE)
plot(f.results["chem02"], package = "gg", showCenter = FALSE)

## ----eval=FALSE---------------------------------------------------------------
# # Hiding missing data information (pure white inner circle)
# plot(f.results["chem02"], package = "gg", showMissing = TRUE)
# plot(f.results["chem02"], package = "gg", showMissing = FALSE)

## ----fig.show="hold", echo=FALSE----------------------------------------------
# Hiding missing data information (pure white inner circle)
plot(f.results["chem02"], package = "gg", showMissing = TRUE)
plot(f.results["chem02"], package = "gg", showMissing = FALSE)

## ----eval=FALSE---------------------------------------------------------------
# # Hiding the overall profile scores
# plot(f.results["chem02"], package = "gg", showScore = TRUE)
# plot(f.results["chem02"], package = "gg", showScore = FALSE)

## ----fig.show="hold", echo=FALSE----------------------------------------------
# Hiding the overall profile scores
plot(f.results["chem02"], package = "gg", showScore = TRUE)
plot(f.results["chem02"], package = "gg", showScore = FALSE) 

## ----fig.width = 7, fig.align='center'----------------------------------------
# Specifying the number of columns in the plot
plot(f.results, package = "gg", ncol = 5) 

## ----eval=FALSE---------------------------------------------------------------
# # Moving the legend using ggplot built in theme functions
# plot(f.results, package = "gg", ncol = 5) + theme(legend.position = "bottom")
# plot(f.results, package = "gg", ncol = 2) + theme(legend.position = "left")

## ----fig.show="hold", fig.height=5, echo=FALSE--------------------------------
# Moving the legend using ggplot built in theme functions
plot(f.results, package = "gg", ncol = 5) + theme(legend.position = "bottom") 
plot(f.results, package = "gg", ncol = 2) + theme(legend.position = "left")

## ----fig.align='center'-------------------------------------------------------
# Removing plot margins
plot(f.results, package = "gg") + theme(plot.margin = margin(0, 0, 0, 0, "cm"))
# Removing spacing between panels
plot(f.results, package = "gg") + theme(panel.spacing = unit(0, "lines"))
# Removing text labels
plot(f.results, package = "gg") + theme(strip.text.x = element_blank())
# A combination of the above for a rank ordered plot
plot(f.results[order(txpRanks(f.results)[1:9])], package = "gg") +
  theme(
    plot.margin = margin(0, 0, 0, 0, "cm"),
    panel.spacing = unit(0, "lines"),
    strip.text.x = element_blank(),
    legend.position = "none"
  )

## ----fig.align='center'-------------------------------------------------------
## Creating an example with more variable missing data amounts
f.input <- txp_example_input

# Add more missing data to slice 2 via metric3 in the raw data
f.input[3:10, "metric3"] <- NA

# Modify transformation function for slice 4
txpSlices(f.model)[[4]] <- TxpSlice("metric8", c(fn = \(x) sqrt(x - 30)))

# Calculate new ToxPi results
f.results_missing <- txpCalculateScores(f.model, f.input, id.var = "name")

# View missing data proportions
txpMissing(f.results_missing)

# View new result profile
plot(f.results_missing["chem02"], package = "gg")

## ----fig.width=7, fig.height=3------------------------------------------------
library(grid) # Load library
plot(f.results) # ToxPi visuals
# grid.ls() #List grid info

# Highlight one figure using its label
grid.edit("pie-1", fills = c("red", "blue", "black", "brown"))

# Or just one slice in a figure
grid.edit("pie-10::slice1", gp = gpar(fill = "#FC0FC0"))

## ----eval=FALSE---------------------------------------------------------------
# #Single sample
# plot(f.results["chem02"])
# plot(f.results["chem02"], package = "gg")

## ----fig.show="hold", echo=FALSE----------------------------------------------
#Single sample
plot(f.results["chem02"])
plot(f.results["chem02"], package = "gg") 

## ----eval=FALSE---------------------------------------------------------------
# # Subset plots
# plot(f.results[order(txpRanks(f.results))[1:4]]) #Profiles ranked 1-4
# plot(f.results[order(txpRanks(f.results))[1:4]], package = "gg") #Profiles ranked 1-4

## ----fig.show="hold", echo=FALSE----------------------------------------------
# Subset plots
plot(f.results[order(txpRanks(f.results))[1:4]]) #Profiles ranked 1-4
plot(f.results[order(txpRanks(f.results))[1:4]], package = "gg") #Profiles ranked 1-4

## ----eval=FALSE---------------------------------------------------------------
# ## Long sample names for cramped plots
# 
# #change the first sample name in f.results
# txpIDs(f.results)[1] <- "I am a long sample name"
# 
# plot(f.results) #grid plot for all samples
# plot(f.results, package = "gg") #ggplot for all samples
# 
# txpIDs(f.results)[1] <- "chem01" # Change the sample name back

## ----fig.show="hold", echo=FALSE----------------------------------------------
## Long sample names for cramped plots

#change the first sample name in f.results
txpIDs(f.results)[1] <- "I am a long sample name"

plot(f.results) #grid plot for all samples
plot(f.results, package = "gg") #ggplot for all samples

txpIDs(f.results)[1] <- "chem01" # Change the sample name back

## ----eval=FALSE---------------------------------------------------------------
# ## Long slice names for cramped plots
# 
# #change first slice name in model slot
# names(f.results@txpModel)[1] <- "long slice name"
# #change first slice name in scores slot
# colnames(f.results@txpSliceScores)[1] <- "long slice name"
# #change first slice name in missing data slot
# names(f.results@txpMissing)[1] <- "long slice name"
# 
# #plot results using grid
# plot(f.results)
# #plot results using ggplot
# plot(f.results, package = "gg") + theme(
#   legend.position = "bottom",
#   legend.title = element_text(size = 10),
#   legend.text = element_text(size = 6)
# )
# 
# #change slice name back
# names(f.results@txpModel)[1] <- "s1"
# colnames(f.results@txpSliceScores)[1] <- "s1"
# names(f.results@txpMissing)[1] <- "s1"

## ----fig.show="hold", echo=FALSE----------------------------------------------
## Long slice names for cramped plots

#change first slice name in model slot
names(f.results@txpModel)[1] <- "long slice name"
#change first slice name in scores slot
colnames(f.results@txpSliceScores)[1] <- "long slice name"
#change first slice name in missing data slot
names(f.results@txpMissing)[1] <- "long slice name"

#plot results using grid
plot(f.results)
#plot results using ggplot
plot(f.results, package = "gg") + theme(
  legend.position = "bottom",
  legend.title = element_text(size = 10),
  legend.text = element_text(size = 6)
)

#change slice name back
names(f.results@txpModel)[1] <- "s1"
colnames(f.results@txpSliceScores)[1] <- "s1"
names(f.results@txpMissing)[1] <- "s1"

## ----fig.width=5, fig.height=4, fig.align='center'----------------------------
# Rank plot
plot(f.results, y = txpRanks(f.results), labels = 1:10)

# Hierarchical Clustering
f.hc <- hclust(dist(txpSliceScores(f.results)))

plot(f.hc, hang = -1, labels = txpIDs(f.results), xlab = '', sub = '')

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## -----------------------------------------------------------------------------
library(toxpiR)

# Load example model from "Import ToxPi GUI Files" vignette
data_format_C <- tempfile()
download.file(
  url = "https://raw.githubusercontent.com/ToxPi/ToxPi-example-files/main/format_C.csv",
  destfile = data_format_C,
  quiet = TRUE
)
gui1 <- txpImportGui(data_format_C)

## -----------------------------------------------------------------------------
# Export back into GUI format
data_exported <- tempfile()
txpExportGui(
  fileName = data_exported,
  input = gui1$input,
  model = gui1$model,
  id.var = 'Name',
  fills = gui1$fills
)

## ----echo=FALSE---------------------------------------------------------------
knitr::kable(read.csv(data_format_C, header = FALSE, stringsAsFactors = FALSE))

## ----echo=FALSE---------------------------------------------------------------
df <- read.csv(data_exported, header = FALSE, stringsAsFactors = FALSE)
df[6:nrow(df), 2:ncol(df)] <- format(as.numeric(as.matrix(df[6:nrow(df), 2:ncol(df)])), digits = 4)
knitr::kable(df)

## -----------------------------------------------------------------------------
gui2 <- txpImportGui(data_exported)

res1 <- txpCalculateScores(gui1$model, gui1$input)
res2 <- txpCalculateScores(gui2$model, gui2$input)

all.equal(
  txpScores(res1),
  txpScores(res2)
)

all.equal(
  txpSliceScores(res1),
  txpSliceScores(res2)
)

##----------------------------------------------------------------------------##
## methods-Slice
##----------------------------------------------------------------------------##

#' @name TxpSlice-class
#' @title ToxPi Slice
#' @description S4 class to store ToxPi slices
#' 
#' @slot txpValueNames `vector(<character>)` specifying the input columns to 
#' include in the slice
#' @slot txpTransFuncs [TxpTransFuncList] with one function per entry in 
#' `txpValueNames` or an object that can be coerced to `TxpTransFuncList`; 
#' when `NULL`, no transformation function applied
#' 
#' 
#' @param txpValueNames Passed to `txpValueNames` slot
#' @param txpTransFuncs Passed to `txpTransFuncs` slot
#' @param x,y `TxpSlice` object
#' @param value Replacement value
#' 
#' @details 
#' If the user supplies `txpTransFuncs` a single function/[TxpTransFunc] object, 
#' the given function will be recycled for each input with a warning. 
#' 
#' @examples 
#' ## Create TxpSlice object 
#' # Without transform functions
#' TxpSlice(txpValueNames = c("sqrData", "expData")) 
#' # With transform functions
#' TxpSlice(txpValueNames = c("sqrData", "expData"),
#'          txpTransFuncs = c(sq = function(x) x^2, log = function(x) log(x)))
#' 
#' # Transformation function recycled with warning when single function given
#' TxpSlice(txpValueNames = c("sqrData", "expData"), 
#'          txpTransFuncs = function(x) x^2) 
#'          
#' 
#' ## Access TxpSlice slots
#' sl <- TxpSlice(txpValueNames = c("sqrData", "expData"),
#'                txpTransFuncs = c(sq = function(x) x^2, 
#'                                  log = function(x) log(x)))
#' txpValueNames(sl)
#' txpTransFuncs(sl)
#' 
#' ## Replacement
#' txpValueNames(sl)[1] <- "hello"
#' sl
#' 
#' txpTransFuncs(sl)[[2]](exp(1))
#' txpTransFuncs(sl)[[2]] <- function(x) sqrt(x)
#' txpTransFuncs(sl)[[2]](exp(1))
#' 
#' # Note that replacing a single list element does NOT update the name
#' sl
#' names(txpTransFuncs(sl))[2] <- "sqrt" 
#' sl
#' 
#' # Replacing the whole list DOES update the names
#' txpTransFuncs(sl) <- list(sqrt = function(x) sqrt(x), 
#'                           log = function(x) log(x))
#' sl
#' 
#' ## length -- returns number of inputs
#' length(TxpSlice(letters))
#' 
#' ## merge
#' s1 <- TxpSlice("hello")
#' s2 <- TxpSlice("data")
#' merge(s1, s2)
#' 
#' # Note, input names still must be unique
#' \dontrun{merge(s1, s1)} ## produces error

NULL

##----------------------------------------------------------------------------##
## constructor

.TxpSlice.handle.funcs <- function(vn, tf) {
  vnl <- length(vn)
  if (is.null(tf)) tf <- vector("list", vnl)
  if (class(tf) %in% c("function", "TxpTransFunc") && vnl > 1) {
    warning("Recycling given 'txpTransFuncs' for each input.")
    tf <- .repFunc(tf, vnl)
  }
  if (!is(tf, "TxpTransFuncList")) {
    tf <- as.TxpTransFuncList(tf)
  }
  tf
}

#' @rdname TxpSlice-class
#' @export 

TxpSlice <- function(txpValueNames, txpTransFuncs = NULL) {
  txpTransFuncs <- .TxpSlice.handle.funcs(txpValueNames, txpTransFuncs)
  new2("TxpSlice", txpValueNames = txpValueNames, txpTransFuncs = txpTransFuncs) 
}

##----------------------------------------------------------------------------##
## accessors

#' @describeIn TxpSlice-class Return `txpValueNames` slot
#' @export

setMethod("txpValueNames", "TxpSlice", function(x) { x@txpValueNames })

#' @rdname TxpSlice-class
#' @export

setReplaceMethod("txpValueNames", "TxpSlice", function(x, value) {
  x@txpValueNames <- value
  validObject(x)
  x
})

#' @describeIn TxpSlice-class Return `txpTransFuncs` slot
#' @export

setMethod("txpTransFuncs", "TxpSlice", function(x) { x@txpTransFuncs })

#' @rdname TxpSlice-class
#' @export

setReplaceMethod("txpTransFuncs", "TxpSlice", function(x, value) {
  value <- .TxpSlice.handle.funcs(txpValueNames(x), value)
  x@txpTransFuncs <- value
  validObject(x)
  x
})

#' @describeIn TxpSlice-class Return number of inputs in slice; shortcut for
#' `length(txpValueNames(x))`
#' @export

setMethod("length", "TxpSlice", function(x) { length(txpValueNames(x)) })

##----------------------------------------------------------------------------##
## validity

.TxpSlice.validity <- function(object) {
  msg <- NULL
  vl <- txpValueNames(object)
  fx <- txpTransFuncs(object)
  if (any(duplicated(vl))) {
    msg <- c(msg, "txpValueNames(<TxpSlice>) must be unique.")
  }
  if (length(vl) != length(fx)) {
    tmp <- paste("length(txpValueNames(<TxpSlice>)) !=",
                 "length(txpTransFuncs(<TxpSlice>))")
    msg <- c(msg, tmp)
  }
  if (is.null(msg)) return(TRUE)
  msg
}

setValidity2("TxpSlice", .TxpSlice.validity)

##----------------------------------------------------------------------------##
## show

.TxpSlice.show <- function(object) {
  n <- length(txpValueNames(object))
  cat(sprintf("TxpSlice with %d input%s.\n", n, ifelse(n > 1, "s", "")))
  .coolcat("  txpValueNames(%d): %s\n", txpValueNames(object))
  fnms <- .listDisplayNames(txpTransFuncs(object))
  .coolcat("  txpTransFuncs(%d): %s\n", fnms)
}

setMethod("show", "TxpSlice", .TxpSlice.show)

##----------------------------------------------------------------------------##
## merge

.TxpSlice.merge <- function(x, y) {
  vns <- c(txpValueNames(x), txpValueNames(y))
  tfs <- c(txpTransFuncs(x), txpTransFuncs(y))
  TxpSlice(txpValueNames = vns, txpTransFuncs = tfs)
}

#' @describeIn TxpSlice-class Merge two `TxpSlice` objects into a single 
#' slice
#' @export

setMethod("merge", c("TxpSlice", "TxpSlice"), .TxpSlice.merge)

##----------------------------------------------------------------------------##

##----------------------------------------------------------------------------##
## TxpModel tests
##----------------------------------------------------------------------------##

##----------------------------------------------------------------------------##
## Initialization

test_that("We can create TxpModel objects", {
  expect_silent({
    slcLst <- list(S1 = TxpSlice("input1"), S2 = TxpSlice("input2"))
    txpSlcLst <- TxpSliceList(S1 = TxpSlice("input1"), S2 = TxpSlice("input2"))
    fxnLst <- list(f1 = function(x) x, f2 = function(x) x^2)
    txpFxnLst <- TxpTransFuncList(f1 = function(x) x, f2 = function(x) x^2)
    wrnLst <- TxpSliceList(S1 = TxpSlice("input1"), S2 = TxpSlice("input1"))
  })
  expect_s4_class(TxpModel(slcLst), "TxpModel")
  expect_s4_class(TxpModel(txpSlcLst), "TxpModel")
  expect_s4_class(TxpModel(txpSlices = slcLst, txpTransFuncs = fxnLst), 
                  "TxpModel")
  expect_s4_class(TxpModel(txpSlices = slcLst, txpTransFuncs = txpFxnLst), 
                  "TxpModel")
  expect_error(TxpModel(txpSlices = txpSlcLst, txpWeights = 1))
  expect_error(TxpModel(txpSlices = txpSlcLst, txpWeights = "1"))
  expect_error(TxpModel(txpSlices = txpSlcLst, txpTransFuncs = txpFxnLst[1]))
  expect_warning(TxpModel(wrnLst))
})

##----------------------------------------------------------------------------##
## Accessors

test_that("TxpModel accessors return expected slots", {
  expect_silent({
    sl <- TxpSliceList(S1 = TxpSlice("input1"), S2 = TxpSlice("input2"))
    md <- TxpModel(sl)
  })
  expect_s4_class(txpSlices(md), "TxpSliceList")
  expect_equal(txpWeights(md), rep(1, 2))
  expect_equal(txpWeights(md, adjust = TRUE), rep(0.5, 2))
  expect_s4_class(txpTransFuncs(md), "TxpTransFuncList")
  expect_equal(txpValueNames(md), list(S1 = "input1", S2 = "input2"))
  expect_equal(txpValueNames(md, simplify = TRUE), 
               c(S1 = "input1", S2 = "input2"))
  expect_named(md, c("S1", "S2"))
  expect_length(md, 2)
})

##----------------------------------------------------------------------------##
## Replace

test_that("We can replace TxpModel slots", {
  expect_silent({
    sl1 <- TxpSliceList(S1 = TxpSlice("input1"), S2 = TxpSlice("input2"))
    sl2 <- TxpSliceList(S1 = TxpSlice("input1"), S3 = TxpSlice("input3"))
    md <- TxpModel(sl1)
    fl <- TxpTransFuncList(f1 = function(x) x, f2 = function(x) sqrt(x))
  })
  expect_s4_class(txpSlices(md) <- sl2, "TxpSliceList")
  expect_named(txpSlices(md), c("S1", "S3"))
  expect_error(txpSlices(md) <- c("A", "B"))
  expect_error(txpSlices(md) <- c(sl1, sl2[2]))
  expect_silent(txpWeights(md) <- 1:2)
  expect_equal(txpWeights(md), 1:2)
  expect_silent(txpTransFuncs(md) <- fl)
  expect_named(txpTransFuncs(md), c("f1", "f2"))
  expect_silent(txpTransFuncs(md) <- as.list(fl)[2:1])
  expect_named(txpTransFuncs(md), c("f2", "f1"))
  expect_silent(txpTransFuncs(md) <- NULL)
  expect_equal(txpTransFuncs(md), TxpTransFuncList(NULL, NULL))
  md <- TxpModel(c(sl1, sl2[2]))
  names(md) <- c("A", "B", "C")
  expect_named(md, c("A", "B", "C"))
  expect_error(names(md) <- "hello")
  names(md)[2] <- "hello"
  expect_named(md, c("A", "hello", "C"))
  names(md)[2:3] <- c("B", "hello")
  expect_named(md, c("A", "B", "hello"))
})


##----------------------------------------------------------------------------##
## Show

test_that("TxpModel show method displays correct information", {
  mdl <- TxpModel(txpSlices = TxpSliceList(S1 = TxpSlice("inpt1"), 
                                           S2 = TxpSlice("input2")),
                  txpWeights = 1:2,
                  txpTransFuncs = list(f1 = function(x) x, NULL))
  expect_output(print(mdl), "txpSlices\\(2\\)")
  expect_output(print(mdl), "S1 S2")
  expect_output(print(mdl), "txpWeights\\(2\\)")
  expect_output(print(mdl), "1 2")
  expect_output(print(mdl), "txpTransFuncs\\(2\\)")
  expect_output(print(mdl), "f1 NULL")
})

##----------------------------------------------------------------------------##
## Merge

test_that("We can merge two TxpModel objects", {
  expect_silent({
    m1 <- TxpModel(txpSlices = c(S1 = TxpSlice("inpt1"), 
                                 S2 = TxpSlice("inpt2")),
                   txpWeights = 1:2,
                   txpTransFuncs = list(NULL, linear = function(x) x))
    m2 <- TxpModel(txpSlices = c(S3 = TxpSlice("inpt3"), 
                                 S4 = TxpSlice("inpt4")),
                   txpWeights = 2:1,
                   txpTransFuncs = list(linear = function(x) x, 
                                        sqrt = function(x) sqrt(x)))
    m3 <- TxpModel(c(S1 = TxpSlice("inpt4")))
    m4 <- TxpModel(c(S4 = TxpSlice("inpt1")), txpWeights = 3)
  })
  expect_s4_class(mrg1 <- merge(m1, m2), "TxpModel")
  expect_length(mrg1, 4)
  expect_named(mrg1, c("S1", "S2", "S3", "S4"))
  expect_equal(names(txpTransFuncs(mrg1)), c("", "linear", "linear", "sqrt"))
  expect_equal(txpTransFuncs(mrg1)[[4]](100), 10)
  expect_equal(txpTransFuncs(mrg1)[[2]](100), 100)
  expect_error(txpTransFuncs(mrg1)[[1]](100))
  expect_error(merge(m1, m3))
  expect_warning(merge(m1, m4))
})

##----------------------------------------------------------------------------##
## TxpSlice tests
##----------------------------------------------------------------------------##

##----------------------------------------------------------------------------##
## Initialization

test_that("We can create TxpSlice objects", {
  vnames <- c("input1", "input2", "input3")
  tfuncs <- TxpTransFuncList(f1 = TxpTransFunc(), f2 = NULL, f3 = NULL)
  expect_s4_class(TxpSlice(txpValueNames = vnames), "TxpSlice")
  expect_s4_class(TxpSlice(txpValueNames = vnames, txpTransFuncs = tfuncs), 
                  "TxpSlice")
  expect_warning(recycle <- TxpSlice(c("a", "b", "c"), function(x) x))
  expect_s4_class(recycle, "TxpSlice")
  expect_length(txpTransFuncs(recycle), 3)
  expect_error(TxpSlice())
  expect_error(TxpSlice(NA))
  expect_error(TxpSlice(NULL))
  expect_error(TxpSlice(c("x", "x")))
  expect_error(TxpSlice(vnames, tfuncs[1:2]))
})

##----------------------------------------------------------------------------##
## Accessors

test_that("TxpSlice accessors return expected slots", {
  sl <- TxpSlice(txpValueNames = c("input1", "input2", "input3"),
                 txpTransFuncs = TxpTransFuncList(f1 = TxpTransFunc(), 
                                                  f2 = NULL, 
                                                  f3 = NULL))
  expect_s4_class(txpTransFuncs(sl), "TxpTransFuncList")
  expect_equal(txpValueNames(sl), c("input1", "input2", "input3"))
})

##----------------------------------------------------------------------------##
## Replace

test_that("We can replace TxpSlice slots", {
  sl <- TxpSlice("input1")
  expect_s4_class({txpValueNames(sl) <- "input2"; sl}, "TxpSlice")
  expect_equal(txpValueNames(sl), "input2")
  expect_error(txpValueNames(sl) <- 3)
  expect_error(txpValueNames(sl) <- c("a", "b"))
  expect_s4_class({txpTransFuncs(sl) <- function(x) x; sl}, "TxpSlice")
  expect_s4_class({names(txpTransFuncs(sl)) <- "linear"; sl}, "TxpSlice")
  expect_equal(txpTransFuncs(sl)[[1]](10), 10)
  expect_named(txpTransFuncs(sl), "linear")
})


##----------------------------------------------------------------------------##
## Show

test_that("TxpSlice shows correct information", {
  sl <- TxpSlice(c("input1", "input2"), list(f1 = function(x) x, NULL))
  expect_output(print(sl), "txpValueNames\\(2\\)")
  expect_output(print(sl), "input1 input2")
  expect_output(print(sl), "txpTransFuncs\\(2\\)")
  expect_output(print(sl), "f1 NULL")
})

##----------------------------------------------------------------------------##
## Length

test_that("TxpSlice length returns correct length", {
  expect_equal(length(TxpSlice(letters)), 26)
  expect_equal(length(TxpSlice(letters[1:5])), 5)
})

##----------------------------------------------------------------------------##
## Merge

test_that("We can merge two TxpSlice objects", {
  s1 <- TxpSlice(c("input1", "input2"), list(NULL, linear = function(x) x))
  s2 <- TxpSlice(c("input3", "input4"), list(NULL, linear = function(x) x))
  expect_s4_class(smrg <- merge(s1, s2), "TxpSlice")
  expect_equal(txpValueNames(smrg), c("input1", "input2", "input3", "input4"))
  expect_named(txpTransFuncs(smrg), c("", "linear", "", "linear"))
})

##----------------------------------------------------------------------------##
## txpExportGui tests
##----------------------------------------------------------------------------##

test_that("We reproduce GUI results", {
  
  # Get expected results from GUI output
  resExpected <- read.csv(file.path("guiFiles", "gui_test_results.csv"), check.names = FALSE)
  resExpected <- resExpected[order(resExpected$Name), -c(2:5)]
  names(resExpected) <- c('score', sapply(strsplit(names(resExpected)[-1], '!'), '[', 1))

  ##------------------------------##
  # Using txpImportGui
  ##------------------------------##
  
  # Load data and model
  expect_warning({
    gui1 <- txpImportGui(file.path("guiFiles", "gui_test_data.csv"))
  })
  
  # Compute scores
  expect_silent({
    res1 <- txpCalculateScores(gui1$model, gui1$input)
  })
  
  # Compare ToxPi Scores
  expect_equal(
    txpScores(res1),
    resExpected$score,
    ignore_attr = TRUE
  )
  
  # Compare Slice Scores
  expect_equal(
    as.data.frame(txpSliceScores(res1, adjusted = FALSE)),
    resExpected[,-1],
    ignore_attr = TRUE
  )
  
  ##------------------------------##
  # Using output from txpExportGui
  ##------------------------------##
  
  # Export model, suppress expected warnings about containing negative values
  data_exported <- tempfile()
  expect_silent({
    suppressWarnings(
      txpExportGui(
        fileName = data_exported,
        input = gui1$input,
        model = gui1$model,
        id.var = 'Name',
        fills = gui1$fills
      )
    )
  })
  
  # Load data and model
  expect_silent({
    gui2 <- txpImportGui(data_exported)
  })
  
  # Compute scores
  expect_silent({
    res2 <- txpCalculateScores(gui2$model, gui2$input)
  })
  
  # Compare ToxPi Scores
  expect_equal(
    txpScores(res2),
    resExpected$score,
    ignore_attr = TRUE
  )
  
  # Compare Slice Scores
  expect_equal(
    as.data.frame(txpSliceScores(res2, adjusted = FALSE)),
    resExpected[,-1],
    ignore_attr = TRUE
  )
  
  ##------------------------------##
  # Manually created model
  ##------------------------------##
  
  # Create model
  input <- read.csv(file.path("guiFiles", "gui_test_data.csv"), skip = 40, check.names = FALSE, stringsAsFactors = FALSE)

  slices <- TxpSliceList()
  nFn <- 10
  for ( i in 1:nFn ) {
    fn <- switch(
      i,
      function(x) x,                                        # 1: linear(x)
      function(x) as.integer(x != 0),                       # 2: hit count
      function(x) ifelse(x <= 0, NA, -log10(x)),            # 3: -log10(x)
      function(x) ifelse(x <= 0, NA, -log10(x) + log10(max(x, na.rm = TRUE))), # 4: -log10(x)+log10(max(x))
      function(x) ifelse(x <= 0, NA, -log10(x) + 6),        # 5: -log10(x)+6
      function(x) ifelse(x <= 0, NA, -log(x)),              # 6: -ln(x)
      function(x) ifelse(x <= 0, NA, log10(x)),             # 7: log10(x)
      function(x) sqrt(x),                                  # 8: sqrt(x)
      function(x) (x - mean(x, na.rm = TRUE))/sd(x, na.rm = TRUE), # 9:  zscore(x)
      function(x) (x - min(x, na.rm = TRUE))/diff(range(x, na.rm = TRUE)), # 10: uniform(x)
      function(x) x                                         # default
    )
    addSlices <- TxpSliceList(
      b = TxpSlice(c("y1a", "y1b"), TxpTransFuncList(fn, fn)),
      a = TxpSlice("y1a", TxpTransFuncList(fn)),
      d = TxpSlice(c("y2a", "y2b"), TxpTransFuncList(fn, fn)),
      c = TxpSlice("y2a", TxpTransFuncList(fn))
    )
    names(addSlices) <- paste0('Slice', i, c('_1ab', '_1a', '_2ab', '_2a'))
    slices <- c(slices, addSlices)
  }
  
  expect_warning({
    model <- TxpModel(
      txpSlices = slices,
      txpWeights = rep(c(2,1,2,1), nFn)
    )
  })
  
  # Compute scores
  expect_silent({
    res3 <- txpCalculateScores(model, input, negative.value.handling = 'missing')
  })
  
  # Compare ToxPi Scores
  expect_equal(
    txpScores(res3),
    resExpected$score,
    ignore_attr = TRUE
  )
  
  # Compare Slice Scores
  expect_equal(
    as.data.frame(txpSliceScores(res3, adjusted = FALSE)),
    resExpected[,-1],
    ignore_attr = TRUE
  )
  
})

##----------------------------------------------------------------------------##
## methods-txpSliceList
##----------------------------------------------------------------------------##

#' @name TxpSliceList-class
#' @title List of TxpSlice objects
#' @description Extension of [S4Vectors::SimpleList] that requires 
#' uniquely-named elements and holds only [TxpSlice] objects.
#' 
#' @param ... [TxpSlice] object to create `TxpSliceList` object; MUST give 
#' unique names to each slice
#' @param x `TxpSliceList` object
#' @param simplify Scalar logical, when `TRUE` the returned `list` is simplified
#' to a `vector`/[TxpTransFuncList] object
#' 
#' @details 
#' Note, there is no coercion for [TxpSlice] to `TxpSliceList` because unique
#' names are required.
#' 
#' @examples
#' ## Create TxpSlice objects
#' s1 <- TxpSlice("input1", list(linear = function(x) x))
#' s2 <- TxpSlice(c("input2", "input3"), 
#'                list(log = function(x) log(x), sqrt = function(x) sqrt(x)))
#' 
#' ## Create TxpSliceList
#' sl <- TxpSliceList(s1 = s1, s2 = s2)
#' 
#' ## Accessors
#' txpValueNames(sl)
#' txpValueNames(sl, simplify = TRUE)
#' 
#' txpTransFuncs(sl)
#' txpTransFuncs(sl, simplify = TRUE)
#' 
#' ## Coercion
#' as(list(s1 = TxpSlice("hello"), s2 = TxpSlice("user")), "TxpSliceList")
#' as.TxpSliceList(c(s1 = TxpSlice("hello"), s2 = TxpSlice("user")))
#' 
#' ## Concatenation
#' c(sl, TxpSliceList(s3 = TxpSlice("input4")))
#' 
#' ## Reduce TxpSliceList to single slice
#' Reduce(merge, sl)

NULL

##----------------------------------------------------------------------------##
## constructor

#' @rdname TxpSliceList-class
#' @export 

TxpSliceList <- function(...) {
  listData <- list(...)
  new2("TxpSliceList", listData)
}

##----------------------------------------------------------------------------##
## validity

.TxpSliceList.validity <- function(object) {
  msg <- NULL
  valid <- vapply(object@listData, is, logical(1), "TxpSlice")
  if (any(!valid)) {
    msg <- c(msg, "All TxpSlice objects must be of class 'TxpSlice.'")
  }
  if (is.null(msg)) return(TRUE)
  msg
}

setValidity2("TxpSliceList", .TxpSliceList.validity)

##----------------------------------------------------------------------------##
## accessors

#' @describeIn TxpSliceList-class Return `list` of `txpValueNames` slots for the 
#' contained [TxpSlice] objects, or `vector` when `simplify = TRUE`
#' @importFrom rlang is_scalar_logical
#' @export

setMethod("txpValueNames", "TxpSliceList", function(x, simplify = FALSE) {
  stopifnot(is_scalar_logical(simplify))
  nms <- lapply(x, txpValueNames)
  if (simplify) nms <- unlist(nms)
  nms
})

#' @describeIn TxpSliceList-class Return `list` of `txpTransFuncs` slots for the 
#' contained [TxpSlice] objects, or [TxpTransFuncList] when `simplify = TRUE`
#' @importFrom rlang is_scalar_logical
#' @export

setMethod("txpTransFuncs", "TxpSliceList", function(x, simplify = FALSE) { 
  stopifnot(is_scalar_logical(simplify))
  fxs <- lapply(x, txpTransFuncs)
  if (simplify) fxs <- Reduce(c, fxs)
  fxs
})

##----------------------------------------------------------------------------##
## duplicated

#' @describeIn TxpSliceList-class Returns logical vector of `length(x)`, where 
#' `TRUE` indicates a duplicate slice in the list; see [base::duplicated]
#' @export

setMethod("duplicated", "TxpSliceList", function(x) .dupList(x))

##----------------------------------------------------------------------------##
## coercion 

.TxpSliceList.from.list <- function(from) {
  do.call("TxpSliceList", from)
}

setAs("list", "TxpSliceList", .TxpSliceList.from.list)

#' @rdname TxpSliceList-class
#' @export

as.TxpSliceList <- function(x) as(x, "TxpSliceList")

##----------------------------------------------------------------------------##

##----------------------------------------------------------------------------##
## TxpSliceList tests
##----------------------------------------------------------------------------##

##----------------------------------------------------------------------------##
## Initialization

test_that("We can create TxpSliceList objects", {
  expect_s4_class(TxpSliceList(), "TxpSliceList")
  expect_s4_class(TxpSliceList(S1 = TxpSlice("inpt1"), S2 = TxpSlice("inpt2")),
                  "TxpSliceList")
  expect_error(TxpSliceList(TxpSlice("inpt1")))
  expect_error(TxpSliceList(S1 = TxpSlice("inpt1"), S1 = TxpSlice("inpt2")))
  expect_error(TxpSliceList(NULL))
  expect_error(TxpSliceList("a"))
})

test_that("We can coerce list to TxpSliceList", {
  l <- list(S1 = TxpSlice("inpt1"), S2 = TxpSlice("inpt2"))
  expect_s4_class(as.TxpSliceList(l), "TxpSliceList")
})

##----------------------------------------------------------------------------##
## Accessors

test_that("We can access TxpSlice slots from TxpSliceList", {
  sl <- TxpSliceList(S1 = TxpSlice("inpt1"), S2 = TxpSlice(c("inpt2", "inpt3")))
  expect_type(txpValueNames(sl), "list")
  expect_length(txpValueNames(sl), 2)
  expect_type(txpValueNames(sl, simplify = TRUE), "character")
  expect_length(txpValueNames(sl, simplify = TRUE), 3)
  expect_type(txpTransFuncs(sl), "list")
  expect_length(txpTransFuncs(sl), 2)
  expect_s4_class(txpTransFuncs(sl, simplify = TRUE), "TxpTransFuncList")
  expect_length(txpTransFuncs(sl, simplify = TRUE), 3)
})

##----------------------------------------------------------------------------##
## Duplicated

test_that("We can detect duplicate TxpSlice objects in TxpSliceList", {
  s1 <- TxpSlice("inpt1")
  s2 <- TxpSlice("inpt1")
  s3 <- TxpSlice("inpt1", txpTransFuncs = function(x) x^2)
  expect_true(any(duplicated(TxpSliceList(s1 = s2, s2 = s2))))
  expect_false(any(duplicated(TxpSliceList(s1 = s1, s3 = s3))))
})

##----------------------------------------------------------------------------##
## Replacement

test_that("We can replace TxpSliceList objects", {
  expect_silent({
    s <- TxpSlice("inpt1")
    l <- TxpSliceList(s1 = s, s2 = s, s3 = s)
  })
  expect_s4_class({l[[1]] <- TxpSlice("hello"); l}, "TxpSliceList")
  expect_equal(txpValueNames(l[[1]]), "hello")
  expect_error(l[[2]] <- "a")
  expect_error(l[2] <- list(NULL))
  expect_error(l[2:3] <- list(TxpSlice("inpt2"), TxpSlice("inpt3")))
  expect_s4_class({
    l[2:3] <- list(s4 = TxpSlice("inpt2"), s5 = TxpSlice("inpt3"))
    l
  }, "TxpSliceList")
  expect_named(l, c("s1", "s2", "s3"))
  expect_equal(txpValueNames(l, simplify = TRUE), 
               c(s1 = "hello", s2 = "inpt2", s3 = "inpt3"))
  expect_named({names(l) <- c("a", "b", "c"); l}, c("a", "b", "c"))
  expect_named({names(l)[1] <- "hello"; l}, c("hello", "b", "c"))
  expect_named({names(l)[1:2] <- c("a", "hello"); l}, c("a", "hello", "c"))
  expect_length({l[2] <- NULL; l}, 2)
  expect_equal(txpValueNames(l, simplify = TRUE), c(a = "hello", c = "inpt3"))
})

##----------------------------------------------------------------------------##
## txpImportGui
##----------------------------------------------------------------------------##

#' @name txpImportGui
#' @title Import data file generated by ToxPi GUI
#' @description Import data file generated by ToxPi GUI
#' 
#' @param guiDataFile Character scalar, the path to a 'data' export from the
#' ToxPi GUI
#' 
#' @details 
#' This function takes the '_data.csv' files generated by the GUI.
#' See \url{https://toxpi.github.io/} for more information.
#' 
#' Because of the way toxpiR implements transformation functions, there is not
#' a way currently to use the GUI 'hitcount' function. 
#' 
#' @return `list` with `$model` containing [TxpModel] object; `$input` 
#' containing `data.frame` with input data; `$fills` containing a vector 
#' of fill colors.
#' 
#' @importFrom utils type.convert read.csv
#' @export

txpImportGui <- function(guiDataFile) {
  
  stopifnot(is_scalar_character(guiDataFile))
  stopifnot(file.exists(guiDataFile))
  
  gui <- read.csv(guiDataFile, stringsAsFactors = FALSE, header = FALSE)
  res <- try(.fromGui(gui), silent = TRUE)
  if (is(res, "try-error")) stop("The given 'guiDataFile' could not be parsed.")
  if (is(res, "simpleCondition")) stop(conditionMessage(res))
  res
  
}

#' @importFrom tidyr separate
#' @importFrom rlang is_scalar_character

.fromGui <- function(gui) {
  
  sliceInfoInd <- grepl('^#', gui[ , 1])
  infoNms <- c("name", "wt", "col", "scale")
  sliceInfo <- tidyr::separate(data = gui[sliceInfoInd, ], 
                               col = "V1", 
                               into = infoNms, 
                               sep = "!", 
                               convert = FALSE)
  sliceInfo$name <- sub('^#\\s+', '', sliceInfo$name)
  sliceInfo$col  <- sub('^0x', '#', sliceInfo$col)
  sliceInfo$wt   <- sapply(strsplit(sliceInfo$wt, split = '/'), function(x) {
    as.numeric(x[1]) / as.numeric(ifelse(length(x) == 2, x[2], 1))
  })
  sliceInfo <- sliceInfo[ , infoNms]
  validFuncs <- sliceInfo$scale %in% names(TXP_GUI_FUNCS)
  if (!all(validFuncs)) {
    f <- paste(sliceInfo$scale[!validFuncs], collapse = ", ")
    msg <- sprintf(paste("Given scaling function(s), '%s', not compatible with",
                         "toxpiR. See ?txpImportGui for more information."),
                   f)
    return(simpleCondition(msg))
  }
  sliceInfo$ind <- apply(gui[sliceInfoInd, ], 1, function(x) which(x == "x"))
  
  inputStart <- which(grepl('^row$', gui[ , 1], ignore.case = TRUE))
  if (length(inputStart) != 1) {
    inputStart <- which(gui[ , 1] == '') # Format D
  }
  inputNms <- as.character(gui[inputStart, ])
  input <- gui[(inputStart + 1):nrow(gui), ]
  input[] <- lapply(input, type.convert, as.is = TRUE)
  names(input) <- inputNms
  input[input < 0] <- NA
  row.names(input) <- 1:nrow(input)
  
  mkSl <- function(i) {
    s <- TxpSlice(txpValueNames = inputNms[sliceInfo[i, "ind"][[1]]])
    sl <- length(s)
    tnm <- sliceInfo[i, "scale"]
    tfs <- .repFunc(TXP_GUI_FUNCS[[tnm]], sl)
    names(tfs) <- rep(tnm, sl)
    txpTransFuncs(s) <- tfs
    s
  }
  
  sliceLst <- lapply(seq(nrow(sliceInfo)), mkSl)
  names(sliceLst) <- sliceInfo$name
  sliceLst <- as.TxpSliceList(sliceLst) 
  
  model <- TxpModel(txpSlices = sliceLst, txpWeights = sliceInfo[ , "wt"])
  
  vnms <- unique(txpValueNames(txpSlices(model), simplify = TRUE))
  numCols <- sapply(input[vnms], is.numeric)
  if (!all(numCols)) {
    cols <- paste(vnms[numCols], collapse = ", ")
    msg <- sprintf(paste("Following input column(s), '%s', could not be",
                         "coerced to numeric."),
                   cols)
    return(simpleCondition(msg))
  }
  
  list(model = model, input = input, fills = sliceInfo$col)
  
}

#' @importFrom stats sd

TXP_GUI_FUNCS <- list(
  'linear(x)' = function(x) { x },
  'hit count' = function(x) { as.integer(x != 0) },
  '-log10(x)' = function(x) { ifelse(x <= 0, NA, -log10(x)) },
  '-log10(x)+log10(max(x))' = function(x) {
    ifelse(x <= 0, NA, -log10(x) + log10(max(x, na.rm = TRUE)))
  },
  '-log10(x)+6' = function(x) { ifelse(x <= 0, NA, -log10(x) + 6) },
  '-ln(x)' = function(x) { ifelse(x <= 0, NA, -log(x)) },
  'log10(x)' = function(x) { ifelse(x <= 0, NA, log10(x)) },
  'sqrt(x)' = function(x) { sqrt(x) },
  'zscore(x)' = function(x) { (x - mean(x, na.rm = TRUE))/sd(x, na.rm = TRUE) },
  'uniform(x)' = function(x) {
    xmn <- min(x, na.rm = TRUE)
    xmx <- max(x, na.rm = TRUE)
    (x - xmn)/(xmx - xmn)
  }
)

##----------------------------------------------------------------------------##

##----------------------------------------------------------------------------##
## txpImportGui tests
##----------------------------------------------------------------------------##

test_that("We can import GUI outputs", {
  expect_warning({
    l <- txpImportGui(file.path("guiFiles", "gui_output_data.csv"))
  })
  expect_type(l, "list")
  expect_s4_class(l$model, "TxpModel")
  expect_named(l$model, c("Slice1", "Slice2", "Slice3", "Slice4"))
  expect_s3_class(l$input, "data.frame")
  expect_named(l$input, 
               c("row", "SID", "CASRN", "Name", "metric1", 
                 "metric2", "metric4", "metric3"))
  expect_error(txpImportGui(file.path("guiFiles", "gui_bad_input.csv")))
  expect_error(txpImportGui(file.path("guiFiles", 
                                      "gui_output_missingFuncs.csv")), 
               "hitcall\\(x\\), function\\(x\\), f\\(x\\), hello\\(x\\)")
  expect_warning(expect_error({
    txpImportGui(file.path("guiFiles", "gui_output_nonNumeric.csv"))
  }, "metric1, metric3"))
  # expect_silent({
  #   dl <- txpImportGui(file.path("guiFiles", "gui_distributions.csv"))
  #   expect_warning(txpCalculateScores(dl$model, dl$input), "NaNs produced")
  # })
})

##----------------------------------------------------------------------------##
## txpExportGui tests
##----------------------------------------------------------------------------##

test_that("We can export GUI-ready files", {
  
  # Note1
  # Modifying transformation functions in the model causes the "columns are duplicated"
  # message to be repeated, I'm not sure if that is the desired response
  
  # Output file for txpExportGui()
  data_exported <- tempfile()
  # Load data and model
  expect_warning({
    gui <- txpImportGui(file.path("guiFiles", "gui_output_data.csv"))
  })
  # No warnings/errors expected for original imported file
  expect_silent({
    txpExportGui(
      fileName = data_exported,
      input = gui$input,
      model = gui$model,
      id.var = 'Name',
      fills = gui$fills
    )
  })
  # Non-integer weights
  test_model <- gui$model
  expect_warning({
    # See Note1 above
    txpWeights(test_model)[1] <- 0.5
  })
  expect_error({
    txpExportGui(
      fileName = data_exported,
      input = gui$input,
      model = test_model,
      id.var = 'Name',
      fills = gui$fills
    )
  })
  # Slice-level transformation function
  test_model <- gui$model
  expect_warning({
    # See Note1 above
    txpTransFuncs(test_model)[[1]] <- function(x) log10(x)
  })
  expect_warning({
    txpExportGui(
      fileName = data_exported,
      input = gui$input,
      model = test_model,
      id.var = 'Name',
      fills = gui$fills
    )
  })
  # Input-level transformation function that creates negative values
  test_model <- gui$model
  expect_warning({
    # See Note1 above
    txpTransFuncs(txpSlices(test_model)[[1]])[[1]] <- function(x) -x
  })
  expect_warning({
    txpExportGui(
      fileName = data_exported,
      input = gui$input,
      model = test_model,
      id.var = 'Name',
      fills = gui$fills
    )
  })
  # Negative input values, expect 3 warnings for the 3 affected slices
  test_input <- gui$input
  test_input[, 6] <- -test_input[, 6]
  expect_warning(expect_warning(expect_warning({
    txpExportGui(
      fileName = data_exported,
      input = test_input,
      model = gui$model,
      id.var = 'Name',
      fills = gui$fills
    )
  })))
})

##----------------------------------------------------------------------------##
## methods-txpModelList
##----------------------------------------------------------------------------##

#' @name TxpModelList-class
#' @title List of TxpModel objects
#' @description Extension of [S4Vectors::SimpleList] that holds only [TxpModel] 
#' objects.
#' 
#' @param ... [TxpModel] object to create `TxpModelList` object
#' @param x `TxpModelList` object
#' 
#' @examples
#' ## Create some TxpModel objects; see ?TxpModel for more details
#' s1 <- list(S1 = TxpSlice("inpt1"), S2 = TxpSlice("inpt2"))
#' tf <- list(NULL, sqrt = function(x) sqrt(x))
#' m1 <- TxpModel(txpSlices = s1, txpWeights = 2:1, txpTransFuncs = tf)
#' m2 <- m1
#' txpSlices(m2) <- list(S3 = TxpSlice("inpt3"), S4 = TxpSlice("inpt4"))
#' m3 <- merge(m1, m2)
#' 
#' ## Build a TxpModelList object
#' TxpModelList(m1 = m1, m2 = m2, m3 = m3)
#' 
#' ## Note: names are printed as '' when all are NULL
#' TxpModelList(m1, m2, m3)
#' names(TxpModelList(m1, m2, m3))
#' 
#' ## Test for duplicates
#' duplicated(TxpModelList(m1 = m1, m2 = m2, m3 = m3))
#' duplicated(TxpModelList(m1 = m1, m2 = m1, m3 = m3))
#' 
#' ## Coerce lists/TxpModel objects to TxpModelList
#' as(list(m1 = m1, m2 = m2, m3 = m3), "TxpModelList")
#' as.TxpModelList(list(m1 = m1, m2 = m2, m3 = m3))
#' 
#' as(m1, "TxpModelList")
#' as.TxpModelList(m1)

NULL

##----------------------------------------------------------------------------##
## constructor

#' @rdname TxpModelList-class
#' @export 

TxpModelList <- function(...) {
  listData <- list(...)
  new2("TxpModelList", listData)
}

##----------------------------------------------------------------------------##
## validity

.TxpModelList.validity <- function(object) {
  msg <- NULL
  valid <- vapply(object@listData, is, logical(1), "TxpModel")
  if (any(!valid)) {
    msg <- c(msg, "All TxpModel objects must be of class 'TxpModel.'")
  }
  if (is.null(msg)) return(TRUE)
  msg
}

setValidity2("TxpModelList", .TxpModelList.validity)

##----------------------------------------------------------------------------##
## txpCalculateScores

.TxpModelList.calc <- function(model, input, 
                               id.var = NULL,
                               rank.ties.method = c("average", "first", "last", 
                                                    "random", "max", "min"),
                               negative.value.handling = c("keep", "missing")) {
  if (is.list(model)) {
    model <- try(as.TxpModelList(model), silent = TRUE)
    if (is(model, "try-error")) {
      stop("Given list could not be coerced to TxpModelList")
    }
  }
  resLst <- lapply(model, .calculateScores,
                   input = input, 
                   id.var = id.var, 
                   rank.ties.method = rank.ties.method,
                   negative.value.handling = negative.value.handling)
  as.TxpResultList(resLst)
}

#' @rdname txpCalculateScores
#' @export

setMethod("txpCalculateScores", 
          c("TxpModelList", "data.frame"), 
          .TxpModelList.calc)

#' @rdname txpCalculateScores
#' @export

setMethod("txpCalculateScores", c("list", "data.frame"), .TxpModelList.calc)

##----------------------------------------------------------------------------##
## show

.TxpModelList.show <- function(object) {
  lnms <- .listDisplayNames(object)
  .coolcat("  TxpModelList of length %d: %s\n", lnms)
}

setMethod("show", "TxpModelList", .TxpModelList.show)

##----------------------------------------------------------------------------##
## duplicated

#' @describeIn TxpModelList-class Returns logical vector of `length(x)`, where 
#' `TRUE` indicates a duplicate model in the list; see [base::duplicated]
#' @export

setMethod("duplicated", "TxpModelList", function(x) .dupList(x))

##----------------------------------------------------------------------------##
## coercion 

.TxpModelList.from.list <- function(from) {
  do.call("TxpModelList", from)
}

setAs("list", "TxpModelList", .TxpModelList.from.list)

.TxpModelList.from.TxpModel <- function(from) {
  TxpModelList(from)
}

setAs("TxpModel", "TxpModelList", .TxpModelList.from.TxpModel)

#' @describeIn TxpModelList-class Coerce list or [TxpModel] objects to 
#' TxpModelList
#' @export

as.TxpModelList <- function(x) as(x, "TxpModelList")

##----------------------------------------------------------------------------##

##----------------------------------------------------------------------------##
## TxpModelList tests
##----------------------------------------------------------------------------##

##----------------------------------------------------------------------------##
## Initialization

test_that("We can create TxpModelList objects", {
  expect_silent(mdl <- TxpModel(TxpSliceList(s1 = TxpSlice("hello"))))
  expect_s4_class(TxpModelList(mdl, mdl), "TxpModelList")
  expect_s4_class(TxpModelList(m1 = mdl, m2 = mdl), "TxpModelList")
  expect_error(TxpModelList(mdl, NULL))
  expect_error(TxpModelList(NULL))
  expect_error(TxpModelList(3))
  expect_length(TxpModelList(mdl, mdl, mdl), 3)
  expect_named(TxpModelList(mdl, m = mdl, mdl), c('', 'm', ''))
})

##----------------------------------------------------------------------------##
## Show

test_that("TxpModelList show method displays correct information", {
  expect_silent({
    mdl <- TxpModel(TxpSliceList(s1 = TxpSlice("hello")))
    l <- TxpModelList(m1 = mdl, mdl, m3 = mdl)
  })
  expect_output(print(l), "TxpModelList of length 3")
  expect_output(print(l), "m1 '' m3")
  expect_silent(names(l) <- NULL)
  expect_output(print(l), "'' '' ''")
})

##----------------------------------------------------------------------------##
## Concatenation

test_that("We can concatenate TxpModelList objects", {
  expect_silent({
    mdl <- TxpModel(TxpSliceList(s1 = TxpSlice("hello")))
    l <- TxpModelList(m1 = mdl, mdl, m3 = mdl)
  })
  expect_s4_class(cl <- c(l, rev(l), l), "TxpModelList")
  expect_length(cl, 9)
  expect_named(cl, c('m1', '', 'm3', 'm3', '', 'm1', 'm1', '', 'm3'))
})

##----------------------------------------------------------------------------##
## Coercion

test_that("We can coerce to TxpModelList objects", {
  expect_silent({
    mdl <- TxpModel(TxpSliceList(s1 = TxpSlice("hello")))
    l <- list(m1 = mdl, mdl, m3 = mdl)
  })
  expect_s4_class(as.TxpModelList(l), "TxpModelList")
  expect_s4_class(as.TxpModelList(l[[1]]), "TxpModelList")
})

##----------------------------------------------------------------------------##
## Duplicated

test_that("We can detect duplicate TxpModel objects in TxpModelList", {
  m1 <- TxpModel(c(S1 = TxpSlice("inpt1")))
  m2 <- TxpModel(c(S1 = TxpSlice("inpt1"), S2 = TxpSlice("inpt2")))
  m3 <- TxpModel(c(S1 = TxpSlice("inpt1")), 2)
  expect_false(any(duplicated(TxpModelList(m1 = m1, m2 = m2, m3 = m3))))
  expect_true(any(duplicated(TxpModelList(m1 = m1, m2 = m1))))
})

##----------------------------------------------------------------------------##
## methods-txpTransFuncList
##----------------------------------------------------------------------------##

#' @name TxpTransFuncList-class
#' @title List of TxpTransFunc objects
#' @description Extension of [S4Vectors::SimpleList] that holds only `NULL` or 
#' [TxpTransFunc] objects.
#' 
#' @param ... [TxpTransFunc] object or function to create `TxpTransFuncList` 
#' object
#' @param x `list`, `function`, or [TxpTransFunc] object to coerce to 
#' `TxpTransFuncList`
#' 
#' @details 
#' When `...` includes function objects, `TxpTransFuncList` will attempt to 
#' coerce them to [TxpTransFunc] and return an error if any of the elements
#' cannot be coerced to [TxpTransFunc].
#' 
#' @examples 
#' ## Create TxpTransFunc objects
#' tf1 <- TxpTransFunc(function(x) x)
#' tf2 <- TxpTransFunc(function(x) sqrt(x))
#' 
#' ## Create TxpTransFuncList 
#' tfl <- TxpTransFuncList(linear = tf1, sqrt = tf2, cube = function(x) x^3)
#' tfl[[3]](3) == 27
#' tfl[["sqrt"]](4) == 2
#' 
#' ## Concatenate
#' c(tfl, tfl)
#' 
#' ## names
#' names(c(tfl, tfl))
#' 
#' # note: names are printed as '' when missing; NULL is printed when list item
#' # is NULL
#' names(TxpTransFuncList(function(x) x, NULL))
#' TxpTransFuncList(function(x) x, NULL)
#' 
#' ## coercion
#' as(function(x) x, "TxpTransFuncList")
#' as.TxpTransFuncList(function(x) x)
#' 
#' as(TxpTransFunc(function(x) x), "TxpTransFuncList")
#' as.TxpTransFuncList(TxpTransFunc(function(x) x))
#' 
#' as(list(function(x) x, sqrt = function(x) sqrt(x)), "TxpTransFuncList")
#' as.TxpTransFuncList(list(function(x) x, sqrt = function(x) sqrt(x)))

NULL

##----------------------------------------------------------------------------##
## constructor

.TxpTransFuncList.toTransFunc <- function(x) {
  if (!is.null(x) && !inherits(x, "TxpTransFunc")) { 
    x <- try(TxpTransFunc(x), silent = TRUE)
  }
  x
}

#' @rdname TxpTransFuncList-class
#' @export 

TxpTransFuncList <- function(...) {
  listData <- lapply(list(...), .TxpTransFuncList.toTransFunc)
  valid <- vapply(listData, is, logical(1), "TxpTransFunc_OR_NULL")
  if (any(!valid)) {
    stop("Some of the given list items could not be coerced to 'TxpTransFunc'.")
  }
  new2("TxpTransFuncList", listData)
}

##----------------------------------------------------------------------------##
## validity

.TxpTransFuncList.validity <- function(object) {
  msg <- NULL
  valid <- vapply(object@listData, is, logical(1), "TxpTransFunc_OR_NULL")
  if (any(!valid)) {
    msg <- c(msg, "All TxpFuncList objects must be of class 'TxpTransFunc.'")
  }
  if (is.null(msg)) return(TRUE)
  msg
}

setValidity2("TxpTransFuncList", .TxpTransFuncList.validity)

##----------------------------------------------------------------------------##
## show

.TxpTransFuncList.show <- function(object) {
  lnms <- .listDisplayNames(object)
  .coolcat("  TxpTransFuncList of length %d: %s\n", lnms)
}

setMethod("show", "TxpTransFuncList", .TxpTransFuncList.show)

##----------------------------------------------------------------------------##
## coercion 

.TxpTransFuncList.from.list <- function(from) {
  do.call("TxpTransFuncList", from)
}

setAs("list", "TxpTransFuncList", .TxpTransFuncList.from.list)

.TxpTransFuncList.from.func <- function(from) {
  TxpTransFuncList(from)
}

setAs("function", "TxpTransFuncList", .TxpTransFuncList.from.func)

#' @rdname TxpTransFuncList-class
#' @export

as.TxpTransFuncList <- function(x) as(x, "TxpTransFuncList")

##----------------------------------------------------------------------------##

##----------------------------------------------------------------------------##
## TxpTransFuncList tests
##----------------------------------------------------------------------------##

##----------------------------------------------------------------------------##
## Initialization

test_that("We can create TxpTransFuncList objects", {
  expect_s4_class(TxpTransFuncList(TxpTransFunc()), "TxpTransFuncList")
  expect_s4_class(TxpTransFuncList(f = TxpTransFunc()), "TxpTransFuncList")
  expect_s4_class(TxpTransFuncList(function(x) x), "TxpTransFuncList")
  expect_s4_class(TxpTransFuncList(NULL), "TxpTransFuncList")
  f <- function(x) x
  expect_s4_class(l <- TxpTransFuncList(NULL, NULL, f1 = f, f, NULL), 
                  "TxpTransFuncList")
  expect_length(l, 5)
  expect_named(l, c('', '', 'f1', '', ''))
  expect_error(TxpTransFuncList(NULL, "a"))
  expect_error(TxpTransFuncList(function(x) "a"))
  expect_silent(l <- as.list(l))
  expect_s4_class(as.TxpTransFuncList(l), "TxpTransFuncList")
  expect_s4_class(as(l, "TxpTransFuncList"), "TxpTransFuncList")
  expect_s4_class(as.TxpTransFuncList(function(x) x), "TxpTransFuncList")
  expect_s4_class(as(function(x) x, "TxpTransFuncList"), "TxpTransFuncList")
})

##----------------------------------------------------------------------------##
## Show

test_that("TxpTransFuncList show method displays correct information", {
  expect_silent(f <- function(x) x)
  expect_silent(l <- TxpTransFuncList(NULL, NULL, f1 = f, f, NULL))
  expect_output(print(l), "TxpTransFuncList of length 5")
  expect_output(print(l), "NULL NULL f1 '' NULL")
  expect_silent(names(l) <- NULL)
  expect_output(print(l), "NULL NULL '' '' NULL")
})

##----------------------------------------------------------------------------##
## Concatenation

test_that("We can concatenate TxpTransFuncList objects", {
  expect_silent({
    f <- TxpTransFunc()
    l <- TxpTransFuncList(f = f, f, NULL)
  })
  expect_s4_class(cl <- c(l, rev(l), l), "TxpTransFuncList")
  expect_length(cl, 9)
  expect_named(cl, c('f', '', '', '', '', 'f', 'f', '', ''))
})

##----------------------------------------------------------------------------##
## Replacement

test_that("We can replace TxpTransFuncList objects", {
  expect_silent({
    f <- TxpTransFunc()
    l <- TxpTransFuncList(f = f, f, NULL)
  })
  expect_s4_class({l[[2]] <- function(x) x^2; l}, "TxpTransFuncList")
  expect_equal(l[[2]](10), 100)
  expect_error(l[[2]] <- "a")
  expect_error(l[[2]] <- function(x) x + "hello")
  expect_s4_class({l[2:3] <- list(function(x) x, function(x) sqrt(x)); l}, 
                  "TxpTransFuncList")
  expect_equal(l[[2]](10), 10)
  expect_equal(l[[3]](100), 10)
  expect_length({l[2] <- list(NULL); l}, 3)
  expect_output(print(l), "f NULL ''")
  expect_length({l[[1]] <- NULL; l}, 2)
  expect_output(print(l), "NULL ''")
})

##----------------------------------------------------------------------------##
## methods-TxpResultParam
##----------------------------------------------------------------------------##

#' @name TxpResultParam-class
#' @aliases TxpResultParam
#' @title ToxPi Result Parameters
#' @description S4 class to store ToxPi result calculation parameters
#' 
#' @slot rank.ties.method Character scalar, method used to calculate score 
#' ranks passed to [base::rank]
#' @slot negative.value.handling Character scalar, how negative values are 
#' handled, see details
#' 
#' @param rank.ties.method Passed to `rank.ties.method` slot
#' @param negative.value.handling Passed to `negative.value.handling` slot
#' 
#' @details 
#' If more than one value is passed to `TxoResultParam` scalar options, e.g. 
#' `rank.ties.method`, only the first value is kept. 
#' 
#' The `rank.ties.method` slot is passed to [base::rank] for calculating the 
#' ranks of observations, with the highest-scoring observation having the rank
#' of 1. 
#' 
#' `negative.value.handling` indicates how to handle negative values in the 
#' inputs. The ToxPi algorithm originally intended to accept non-negative 
#' potency values; the GUI, therefore, treats negative values in the input as 
#' missing. By default, [txpCalculateScores] keeps negative values
#' (`negative.value.handling = "keep"`). To replicate the GUI behavior, users
#' can set `negative.value.handling = "missing"`. 
#' 
#' @seealso [txpCalculateScores], [TxpResult]
#' 

NULL

##----------------------------------------------------------------------------##
## constructor -- NOT exported

TxpResultParam <- function(rank.ties.method, negative.value.handling) {
  new2("TxpResultParam", 
       rank.ties.method = rank.ties.method[1], 
       negative.value.handling = negative.value.handling[1])
}

##----------------------------------------------------------------------------##
## validity

#' @importFrom rlang is_scalar_character

.TxpResultParam.validity <- function(object) {
  msg <- NULL
  rankMthd <- slot(object, "rank.ties.method")
  if (!is_scalar_character(rankMthd)) {
    msg <- c(msg, "rank.ties.method must be scalar character")
  }
  validRnkMthd <- c("average", "first", "last", "random", "max", "min")
  if (is_scalar_character(rankMthd) && !rankMthd %in% validRnkMthd)  {
    msg <- c(msg, "Invalid rank.ties.method; see ?base::rank")
  }
  negHndl <- slot(object, "negative.value.handling")
  if (!is_scalar_character(negHndl)) {
    msg <- c(msg, "negative.value.handling must be scalar character")
  }
  validNegHndl <- c("keep", "missing")
  if (is_scalar_character(negHndl) && !negHndl %in% validNegHndl)  {
    msg <- c(msg, "Invalid negative.value.handling; see ?TxpResultParam")
  }
  if (is.null(msg)) return(TRUE)
  msg
}

setValidity2("TxpResultParam", .TxpResultParam.validity)

##----------------------------------------------------------------------------##
## show

.TxpResultParam.show <- function(object) {
  cat("TxpResultParam:\n")
  sapply(names(getSlots("TxpResultParam")), .catslot, object = object)
}

setMethod("show", "TxpResultParam", .TxpResultParam.show)


##----------------------------------------------------------------------------##

#' @keywords internal
"_PACKAGE"

## usethis namespace: start
## usethis namespace: end
NULL

# TXP_FILLS = c("dodgerblue",
#               "bisque",
#               "darkolivegreen3",
#               "darkorchid3",
#               "mistyrose2",
#               "darkgoldenrod1")
TXP_FILLS = c(
  "#f3622d",
  "#fba71b",
  "#57b757",
  "#41a9c9",
  "#4258c9",
  "#9a42c8",
  "#c84164",
  "#888888"
)

#' @name toxpiR-datasets
#' @title toxpiR data objects
#' @description Objects included in the toxpiR package, loaded with
#' [utils::data]
#' @aliases txp_example_input txp_example_model
#'
#' @usage data(txp_example_input, package = "toxpiR")
#' @usage data(txp_example_model, package = "toxpiR")
#'
#' @section txp_example_input:
#'
#' Small example input data to be used with [txpCalculateScores] in creating
#' [TxpResult] objects. A [base::data.frame] with 10 rows and 9 variables
#' \describe{
#'   \item{name}{Observation names}
#'   \item{metric#}{Input data for ToxPi models}
#' }
#'
#' @source <https://github.com/ToxPi/ToxPi-example-files>
#'
#' @section txp_example_model:
#'
#' Example [TxpModel] object intended for `txp_example_data`; model with 4
#' slices.
#'
#' @examples
#' data(txp_example_input, package = "toxpiR")
#' data(txp_example_model, package = "toxpiR")
#' txp_example_input
#' txp_example_model
#'
#' ## Code to create txp_example_model
#' tf1 <- TxpTransFuncList(linear = function(x) x)
#' sl <- TxpSliceList(s1 = TxpSlice(sprintf("metric%d", 1:2)),
#'                    s2 = TxpSlice("metric3"),
#'                    s3 = TxpSlice(sprintf("metric%d", 4:7),
#'                                  tf1[rep("linear", 4)]),
#'                    s4 = TxpSlice("metric8", tf1))
#' tf2 <- TxpTransFuncList(NULL, linear = function(x) x, NULL, NULL)
#' TxpModel(txpSlices = sl, txpWeights = c(2, 1, 3, 2), txpTransFuncs = tf2)
#'
#' @importFrom utils data

NULL

##----------------------------------------------------------------------------##
## methods-TxpResult
##----------------------------------------------------------------------------##

#' @name TxpResult-class
#' @aliases TxpResult
#' @title ToxPi Result
#' @description S4 class to store ToxPi results
#'
#' @slot txpScores `vector(<numeric>)` of model scores
#' @slot txpSliceScores `matrix(<numeric>)`, sample by slice `matrix` with
#' individual slice scores
#' @slot txpRanks `vector(<numeric>)` with rank of scores
#' @slot txpMissing `vector(<numeric>)` with data missingness
#' @slot txpModel [TxpModel] object
#' @slot txpIDs `vector(<character>)` of observation IDs
#' @slot txpResultParam [TxpResultParam] object
#'
#' @param x [TxpResult] object
#' @param value Replacement value
#' @param adjusted Logical scalar, when `TRUE` the weights are adjusted to sum
#' to 1 or the slice scores are scaled to their respective weight
#' @param level `c('model', 'slices')`; indicates whether to retrieve
#' `txpTransFuncs` slot from the model or underlying slices
#' @param simplify Logical scalar, flatten `txpValueNames` or `txpTransFunc`
#' slots when retrieving slice-level information
#' @param i Subsetting index
#' @param j,drop,optional Not currently implemented
#' @param decreasing,na.last Passed to [base::sort]
#' @param row.names Passed to [base::data.frame]
#' @param id.name,score.name,rank.name Character scalar; when coercing to
#' [base::data.frame], the name for the `txpIDs`, `txpScores`, and `txpRanks`
#' columns, respectively
#' @param ... Passed to [base::data.frame] in `as.data.frame` or [base::sort]
#' in `sort`
#'
#' @seealso [txpCalculateScores], [plot], [TxpResultList]
#'
#' @template roxgn-loadExamples
#' @template roxgn-calcTxpModel
#'
#' @examples
#' ## Accessors
#' txpScores(res)
#'
#' txpSliceScores(res) ## adjusted for weight, by default
#' apply(txpSliceScores(res), 2, max, na.rm = TRUE)
#'
#' txpSliceScores(res, adjusted = FALSE) ## each score should have maximum of 1
#' apply(txpSliceScores(res, adjusted = FALSE), 2, max, na.rm = TRUE)
#'
#' txpRanks(res)
#'
#' txpMissing(res)
#'
#' txpModel(res)
#' identical(txpModel(res), txp_example_model)
#'
#' txpIDs(res)
#' names(res) ## identical to txpIDs(res)
#' identical(txpIDs(res), names(res))
#'
#' # Can access TxpModel slots directly
#' txpWeights(res)
#' txpWeights(res, adjusted = TRUE)
#' txpSlices(res)
#' # When retrieving transform functions, must specify level because both
#' # models and slices have transform functions
#' txpTransFuncs(res, level = "model")
#'
#' # Can access TxpSliceList slots directly
#' txpValueNames(res)
#' txpValueNames(res, simplify = TRUE)
#' txpTransFuncs(res, level = "slices")
#' txpTransFuncs(res, level = "slices", simplify = TRUE)
#'
#' ## Subsetting
#' res[1]
#' res[c("chem01", "chem09")]
#' res[grepl("4|6", txpIDs(res))]
#' \dontrun{
#' res[c(TRUE, FALSE)] ## gets recycled with warning
#' }
#'
#' ## length -- returns number of observations
#' length(res)
#' length(res[1:5])
#'
#' ## sort
#' names(res)
#' names(sort(res))
#'
#' txpScores(res)
#' txpScores(sort(res))
#' txpScores(sort(res, decreasing = FALSE))
#'
#' ## as.data.frame
#' as.data.frame(res)
#' as.data.frame(res, id.name = "nm", score.name = "scr", rank.name = "rnk")

NULL

##----------------------------------------------------------------------------##
## constructor -- NOT exported

TxpResult <- function(txpScores, txpSliceScores, txpRanks, txpMissing,
                      txpModel, txpIDs = NULL, txpResultParam) {
  new2("TxpResult",
       txpScores = txpScores,
       txpSliceScores = txpSliceScores,
       txpRanks = txpRanks,
       txpMissing = txpMissing,
       txpModel = txpModel,
       txpIDs = txpIDs,
       txpResultParam = txpResultParam)
}

##----------------------------------------------------------------------------##
## accessors

#' @describeIn TxpResult-class Return `txpScores` slot
#' @export

setMethod("txpScores", "TxpResult", function(x) { x@txpScores })

#' @describeIn TxpResult-class Return `txpSliceScores` slot; default
#' `adjusted = TRUE`, i.e. return slice scores adjusted for weight
#' @importFrom rlang is_scalar_logical
#' @export

setMethod("txpSliceScores", "TxpResult", function(x, adjusted = TRUE) {
  stopifnot(is_scalar_logical(adjusted))
  scr <- x@txpSliceScores
  if (adjusted) {
    wts <- txpWeights(x, adjusted = TRUE)
    scr <- scr*rep(wts, each = NROW(scr))
  }
  scr
})

#' @describeIn TxpResult-class Return `txpRanks` slot
#' @export

setMethod("txpRanks", "TxpResult", function(x) { x@txpRanks })

#' @describeIn TxpResult-class Return `txpMissing` slot
#' @export

setMethod("txpMissing", "TxpResult", function(x) { x@txpMissing })

#' @describeIn TxpResult-class Return `txpResultParam` slot
#' @export

setMethod("txpResultParam", "TxpResult", function(x) { x@txpResultParam })

#' @describeIn TxpResult-class Return `txpModel` slot
#' @export

setMethod("txpModel", "TxpResult", function(x) { x@txpModel })

#' @describeIn TxpResult-class Return `txpIDs` slot
#' @export

setMethod("txpIDs", "TxpResult", function(x) { x@txpIDs })

.TxpResult.replaceIDs <- function(x, value) {
  x@txpIDs <- value
  validObject(x)
  x
}

#' @rdname TxpResult-class
#' @export

setReplaceMethod("txpIDs", "TxpResult", .TxpResult.replaceIDs)

#' @describeIn TxpResult-class Return `txpWeights` slot from model -- shortcut
#' for `txpWeights(txpModel(x))`; default `adjusted = FALSE`, i.e. return
#' unadjusted weights
#' @importFrom rlang is_scalar_logical
#' @export

setMethod("txpWeights", "TxpResult", function(x, adjusted = FALSE) {
  stopifnot(is_scalar_logical(adjusted))
  txpWeights(txpModel(x), adjusted = adjusted)
})

#' @describeIn TxpResult-class Return `txpSlices` slot from model -- shortcut
#' for `txpSlices(txpModel(x))`
#' @export

setMethod("txpSlices", "TxpResult", function(x) { txpSlices(txpModel(x)) })

.TxpResult.txpTransFuncs <- function(x, level, simplify = FALSE) {
  stopifnot(is_scalar_logical(simplify))
  level <- match.arg(level, c("model", "slices"))
  if (level == "model") {
    return(txpTransFuncs(txpModel(x)))
  } else {
    return(txpTransFuncs(txpSlices(txpModel(x)), simplify = simplify))
  }
}

#' @describeIn TxpResult-class Return `txpTransFuncs` slot from model --
#' shortcut for `txpTransFuncs(txpModel(x))`
#' @importFrom rlang is_scalar_logical
#' @export

setMethod("txpTransFuncs", "TxpResult", .TxpResult.txpTransFuncs)

#' @describeIn TxpResult-class Return `txpValueNames` slot from slices --
#' shortcut for `txpValueNames(txpSlices(txpModel(x)))`
#' @export

setMethod("txpValueNames", "TxpResult", function(x, simplify = FALSE) {
  txpValueNames(txpSlices(txpModel(x)), simplify = simplify)
})

.TxpResult.squareBracket <- function(x, i, j, ..., drop = FALSE) {
  ss <- txpSliceScores(x, adjusted = FALSE)[i, , drop = FALSE]
  TxpResult(txpScores = txpScores(x)[i],
            txpSliceScores = ss,
            txpRanks = txpRanks(x)[i],
            txpMissing = txpMissing(x),
            txpModel = txpModel(x),
            txpIDs = txpIDs(x)[i],
            txpResultParam = txpResultParam(x))
}

#' @rdname TxpResult-class
#' @export

setMethod("[",
          c("TxpResult", "logical", "missing"),
          function(x, i, j, ..., drop = FALSE) {
  if (length(i) < length(x)) {
    warning("Length of logical vector less than length of object; ",
            "recycling vector")
  }
  .TxpResult.squareBracket(x, i)
})

#' @rdname TxpResult-class
#' @export

setMethod("[", c("TxpResult", "integer", "missing"), .TxpResult.squareBracket)

#' @rdname TxpResult-class
#' @export

setMethod("[", c("TxpResult", "numeric", "missing"), .TxpResult.squareBracket)

#' @rdname TxpResult-class
#' @export

setMethod("[",
          c("TxpResult", "character", "missing"),
          function(x, i, j, ..., drop = FALSE) {
  ids <- txpIDs(x)
  if (is.null(ids)) {
    stop("TxpResult object must have assigned names, e.g. txpIDs(), to ",
         "susbet using a character vector.")
  }
  ind <- match(i, ids)
  if (length(ind) == 1 && is.na(ind)) return(suppressWarnings(x[0]))
  .TxpResult.squareBracket(x, ind)
})


#' @describeIn TxpResult-class Return the number of observations; shortcut for
#' `length(txpScores(x))`
#' @export

setMethod("length", "TxpResult", function(x) { length(txpScores(x)) })

#' @describeIn TxpResult-class Sort the ``TxpResult` object by their ranks
#' @export

setMethod("sort", "TxpResult", function(x, decreasing = TRUE,
                                        na.last = TRUE, ...) {
  ind <- order(txpScores(x), decreasing = decreasing, na.last = na.last, ...)
  x[ind]
})

#' @describeIn TxpResult-class Returns IDs; equal to `txpIDs(x)`
#' @export

setMethod("names", "TxpResult", function(x) txpIDs(x))

#' @rdname TxpResult-class
#' @export

setReplaceMethod("names", "TxpResult", .TxpResult.replaceIDs)

##----------------------------------------------------------------------------##
## validity

.TxpResult.validity <- function(object) {
  msg <- NULL
  scores <- txpScores(object)
  sliceScores <- txpSliceScores(object, adjusted = FALSE)
  model <- txpModel(object)
  ids <- txpIDs(object)
  if (!is(sliceScores[1], "numeric")) {
    msg <- c(msg, "Entries in txpSliceScores must be \"numeric\"")
  }
  if (length(scores) != nrow(sliceScores)) {
    msg <- c(msg, "length(txpScores) != nrow(txpSliceScores)")
  }
  if (!is.null(ids) && length(ids) != length(scores)) {
    msg <- c(msg, "length(txpIDs) != length(object)")
  }
  if (is.null(msg)) return(TRUE)
  msg
}

setValidity2("TxpResult", .TxpResult.validity)

##----------------------------------------------------------------------------##
## coerce

#' @importFrom rlang is_scalar_character is_scalar_logical

.TxpResult.as.data.frame <- function(x,
                                     row.names = NULL,
                                     optional = FALSE,
                                     ...,
                                     id.name = "id",
                                     score.name = "score",
                                     rank.name = "rank",
                                     adjusted = FALSE) {
  stopifnot(is_scalar_character(id.name))
  stopifnot(is_scalar_character(score.name))
  stopifnot(is_scalar_character(rank.name))
  stopifnot(is_scalar_logical(adjusted))
  df <- as.data.frame(txpSliceScores(x, adjusted = adjusted), ...)
  df[[score.name]] <- txpScores(x)
  df[[rank.name]] <- txpRanks(x)
  df[[id.name]] <- txpIDs(x)
  outCols <- c(score.name, rank.name, names(txpSlices(x)))
  if (!is.null(df[[id.name]])) {
    outCols <- c(id.name, outCols)
  } else {
    warning("txpIDs(x) is NULL; no ID column in returned data.frame.")
  }
  df[ , outCols]
}

#' @describeIn TxpResult-class Coerce TxpResult to [base::data.frame] object
#' with IDs, scores, ranks, and slice scores
#' @export

setMethod("as.data.frame", "TxpResult", .TxpResult.as.data.frame)


##----------------------------------------------------------------------------##
## show

.TxpResult.show <- function(object) {
  cat(sprintf("TxpResult of length %s\n", length(object)))
  .coolcat("names(%d): %s\n", names(object))
}

setMethod("show", "TxpResult", .TxpResult.show)


##----------------------------------------------------------------------------##

##---------------------------------------------------------------------------##
## Non-exported, non-documented, package-wide utility functions
##---------------------------------------------------------------------------##

#' @importFrom S4Vectors coolcat

.coolcat <- function(...) coolcat(..., indent = 2)

.catslot <- function(x, object) {
  cat("  ", x, ":", " ", slot(object = object, name = x), "\n", sep = "")
}

.repFunc <- function(func, times) {
  lst <- vector(mode = "list", length = times)
  for (i in 1:times) lst[[i]] <- func
  do.call("TxpTransFuncList", lst)
}

.listDisplayNames <- function(x) {
  n <- length(x)
  lnms <- names(x)
  if (is.null(lnms)) lnms <- rep('', n)
  lnms[sapply(x, is.null)] <- "NULL"
  lnms
}

.dupList <- function(x) {
  duplicated(as.list(x))
}

.chkModelInput <- function(model, input) {
  stopifnot(is(model, "TxpModel"))
  stopifnot(is.data.frame(input))
  valNms <- txpValueNames(model, simplify = TRUE)
  inptNms <- names(input)
  if (!all(valNms %in% inptNms)) {
    miss <- valNms[!valNms %in% inptNms]
    msg <- "'input' missing the following data specified by 'model':\n    %s"
    stop(sprintf(msg, paste(miss, collapse = ", ")))
  }
  tstClass <- function(x) is.numeric(input[[x]])
  inptCls <- sapply(valNms, tstClass)
  if (!all(inptCls)) {
    nc2n <- valNms[!inptCls]
    msg <- "The following 'input' columns not numeric:\n    %s"
    stop(sprintf(msg, paste(nc2n, collapse = ", ")))
  }
}

.rmInfinite <- function(model, input) {
  ## Clean up infinite in input
  valNms <- txpValueNames(txpSlices(model), simplify = TRUE)
  notFinite <- sapply(valNms, function(x) any(is.infinite(input[[x]])))
  if (any(notFinite)) {
    warning("Some of the given inputs contained infinite values.")
    for (i in valNms[notFinite]) input[[i]][is.infinite(input[[i]])] <- NaN
  }
  input
}

#' @importFrom grDevices col2rgb rgb

.col2hex <- function(x) {
  mat <- col2rgb(x)
  rgb(red = mat[1, ], green = mat[2, ], blue = mat[3, ], maxColorValue = 255)
}

# replicated pryr functions

.to_env <- function(x, quiet = FALSE) {
  if (is.environment(x)) {
    x
  } else if (is.list(x)) {
    list2env(x)
  } else if (is.function(x)) {
    environment(x)
  } else if (length(x) == 1 && is.character(x)) {
    if (!quiet) message("Using environment ", x)
    as.environment(x)
  } else if (length(x) == 1 && is.numeric(x) && x > 0) {
    if (!quiet) message("Using environment ", search()[x])
    as.environment(x)
  } else {
    stop("Input can not be coerced to an environment", call. = FALSE)
  }
}

.substitute_q <- function(x, env) {
  stopifnot(is.language(x))
  env <- .to_env(env)
  
  call <- substitute(substitute(x, env), list(x = x))
  eval(call)
  
}

.make_function <- function(args, body, env = parent.frame()) {
  args <- as.pairlist(args)
  stopifnot(
    .all_named(args),
    is.language(body))
  env <- .to_env(env)
  
  eval(call("function", args, body), env)
}

.all_named <- function(x) {
  if (length(x) == 0) return(TRUE)
  !is.null(names(x)) && all(names(x) != "")
}
##----------------------------------------------------------------------------##

##----------------------------------------------------------------------------##
## methods-txpResultList
##----------------------------------------------------------------------------##

#' @name TxpResultList-class
#' @title List of TxpResult objects
#' @description Extension of [S4Vectors::SimpleList] that holds only [TxpResult] 
#' objects.
#' 
#' @param ... [TxpResult] object to create `TxpResultList` object
#' @param x `TxpResultList` object
#' 
#' @template roxgn-loadExamples
#' @template roxgn-calcTxpModelList
#' 
#' @seealso [TxpResult], [txpCalculateScores]
#' 
#' @examples
#' ## duplicated
#' duplicated(resLst)
#' 
#' ## Coercion
#' as(list(resLst[[1]], resLst[[2]]), "TxpResultList")
#' as.TxpResultList(list(res1 = resLst[[1]], res2 = resLst[[2]]))
#' 
#' as(resLst[[1]], "TxpResultList")
#' as.TxpResultList(resLst[[1]])

NULL

##----------------------------------------------------------------------------##
## constructor

#' @rdname TxpResultList-class
#' @export 

TxpResultList <- function(...) {
  listData <- list(...)
  new2("TxpResultList", listData)
}

##----------------------------------------------------------------------------##
## validity

.TxpResultList.validity <- function(object) {
  msg <- NULL
  valid <- vapply(object@listData, is, logical(1), "TxpResult")
  if (any(!valid)) {
    msg <- c(msg, "All TxpResult objects must be of class 'TxpResult.'")
  }
  if (is.null(msg)) return(TRUE)
  msg
}

setValidity2("TxpResultList", .TxpResultList.validity)

##----------------------------------------------------------------------------##
## show

.TxpResultList.show <- function(object) {
  lnms <- .listDisplayNames(object)
  .coolcat("  TxpResultList of length %d: %s\n", lnms)
}

setMethod("show", "TxpResultList", .TxpResultList.show)

##----------------------------------------------------------------------------##
## duplicated

#' @rdname TxpResultList-class
#' @export

setMethod("duplicated", "TxpResultList", function(x) .dupList(x))

##----------------------------------------------------------------------------##
## coercion 

.TxpResultList.from.list <- function(from) {
  do.call("TxpResultList", from)
}

setAs("list", "TxpResultList", .TxpResultList.from.list)

.TxpResultList.from.TxpResult <- function(from) {
  TxpResultList(from)
}

setAs("TxpResult", "TxpResultList", .TxpResultList.from.TxpResult)

#' @rdname TxpResultList-class
#' @export

as.TxpResultList <- function(x) as(x, "TxpResultList")

##----------------------------------------------------------------------------##

##----------------------------------------------------------------------------##
## TxpResultList tests
##----------------------------------------------------------------------------##

##----------------------------------------------------------------------------##
## Initialization/txpCalculateScores

test_that("We can create TxpResultList objects through txpCalculateScores", {
  data(txp_example_input, package = "toxpiR")
  sl <- TxpSliceList(s1 = TxpSlice(sprintf("metric%d", 1:2)),
                     s2 = TxpSlice(sprintf("metric%d", 3:7)))
  md1 <- TxpModel(sl, txpWeights = c(2, 1))
  md2 <- TxpModel(sl)
  md3 <- TxpModel(sl, 
                  txpTransFuncs = list(f1 = TxpTransFunc(), 
                                       f2 = TxpTransFunc()))
  ml <- TxpModelList(md1, md2, md3)
  expect_s4_class(res <- txpCalculateScores(model = ml, 
                                            input = txp_example_input, 
                                            id.var = "name"),
                  "TxpResultList")
  expect_s4_class(txpCalculateScores(model = as.list(ml),
                                     input = txp_example_input,
                                     id.var = "name"),
                  "TxpResultList")
  expect_equal(txpModel(res[[1]]), md1)
  expect_equal(txpModel(res[[2]]), md2)
  expect_equal(txpModel(res[[3]]), md3)
  expect_error(TxpResultList(NULL))
  expect_error(TxpResultList(NULL, res[[1]]))
  expect_error(TxpResultList(1))
  expect_error(txpCalculateScores(model = c(as.list(ml), "hello"),
                                  input = txp_example_input,
                                  id.var = "name"))
})

##----------------------------------------------------------------------------##
## Show

test_that("TxpResultList show method displays correct information", {
  expect_silent({
    data(txp_example_input, package = "toxpiR")
    sl <- TxpSliceList(s1 = TxpSlice(sprintf("metric%d", 1:2)),
                       s2 = TxpSlice(sprintf("metric%d", 3:7)))
    md1 <- TxpModel(sl, txpWeights = c(2, 1))
    md2 <- TxpModel(sl)
    md3 <- TxpModel(sl, 
                    txpTransFuncs = list(f1 = TxpTransFunc(), 
                                         f2 = TxpTransFunc()))
    ml <- TxpModelList(md1, m2 = md2, md3)
    l <- txpCalculateScores(model = ml, 
                            input = txp_example_input, 
                            id.var = "name")
  })
  expect_output(print(l), "TxpResultList of length 3")
  expect_output(print(l), "'' m2 ''")
  expect_silent(names(l) <- NULL)
  expect_output(print(l), "'' '' ''")
})

##----------------------------------------------------------------------------##
## Concatenation

test_that("We can concatenate TxpResultList objects", {
  expect_silent({
    data(txp_example_input, package = "toxpiR")
    sl <- TxpSliceList(s1 = TxpSlice(sprintf("metric%d", 1:2)),
                       s2 = TxpSlice(sprintf("metric%d", 3:7)))
    md1 <- TxpModel(sl, txpWeights = c(2, 1))
    md2 <- TxpModel(sl)
    md3 <- TxpModel(sl, 
                    txpTransFuncs = list(f1 = TxpTransFunc(), 
                                         f2 = TxpTransFunc()))
    ml <- TxpModelList(m1 = md1, md2, m3 = md3)
    l <- txpCalculateScores(model = ml, 
                            input = txp_example_input, 
                            id.var = "name")
  })
  expect_s4_class(cl <- c(l, rev(l), l), "TxpResultList")
  expect_length(cl, 9)
  expect_named(cl, c('m1', '', 'm3', 'm3', '', 'm1', 'm1', '', 'm3'))
})

##----------------------------------------------------------------------------##
## Duplicated

test_that("We can detect duplicate TxpResult objects in TxpResultList", {
  expect_silent({
    data(txp_example_input, package = "toxpiR")
    sl <- TxpSliceList(s1 = TxpSlice(sprintf("metric%d", 1:2)),
                       s2 = TxpSlice(sprintf("metric%d", 3:7)))
    md1 <- TxpModel(sl, txpWeights = c(2, 1))
    md2 <- TxpModel(sl)
    ml1 <- TxpModelList(m1 = md1, m2 = md1)
    ml2 <- TxpModelList(m1 = md1, m2 = md2)
    l1 <- txpCalculateScores(model = ml1, 
                             input = txp_example_input, 
                             id.var = "name")
    l2 <- txpCalculateScores(model = ml2, 
                             input = txp_example_input, 
                             id.var = "name")
  })
  expect_true(any(duplicated(l1)))
  expect_false(any(duplicated(l2)))
})

##----------------------------------------------------------------------------##
## Coercion

test_that("We can coerce to TxpResultList objects", {
  expect_silent({
    data(txp_example_input, package = "toxpiR")
    sl <- TxpSliceList(s1 = TxpSlice(sprintf("metric%d", 1:2)),
                       s2 = TxpSlice(sprintf("metric%d", 3:7)))
    md1 <- TxpModel(sl, txpWeights = c(2, 1))
    md2 <- TxpModel(sl)
    md3 <- TxpModel(sl, 
                    txpTransFuncs = list(f1 = TxpTransFunc(), 
                                         f2 = TxpTransFunc()))
    ml <- TxpModelList(m1 = md1, md2, m3 = md3)
    l <- lapply(ml, txpCalculateScores, 
                input = txp_example_input, 
                id.var = "name")
  })
  expect_s4_class(as.TxpResultList(l), "TxpResultList")
  expect_s4_class(as.TxpResultList(l[[1]]), "TxpResultList")
})

##----------------------------------------------------------------------------##
## pieGridGrob
##----------------------------------------------------------------------------##

#' @name pieGridGrob
#' @title Make grid of pieGrobs
#' @description Make grid of pieGrobs
#' 
#' @param radMat `matrix(<numeric>)`, observations by slice radii
#' @param wts `vector(<numeric>)`, relative weights of each slice
#' @param fills Vector of colors to fill slices
#' @param labels `vector(<character>)`, (optional) label for each observation
#' @param showRadSum Logical scalar, when `TRUE` show the weighted sum of slices
#' below the label
#' @param nrow,ncol Integer scalar, number of rows and columns for the grid
#' @param byrow Logical scalar, fill the grid by rows when `TRUE`
#' @param name,gp,vp Passed to [grid::gTree]
#' 
#' @examples 
#' \donttest{
#' library(grid)
#' 
#' s <- seq(0.2, 1, by = 0.1)
#' smat <- do.call("rbind", replicate(20, s, simplify = FALSE))
#' grid.newpage()
#' grid.pieGridGrob(radMat = smat)
#' 
#' rownames(smat) <- sprintf("obs%02d", 1:20)
#' grid.newpage()
#' grid.pieGridGrob(radMat = smat, wts = s)
#' grid.newpage()
#' grid.pieGridGrob(radMat = smat, wts = s, showRadSum = TRUE, labels = FALSE)
#' grid.newpage()
#' grid.pieGridGrob(radMat = smat, labels = "hello")
#' grid.newpage()
#' grid.pieGridGrob(radMat = smat, labels = 1:20)
#' 
#' ## Can edit like normal grid objects
#' grid.newpage()
#' grid.pieGridGrob(radMat = smat, wts = s, showRadSum = TRUE)
#' grid.ls() ## shows grid elements
#' grid.edit("pie-20", fills = 1:9)
#' grid.edit("pie-19-label", gp = gpar(font = 2, col = "red"))
#' grid.edit("pie-1", wts = rep(1, 9), rads = rep(1, 9))
#' for (s in sprintf("pie-%d-radSum", 2:4)) {
#'   grid.edit(s, gp = gpar(font = 2, col = "blue"))
#' }
#' }
#' 
#' @return `pieGrob` [grid::grob] object
#' 
#' @import grid
#' @importFrom rlang is_scalar_integerish is_scalar_logical
#' @importFrom grDevices colorRampPalette
#' @export

pieGridGrob <- function(radMat, wts = NULL, fills = NULL, labels = NULL, 
                        showRadSum = FALSE, ncol = NULL, nrow = NULL, 
                        byrow = TRUE, name = NULL, gp = NULL, vp = NULL) {
  
  nPie <- NROW(radMat)
  if (is.null(wts)) wts <- rep(1, NCOL(radMat))
  wts <- wts/sum(wts)
  if (is.null(labels) || (is.logical(labels) && labels)) {
    labels <- rownames(radMat) 
  } else {
    if (is.logical(labels) && !labels) labels <- NULL
  }
  pos <- makePieGridPos(nPie = nPie, nrow = nrow, ncol = ncol, byrow = byrow)
  pieGridVp <- makePieGridViewport(pos = pos, nPie = nPie)
  gTree(radMat = radMat,
        wts = wts,
        fills = fills,
        labels = labels,
        nPie = nPie,
        pos = pos,
        name = name,
        gp = gp,
        vp = vp,
        children = makePieGridGrob(radMat = radMat, 
                                   pos = pos, 
                                   wts = wts, 
                                   fills = fills, 
                                   labels = labels, 
                                   showRadSum = showRadSum,
                                   vp = pieGridVp),
        childrenvp = pieGridVp,
        cls = "pieGridGrob")
  
}

#' @rdname pieGridGrob
#' @export

grid.pieGridGrob <- function(radMat, wts = NULL, fills = NULL, labels = NULL, 
                             showRadSum = FALSE, ncol = NULL, nrow = NULL, 
                             byrow = TRUE, name = NULL, gp = NULL, vp = NULL) {
  g <- pieGridGrob(radMat = radMat,
                   wts = wts, 
                   fills = fills,
                   labels = labels,
                   showRadSum = showRadSum,
                   ncol = ncol,
                   nrow = nrow,
                   byrow = byrow,
                   name = name,
                   vp = vp,
                   gp = gp)
  grid.draw(g)
}

makePieGridPos <- function(nPie, nrow = NULL, ncol = NULL, byrow = TRUE) {
  stopifnot(is_scalar_integerish(nPie))
  stopifnot(is.null(ncol) || is_scalar_integerish(ncol))
  stopifnot(is.null(nrow) || is_scalar_integerish(nrow))
  stopifnot(is_scalar_logical(byrow))
  if (is.null(nrow) && is.null(ncol)) {
    ncol <- ceiling(sqrt(nPie))
    nrow <- ceiling(nPie/ncol)
  } else {
    if (is.null(nrow)) nrow <- ceiling(nPie/ncol)
    if (is.null(ncol)) ncol <- ceiling(nPie/nrow)
  }
  
  if(byrow) {
    pos <- expand.grid(col = 1:ncol, row = 1:nrow) 
  } else {
    pos <- expand.grid(row = 1:nrow, col = 1:ncol)
  }
  pos
}

makePieGridViewport <- function(pos, nPie) {
  nrow <- max(pos$row)
  ncol <- max(pos$col)
  gl <- grid.layout(nrow = nrow, 
                    ncol = ncol, 
                    widths = unit(rep_len(1, ncol), "null"),
                    heights = unit(rep(1, nrow), "null"))
  gridVp <- viewport(layout = gl)
  pieBoxes <- vector("list", nPie)
  for (i in 1:nPie) {
    pieBoxes[[i]] <- viewport(name = sprintf("pieBox-%s", i),
                              layout.pos.row = pos[i, "row"],
                              layout.pos.col = pos[i, "col"])
  }
  vpTree(gridVp, do.call("vpList", pieBoxes))
}

makePieGridGrob <- function(radMat, pos, wts = NULL, fills = NULL, 
                            labels = NULL, showRadSum = FALSE, vp = NULL) {
  
  nPie <- NROW(radMat)
  pies <- vector("list", nPie)
  
  for (i in 1:nPie) {
    rads <- radMat[i, ]
    pies[[i]] <- frameGrob(vp = vpStack(vp$parent, vp$children[[i]]))
    pies[[i]] <- packGrob(frame = pies[[i]],
                          grob = pieGrob(rads = rads,
                                         fills = fills,
                                         wts = wts,
                                         name = sprintf("pie-%s", i)))
    if (showRadSum) {
      rsum <- round(sum(rads*wts, na.rm = TRUE), 4)
      pies[[i]] <- packGrob(frame = pies[[i]], 
                            grob = textGrob(label = rsum,
                                            name = sprintf("pie-%s-radSum", i)),
                            side = "top",
                            height = unit(1.15, units = "char"))
    }
    
    if (!is.null(labels)) {
      pies[[i]] <- packGrob(frame = pies[[i]], 
                            grob = textGrob(label = labels[i],
                                            name = sprintf("pie-%s-label", i)),
                            side = "top",
                            height = unit(1.15, units = "char"))
    }
  }
  
  do.call("gList", pies)
  
}

##----------------------------------------------------------------------------##

##----------------------------------------------------------------------------##
## All classes
##----------------------------------------------------------------------------##

#' @import methods
#' @importFrom S4Vectors setValidity2 new2
#' @importClassesFrom S4Vectors character_OR_NULL

NULL

##----------------------------------------------------------------------------##
## Virtual classes

#' @importClassesFrom S4Vectors SimpleList
#' @importClassesFrom S4Vectors List
#' @importFrom S4Vectors List

setClass("NamedList", contains = c("VIRTUAL", "SimpleList"))

##----------------------------------------------------------------------------##
## TxpTransFunc

#' @rdname TxpTransFunc-class
#' @exportClass TxpTransFunc

setClass("TxpTransFunc", contains = "function", prototype = function(x) x)

setClassUnion("TxpTransFunc_OR_NULL", members = c("TxpTransFunc", "NULL"))

##----------------------------------------------------------------------------##
## TxpTransFuncList

#' @rdname TxpTransFuncList-class
#' @exportClass TxpTransFuncList

setClass("TxpTransFuncList",
         contains = "SimpleList",
         prototype = prototype(elementType = "TxpTransFunc_OR_NULL"))

##----------------------------------------------------------------------------##
## TxpSlice

#' @rdname TxpSlice-class
#' @exportClass TxpSlice

setClass("TxpSlice",
         slots = c(txpValueNames = "character",
                   txpTransFuncs = "TxpTransFuncList"))

setClassUnion("TxpSlice_OR_NULL", members = c("TxpSlice", "NULL"))

##----------------------------------------------------------------------------##
## TxpSliceList

#' @rdname TxpSliceList-class
#' @importClassesFrom S4Vectors SimpleList
#' @exportClass TxpSliceList

setClass("TxpSliceList",
         contains = "NamedList",
         prototype = prototype(elementType = "TxpSlice"))

##----------------------------------------------------------------------------##
## TxpModel

#' @rdname TxpModel-class
#' @exportClass TxpModel

setClass("TxpModel",
         slots = c(txpSlices = "TxpSliceList",
                   txpWeights = "numeric",
                   txpTransFuncs = "TxpTransFuncList"))

setClassUnion("TxpModel_OR_NULL", members = c("TxpModel", "NULL"))

##----------------------------------------------------------------------------##
## TxpModelList

#' @rdname TxpModelList-class
#' @importClassesFrom S4Vectors SimpleList
#' @exportClass TxpModelList

setClass("TxpModelList",
         contains = "SimpleList",
         prototype = prototype(elementType = "TxpModel"))

##----------------------------------------------------------------------------##
## TxpResultParam

#' @name TxpResultParam-class
#' @exportClass TxpResultParam

setClass("TxpResultParam",
         slots = c(rank.ties.method = "character",
                   negative.value.handling = "character"))

##----------------------------------------------------------------------------##
## TxpResult

#' @name TxpResult-class
#' @exportClass TxpResult

setClass("TxpResult",
         slots = c(txpScores = "numeric",
                   txpSliceScores = "matrix",
                   txpRanks = "numeric",
                   txpMissing = "numeric",
                   txpModel = "TxpModel",
                   txpIDs = "character_OR_NULL",
                   txpResultParam = "TxpResultParam"))

setClassUnion("TxpResult_OR_NULL", members = c("TxpResult", "NULL"))

##----------------------------------------------------------------------------##
## TxpResultList

#' @rdname TxpResultList-class
#' @importClassesFrom S4Vectors SimpleList
#' @exportClass TxpResultList

setClass("TxpResultList",
         contains = "SimpleList",
         prototype = prototype(elementType = "TxpResult"))

##----------------------------------------------------------------------------##
## boxLegendGrob
##----------------------------------------------------------------------------##

#' @name boxLegendGrob
#' @title Create a filled-box legend
#' @description Create a filled-box legend
#' @param labels Character, the legend labels
#' @param fills Colors to fill the slices
#' @param name,vp,gp Passed to [grid::frameGrob]
#' 
#' @details 
#' Not yet exported. Need to break out the creation of viewports and grobs as
#' done in the exported grobs. This will allow better grobEdit methods, which
#' also needs to be created for the boxLegendGrob.
#' Also need to do some input checks.
#' 
#' Also, if \code{grid::legendGrob} gets updated to use the 'has.fill' option 
#' this function should be removed and \code{grid::legendGrob} can be used
#' instead.
#' 
#' @import grid 

boxLegendGrob <- function(labels, fills, name = NULL, vp = NULL, gp = NULL) {
  
  wids <- c(unit(1.5, "char"), unit(max(nchar(labels)), "char"))
  hgts <- unit(rep_len(1.5, length(labels)), "char")
  fg <- frameGrob(layout = grid.layout(ncol = 2, 
                                       nrow = length(labels),
                                       widths = wids,
                                       heights = hgts),
                  vp = vp, 
                  name = name, 
                  gp = gp)
  for (i in seq_along(labels)) {
    fg <- placeGrob(frame = fg, 
                    grob = textGrob(label = labels[i]), 
                    col = 2, 
                    row = i)
    fg <- placeGrob(frame = fg, 
                    grob = rectGrob(width = unit(1, "char"),
                                    height = unit(1, "char"),
                                    gp = gpar(fill = fills[i], col = NA)),
                    col = 1, 
                    row = i)
  }
  
  fg

}

##----------------------------------------------------------------------------##

##----------------------------------------------------------------------------##
## plot methods for TxpResult
##----------------------------------------------------------------------------##

#' @name TxpResult-plot
#' @title Plot TxpResult objects
#' @description Plot [TxpResult] objects
#' @aliases plot
#'
#' @param x [TxpResult] object
#' @param y Rank vector, i.e. `txpRanks(x)`
#' @param package Character scalar, choice of "grid" or "ggplot2" for plotting
#' ToxPi profiles
#' @param fills Vector of colors to fill slices. Set to NULL to use default
#' @param showScore Logical scalar, overall score printed below the name when
#' `TRUE`
#' @param labels Integer vector, indices of `x` to label in the rank plot
#' @param margins Passed to [grid::plotViewport]; only affects the scatterplot
#' region margins
#' @param gp,vp,name Passed to [grid::frameGrob] when creating the plotting
#' area
#' @param newpage Logical scalar, [grid::grid.newpage] called prior to plotting
#' when `TRUE`
#' @param ... Passed to [pieGridGrob] when plotting ToxPi and to pointsGrob
#' when plotting ranks
#' @param ncol Number of columns for ggplot2 ToxPi profiles
#' @param bgColor,borderColor,sliceBorderColor,sliceValueColor,sliceLineColor
#' Various color options when creating ggplot2 ToxPi profiles. Set to NULL
#' for no color
#' @param showMissing Boolean for coloring data missingness in ggplot2
#' ToxPi profiles
#' @param showCenter Boolean for showing inner circle in ggplot2 ToxPi 
#' profiles. When set to False overrides showMissing
#'
#' @details
#' It is strongly recommended to use a specific device (e.g., [grDevices::png],
#' [grDevices::pdf]) when creating rank plots.
#' Using a GUI device will likely lead to inaccurate labeling, and any changes
#' to the device size WILL lead to inaccurate labeling.
#'
#' The plotting is built on the [grid::grid-package], and can be adjusted or
#' edited as such.
#'
#' If the labels are running of the device, the top or bottom margins can be
#' increased with the `margins` parameter.
#'
#' ToxPi profiles can also be plotted using the ggplot2 package.
#'
#' @template roxgn-loadExamples
#' @template roxgn-calcTxpModel
#'
#' @examples
#' library(grid)
#' plot(res)
#' plot(res[order(txpRanks(res))[1:4]])
#'
#' library(ggplot2)
#' plot(res, package = "gg")
#' plot(res[order(txpRanks(res))], package = "gg", ncol = 5) +
#'   theme(legend.position = "bottom")
#'
#' plot(res, txpRanks(res))
#' plot(res, txpRanks(res), pch = 16, size = unit(0.75, "char"))
#'
#' ## Will likely make inaccurate labels within a GUI, e.g. RStudio
#' ## use png, pdf, etc. to get accurate labels
#' \dontrun{
#' tmpPdf <- tempfile()
#' pdf(tmpPdf)
#' plot(res, txpRanks(res), labels = c(10, 4, 2), pch = 16)
#' dev.off()
#' }
#'
#' @return No return value when using grid; called for side effect (i.e.
#' drawing in current graphics device). Will return ggplot2 object otherwise.

NULL

.TxpResult.toxpiPlot <- function(
    x,
    package = c("grid", "ggplot2"),
    fills = NULL,
    showScore = TRUE,
    gp = NULL,
    vp = NULL,
    name = NULL,
    newpage = TRUE,
    ...,
    ncol = NULL,
    bgColor = "grey80",
    borderColor = "white",
    sliceBorderColor = "white",
    sliceValueColor = NULL,
    sliceLineColor = NULL,
    showMissing = TRUE,
    showCenter = TRUE) {

  if (tolower(substr(package[1], 0, 2)) == "gg") {
    .TxpResult.toxpiGGPlot(
      x, fills, showScore, ncol, bgColor, borderColor,
      sliceBorderColor, sliceValueColor, sliceLineColor, showMissing, 
      showCenter
    )
  } else {
    .TxpResult.toxpiGridPlot(
      x, fills, showScore, gp, vp, name, newpage, ...
    )
  }

}

#' @describeIn TxpResult-plot Plot ToxPi diagrams
#' @export

setMethod("plot", c("TxpResult", "missing"), .TxpResult.toxpiPlot)

#' @import grid

.TxpResult.toxpiGridPlot <- function(x,
                                     fills = NULL,
                                     showScore = TRUE,
                                     gp = NULL,
                                     vp = NULL,
                                     name = NULL,
                                     newpage = TRUE,
                                     ...) {

  if (is.null(fills)) fills <- getOption("txp.fills", TXP_FILLS)
  sNames <- names(txpSlices(x))
  pg <- pieGridGrob(txpSliceScores(x, adjusted = FALSE),
                    wts = txpWeights(x),
                    labels = txpIDs(x),
                    fills = fills,
                    showRadSum = showScore,
                    ...)
  lg <- boxLegendGrob(labels = sNames, fills = fills)
  wids <- unit(c(10, 1), "grobwidth", lg)
  fg <- frameGrob(layout = grid.layout(nrow = 1, ncol = 2, widths = wids),
                  name = name,
                  gp = gp,
                  vp = vp)
  fg <- placeGrob(frame = fg, grob = pg, row = 1, col = 1)
  fg <- placeGrob(frame = fg, grob = lg, row = 1, col = 2)
  if (newpage) grid.newpage()
  grid.draw(fg)

}

#' @importFrom rlang is_named is_integerish is_scalar_logical
#' @import grid

.TxpResult.rankPlot <- function(x, y, labels = NULL, newpage = TRUE,
                                margins = c(4, 0, 1, 1),
                                name = NULL, gp = NULL, vp = NULL, ...) {

  stopifnot(is_scalar_logical(newpage))
  stopifnot(is.null(labels) || is_integerish(labels))

  drawLabels <- !is.null(labels)

  if (newpage) grid.newpage()

  if (drawLabels) {
    stopifnot(is_named(x))
    names(labels) <- txpIDs(x[labels])
    labelWidth <- .maxStrWidth(names(labels)) + unit(5, "char")
  } else {
    labelWidth <- unit(0, "mm")
  }

  gl <- grid.layout(nrow = 1, ncol = 2, unit.c(labelWidth, unit(1, "null")))

  fg <- frameGrob(layout = gl, name = name, gp = gp, vp = vp)

  rnk <- annScatterGrob(x = txpScores(x),
                        y = y,
                        ann = if (drawLabels) labels else NULL,
                        yscale = rev(extendrange(range(y))),
                        yaxis = FALSE,
                        xlab = "ToxPi Score",
                        margins = margins,
                        ...)

  fg <- placeGrob(frame = fg, grob = rnk, row = 1, col = 2)
  grid.draw(fg)

  if (drawLabels) {
    lblGrob <- .refLabel(names(labels), labelWidth)
    fg <- placeGrob(frame = fg, grob = lblGrob, row = 1, col = 1)
    grid.draw(fg$children[fg$childrenOrder[2]])
  }

}

#' @describeIn TxpResult-plot Plot ToxPi ranks
#' @export

setMethod("plot", c("TxpResult", "numeric"), .TxpResult.rankPlot)

#' @import ggplot2

.TxpResult.toxpiGGPlot <- function(
    x,
    fills = NULL,
    showScore = TRUE,
    ncol = NULL,
    bgColor = "grey80",
    borderColor = "white",
    sliceBorderColor = "white",
    sliceValueColor = NULL,
    sliceLineColor = NULL,
    showMissing = TRUE,
    showCenter = TRUE
    ) {

  # Set to NULL to prevent note from devtools::check()
  left <- right <- mid <- radii <- Slices <- NULL

  if (is.null(fills)) {
    fills <- getOption("txp.fills", TXP_FILLS)
  }

  #get plotting df
  toxResultDF <- as.data.frame(x)
  txpModel <- txpModel(x)
  profileDF <- .getPlotList(txpWeights(x), names(txpModel), toxResultDF)

  #make plot
  if(showCenter){
    innerRad <- 0.1 # percent
  } else {
    innerRad <- 0
  }
  yText <- 1.22

  plot <- ggplot2::ggplot(profileDF) +
    ggplot2::theme_void() +
    ggplot2::ylim(0, ifelse(is.null(sliceValueColor), 1, yText)) +
    ggplot2::theme(plot.margin = ggplot2::margin(2, 2, 2, 2, unit = "mm"))

  if (showScore) {
    plot <- plot + ggplot2::facet_wrap(
      ~factor(NameScore, levels = unique(profileDF$NameScore)),
      ncol = ncol
    )
  } else {
    plot <- plot + ggplot2::facet_wrap(
      ~factor(Name, levels = unique(profileDF$Name)),
      ncol = ncol
    )
  }

  if (!is.null(sliceLineColor)) {
    nSlices <- length(unique(profileDF$Slices))
    x1 <- profileDF$left
    y1 <- rep(innerRad, length(x1))
    xend <- x1
    yend <- rep(1, length(x1))
    plot <- plot + ggplot2::geom_segment(
      ggplot2::aes(x = x1, y = y1, xend = xend, yend = yend),
      linetype = "dashed",
      colour = sliceLineColor
    )
  }
  if(showCenter){
    if (showMissing) {
      missingData <- txpMissing(x)
    } else {
      missingData <- rep(0, length(txpSlices(x)))
    }
    plot <- plot + ggplot2::geom_rect(
      ggplot2::aes(xmin = left, xmax = right, ymin = 0, ymax = innerRad),
      fill = rep(grDevices::gray(1 - missingData), length(x))
    )
  }

  if (!is.null(sliceBorderColor)) {
    plot <- plot + ggplot2::geom_rect(
      ggplot2::aes(
        xmin = left,
        xmax = right,
        ymin = innerRad,
        ymax = innerRad + radii * (1 - innerRad),
        fill = Slices
      ),
      color = sliceBorderColor,
      linewidth = 0.5
    )
  } else {
    plot <- plot + ggplot2::geom_rect(
      ggplot2::aes(
        xmin = left,
        xmax = right,
        ymin = innerRad,
        ymax = innerRad + radii * (1 - innerRad),
        fill = Slices
      )
    )
  }

  plot <- plot + ggplot2::scale_fill_manual(
    breaks = unique(profileDF$Slices),
    values = fills
  )

  if (!is.null(borderColor)) {
    plot <- plot + ggplot2::geom_hline(
      yintercept = 1, color = borderColor, linewidth = 0.5
    )
  }

  if (!is.null(sliceValueColor)) {
    plot <- plot + ggplot2::geom_text(
      ggplot2::aes(
        x = mid,
        y = yText,
        label = as.character(radii)
      ),
      colour = sliceValueColor,
      size = 3
    )
  }

  plot <- plot + ggplot2::geom_hline(
    yintercept = innerRad, color = "black", linewidth = 0.4
  )

  if (!is.null(bgColor)) {
    plot <- plot + ggplot2::theme(
      panel.background = ggplot2::element_rect(fill = bgColor, color = bgColor)
    )
  }

  plot + ggplot2::coord_polar(start = 3 * pi / 2, direction = -1)

}

.getSlicePositions <- function(wts) {
  endWts <- cumsum(wts)
  startWts <- c(0, utils::head(endWts, -1))
  list(start = startWts, end = endWts)
}

# Generate dataframe for plotting a profile
.generateProfileDF <- function(startWts, endWts, radii, sliceNames, id, score) {
  df <- data.frame(
    left = startWts,
    right = endWts,
    mid = (startWts + endWts) / 2,
    radii = round(radii, 3),
    Slices = sliceNames,
    Name = id,
    Score = round(score, 4)
  )
  df$NameScore <- paste(df$Name, df$Score, sep = "\n")
  df
}

#get dataframe containing all necessary info for selected samples
.getPlotList <- function(wts, sliceNames, data) {
  pos <- .getSlicePositions(wts)
  do.call(rbind, lapply(1:nrow(data), function(x) {
    .generateProfileDF(
      pos$start, pos$end, unlist(data[x, sliceNames]), sliceNames,
      data[x, "id"], data[x, "score"]
    )
  }))
}

.maxStrWidth <- function(x) {
  wids <- lapply(x, stringWidth)
  wids[[which.max(sapply(wids, convertWidth, "inches"))]]
}

.refLabel <- function(lbl, xloc) {

  yloc <- do.call("unit.c", sapply(lbl, .getDeviceLoc)["y", ])
  ord <- order(yloc)
  yloc <- yloc[ord]
  lbl <- lbl[ord]

  n <- length(lbl)
  ypos <- yloc
  wd <- convertUnit(unit(1, "char"), "in")
  ht <- wd*1.2
  mid <- (n + 1) %/% 2
  # ypos[mid] <- yloc[mid]
  if (n > 1) {
    for (i in (mid + 1):n) {
      ypos[i] <- max(yloc[i], ypos[i - 1] + ht)
    }
  }
  if (n > 2) {
    for (i in (mid - 1):1) {
      ypos[i] <- min(yloc[i], ypos[i + 1] - ht)
    }
  }

  x1 <- rep(xloc, n)
  x2 <- x1 - 0.5*wd
  x3 <- x2 - 2*wd
  x4 <- x3 - 0.5*wd

  s1 <- segmentsGrob(x0 = unit(x1, "npc"),
                     y0 = unit(yloc, "npc"),
                     x1 = unit(x2, "npc"),
                     y1 = unit(yloc, "npc"))
  s2 <- segmentsGrob(x0 = unit(x2, "npc"),
                     y0 = unit(yloc, "npc"),
                     x1 = unit(x3, "npc"),
                     y1 = unit(ypos, "npc"))
  s3 <- segmentsGrob(x0 = unit(x3, "npc"),
                     y0 = unit(ypos, "npc"),
                     x1 = unit(x4, "npc"),
                     y1 = unit(ypos, "npc"))
  tg <- textGrob(label = lbl, x = wd, y = ypos, just = "left")

  gTree(children = gList(s1, s2, s3, tg))

}

.getDeviceLoc <- function(x, units = "npc") {
  xPth <- grid.grep(x, viewports = TRUE, global = TRUE)[[1]]
  depth <- downViewport(attr(xPth, "vpPath"))
  xGrb <- grid.get(xPth)
  loc <- deviceLoc(xGrb$x, xGrb$y)
  upViewport(depth)
  loc
}

##----------------------------------------------------------------------------##

##----------------------------------------------------------------------------##
## methods-TxpModel
##----------------------------------------------------------------------------##

#' @name TxpModel-class
#' @title ToxPi Model
#' @description S4 class to store ToxPi models
#' 
#' @slot txpSlices [TxpSliceList] object
#' @slot txpWeights numeric vector specifying the relative weight of each slice; 
#' when NULL, defaults to 1 (equal weighting) for each slice 
#' @slot txpTransFuncs [TxpTransFuncList] object (or list of functions 
#' coercible to TxpTransFuncList)
#' 
#' @param txpSlices Passed to `txpSlices` slot
#' @param txpWeights Passed to `txpWeights` slot
#' @param txpTransFuncs Passed to `txpTransFuncs` slot
#' @param x,y TxpModel object
#' @param value Replacement value
#' @param adjusted Scalar logical, when `TRUE` weights are adjusted to sum to 1
#' @param simplify Scalar logical, when `TRUE` the returned `list` is simplified
#' 
#' @examples 
#' ## Create TxpSliceList & TxpTransFuncList objects
#' s1 <- list(S1 = TxpSlice("inpt1"), S2 = TxpSlice("inpt2"))
#' tf <- list(NULL, sqrt = function(x) sqrt(x))
#' 
#' ## Create TxpModel object
#' m1 <- TxpModel(txpSlices = s1, txpWeights = 2:1, txpTransFuncs = tf)
#' m1
#' 
#' ## Access TxpModel slots
#' txpSlices(m1)
#' txpWeights(m1)
#' txpWeights(m1, adjusted = TRUE)
#' txpTransFuncs(m1)
#' 
#' ## length
#' length(m1) ## equal to length(txpSlices(m1))
#' length(m1) == length(txpSlices(m1))
#' 
#' ## names
#' names(m1) ## equal to names(txpSlices(m1))
#' all(names(m1) == names(txpSlices(m1)))
#' 
#' ## Replacement
#' m2 <- m1
#' txpSlices(m2) <- list(S3 = TxpSlice("inpt3"), S4 = TxpSlice("inpt4"))
#' m2
#' names(m2)[2] <- "hello"
#' names(m2)
#' txpTransFuncs(m2) <- NULL
#' m2
#' txpTransFuncs(m2)[[1]] <- function(x) x^2
#' names(txpTransFuncs(m2))[1] <- "sq"
#' m2
#' 
#' ## merge
#' m3 <- merge(m1, m2)
#' m3

NULL

##----------------------------------------------------------------------------##
## constructor

#' @rdname TxpModel-class
#' @export 

TxpModel <- function(txpSlices, txpWeights = NULL, txpTransFuncs = NULL) {
  if (!is(txpSlices, "TxpSliceList")) txpSlices <- as.TxpSliceList(txpSlices)
  n <- length(txpSlices)
  if (is.null(txpWeights)) txpWeights <- rep(1, n)
  if (is.null(txpTransFuncs)) {
    txpTransFuncs <- as(List(vector("list", n)), "TxpTransFuncList")
  }
  if (!is(txpTransFuncs, "TxpTransFuncList")) {
    txpTransFuncs <- as.TxpTransFuncList(txpTransFuncs)
  }
  new2("TxpModel", 
       txpSlices = txpSlices, 
       txpWeights = txpWeights, 
       txpTransFuncs = txpTransFuncs) 
}

##----------------------------------------------------------------------------##
## accessors

#' @describeIn TxpModel-class Return `txpSlices` slot
#' @aliases TxpModel-txpSlices
#' @export

setMethod("txpSlices", "TxpModel", function(x) { 
  x@txpSlices 
})

#' @rdname TxpModel-class
#' @export

setReplaceMethod("txpSlices", "TxpModel", function(x, value) {
  if (!is(value, "TxpSliceList")) value <- as.TxpSliceList(value)
  x@txpSlices <- value
  validObject(x)
  x
})

#' @describeIn TxpModel-class Return `txpWeights` slot
#' @param adjusted Scalar logical, should the returned weights be adjusted 
#' such that they sum to 1?
#' @importFrom rlang is_scalar_logical
#' @export

setMethod("txpWeights", "TxpModel", function(x, adjusted = FALSE) { 
  stopifnot(is_scalar_logical(adjusted))
  wts <- x@txpWeights 
  if (adjusted) wts <- wts/sum(wts)
  wts
})

#' @rdname TxpModel-class
#' @export

setReplaceMethod("txpWeights", "TxpModel", function(x, value) {
  x@txpWeights <- value
  validObject(x)
  x
})

#' @describeIn TxpModel-class Return `txpTransFuncs` slot 
#' @export

setMethod("txpTransFuncs", "TxpModel", function(x) { 
  x@txpTransFuncs 
})

#' @rdname TxpModel-class
#' @export

setReplaceMethod("txpTransFuncs", "TxpModel", function(x, value) {
  if (is.null(value)) value <- vector("list", length(x))
  if (!is(value, "TxpTransFuncList")) value <- as.TxpTransFuncList(value)
  x@txpTransFuncs <- value
  validObject(x)
  x
})

#' @describeIn TxpModel-class Return `list` of `txpValueNames` slots for the 
#' contained [TxpSliceList] object, or `vector` when `simplify = TRUE`
#' @importFrom rlang is_scalar_logical
#' @export

setMethod("txpValueNames", "TxpModel", function(x, simplify = FALSE) {
  stopifnot(is_scalar_logical(simplify))
  nms <- txpValueNames(txpSlices(x), simplify = simplify)
  nms
})

#' @describeIn TxpModel-class Return slice names; shortcut for 
#' `names(txpSlices(x))`
#' @export

setMethod("names", "TxpModel", function(x) { 
  names(txpSlices(x))
})

#' @rdname TxpModel-class
#' @export

setReplaceMethod("names", "TxpModel", function(x, value) {
  names(x@txpSlices) <- value
  validObject(x, complete = TRUE)
  x
})

.TxpModel.calc <- function(model, input, 
                           id.var = NULL,
                           rank.ties.method = c("average", "first", "last", 
                                                "random", "max", "min"),
                           negative.value.handling = c("keep", "missing")) {
  .calculateScores(model = model, 
                   input = input, 
                   id.var = id.var, 
                   rank.ties.method = rank.ties.method,
                   negative.value.handling = negative.value.handling)
}

#' @describeIn TxpModel-class Return number of slices in model; shortcut for
#' `length(txpSlices(x))`
#' @export

setMethod("length", "TxpModel", function(x) { 
  length(txpSlices(x))
})

#' @rdname txpCalculateScores
#' @export

setMethod("txpCalculateScores", c("TxpModel", "data.frame"), .TxpModel.calc)

##----------------------------------------------------------------------------##
## validity

.TxpModel.validity <- function(object) {
  msg <- NULL
  sl <- txpSlices(object)
  wt <- txpWeights(object)
  tf <- txpTransFuncs(object)
  if (length(sl) != length(wt)) {
    tmp <- "length(txpSlices(<TxpModel>)) != length(txpWeights(<TxpModel>))"
    msg <- c(msg, tmp)
  }
  if (length(sl) != length(tf)) {
    tmp <- "length(txpSlices(<TxpModel>)) != length(txpTransFuncs(<TxpModel>))"
    msg <- c(msg, tmp)
  }
  valNms <- txpValueNames(sl, simplify = TRUE)
  valDup <- duplicated(valNms)
  if (any(valDup)) {
    dup <- valNms[valDup]
    wrn <- "The following 'input' columns are duplicated in the model:\n    %s"
    warning(sprintf(wrn, paste(dup, collapse = ", ")))
  }
  if (is.null(msg)) return(TRUE)
  msg
}

setValidity2("TxpModel", .TxpModel.validity)

##----------------------------------------------------------------------------##
## show

.TxpModel.show <- function(object) {
  fnms <- .listDisplayNames(txpTransFuncs(object))
  cat(sprintf("TxpModel with %d slices.\n", length(txpSlices(object))))
  .coolcat("txpSlices(%d): %s\n", names(txpSlices(object)))
  .coolcat("txpWeights(%d): %s\n", txpWeights(object))
  .coolcat("txpTransFuncs(%d): %s\n", fnms)
}

setMethod("show", "TxpModel", .TxpModel.show)

##----------------------------------------------------------------------------##
## merge

.TxpModel.merge <- function(x, y) {
  sls <- c(txpSlices(x), txpSlices(y))
  wts <- c(txpWeights(x), txpWeights(y))
  tfs <- c(txpTransFuncs(x), txpTransFuncs(y))
  TxpModel(txpSlices = sls, txpWeights = wts, txpTransFuncs = tfs)
}

#' @describeIn TxpModel-class Merge two `TxpModel` objects into a single 
#' model
#' @export

setMethod("merge", c("TxpModel", "TxpModel"), .TxpModel.merge)

##----------------------------------------------------------------------------##

##----------------------------------------------------------------------------##
## annScatterGrob -- not yet exported
##----------------------------------------------------------------------------##

#' @importFrom grDevices extendrange
#' @import grid

annScatterGrob <- function(x, y, ann = NULL, xlab = NULL, ylab = NULL,
                           xscale = NULL, yscale = NULL,
                           xaxis = TRUE, yaxis = TRUE,
                           margins = c(5.1, 4.1, 4.1, 2.1),
                           name = NULL, gp = NULL, vp = NULL, ...) {
  
  if (is.null(xscale)) xscale <- extendrange(range(x))
  if (is.null(yscale)) yscale <- extendrange(range(y))
  
  pltVp <- plotViewport(margins = margins, name = "annSctrPlotVp")
  datVp <- viewport(xscale = xscale, yscale = yscale, name = "annSctrDataVp")
  annSctrVp <- vpTree(pltVp, vpList(datVp))
  annSctr <- makeAnnScatter(x = x, 
                            y = y, 
                            ann = ann, 
                            xaxs = xaxis, 
                            yaxs = yaxis, 
                            xlab = xlab, 
                            ylab = ylab, 
                            vp = annSctrVp, 
                            ...)
  gTree(name = name,
        ann = ann,
        xscale = xscale,
        yscale = yscale,
        vp = vp,
        childrenvp = annSctrVp,
        children = annSctr,
        cl = "annScatterGrob")
  
}

grid.annScatterGrob <- function(x, y, ann = NULL, xlab = NULL, ylab = NULL,
                                xscale = NULL, yscale = NULL,
                                xaxis = TRUE, yaxis = TRUE,
                                margins = c(5.1, 4.1, 4.1, 2.1),
                                name = NULL, gp = NULL, vp = NULL, ...) {
  
  g <- annScatterGrob(x = x, 
                      y = y, 
                      ann = ann, 
                      xlab = xlab, 
                      ylab = ylab,
                      xaxis = xaxis, 
                      yaxis = yaxis,
                      xscale = xscale, 
                      yscale = yscale, 
                      margins = margins,
                      name = name, 
                      gp = gp, 
                      vp = vp, 
                      ...)
  grid.draw(g)
  
}

makeAnnScatter <- function(x, y, ann, xaxs, yaxs, xlab, ylab, vp, ...) {
  
  annLst <- vector(mode = "list", length = length(ann)) 
  nms <- names(ann)
  if (is.null(nms)) nms <- sprintf("ann-%s", seq_along(ann))
  for (i in seq_along(ann)) {
    ind <- ann[i]
    annLst[[i]] <- nullGrob(x = unit(x[ind], "native"), 
                            y = unit(y[ind], "native"), 
                            name = nms[i])
  }
  grbLst <- gList()
  grbLst[['annotations']] <- gTree(name = "annotations", 
                                   vp = vp,
                                   children = do.call("gList", annLst))
  grbLst[['sctr']] <- pointsGrob(x = x, y = y, vp = vp, ...)
  if (xaxs) grbLst[['xaxs']] <- grid.xaxis(draw = FALSE, vp = vp)
  if (yaxs) grbLst[['yaxs']] <- grid.yaxis(draw = FALSE, vp = vp)
  if (!is.null(xlab)) {
    grbLst[['xlab']] <- textGrob(xlab, y = unit(-3, "line"), vp = vp)
  }
  if (!is.null(ylab)) {
    grbLst[['ylab']] <- textGrob(ylab, 
                                 x = unit(-3, "line"), 
                                 vp = vp, 
                                 rot = 90)
  }
  
  grbLst
  
}

##----------------------------------------------------------------------------##
## txpCalculateScores
##----------------------------------------------------------------------------##

#' @name txpCalculateScores
#' @title Calculate ToxPi Scores for the given model and input data
#' @description Calculate ToxPi Scores for the given model and input data
#'
#' @param model [TxpModel] object or [TxpModelList] object
#' @param input data.frame object containing the model input data
#' @param id.var Character scalar, column in 'input' to store in
#' @inheritParams TxpResultParam-class
#' @inheritParams txpGenerics
#'
#' @details
#' `txpCalculateScores` is implemented as an S4 generic function with methods
#' for [TxpModel] and [TxpModelList].
#'
#' Ranks are calculated such that the highest ToxPi score has a rank of 1.
#'
#' Missingness is determined after applying input-level transformations but
#' before applying slice-level transformations.
#'
#' @seealso [TxpModel], [TxpResult], [TxpResultParam]
#'
#' @template roxgn-loadExamples
#' @template roxgn-calcTxpModel
#' @template roxgn-calcTxpModelList
#'
#' @return [TxpResult] or [TxpResultList] object
#'
#' @export

NULL

.sumNA <- function(x) {
  if (all(is.na(x))) return(NA_real_)
  sum(x, na.rm = TRUE)
}

.z2o <- function(x) {
  (x - min(x, na.rm = TRUE))/diff(range(x, na.rm = TRUE))
}

.sumSlice <- function(slice, input, negative.value.handling) {
  # Applies input-level transformation functions and sums the values to give
  # a raw slice score
  nms <- txpValueNames(slice)
  dat <- input[nms]
  if (negative.value.handling == "missing") dat[dat < 0]  <- NA
  tfs <- txpTransFuncs(slice)
  for (i in seq_along(nms)) {
    if (is.null(tfs[[i]])) next
    dat[[i]] <- tfs[[i]](dat[[i]])
  }
  x <- apply(dat, MARGIN = 1, .sumNA)
  dat <- unlist(dat)
  y <- sum(!is.finite(dat)) / length(dat)
  list(sum = x, mis = y)
}

.calculateScores <- function(model, input,
                             id.var = NULL,
                             rank.ties.method = c("average", "first", "last",
                                                  "random", "max", "min"),
                             negative.value.handling = c("keep", "missing")) {

  ## Test inputs
  .chkModelInput(model = model, input = input)
  param <- TxpResultParam(rank.ties.method = rank.ties.method,
                          negative.value.handling = negative.value.handling)

  ## Clean up infinite in input
  input <- .rmInfinite(model = model, input = input)

  ## Calculate raw slice scores and missingness
  x <- lapply(
    txpSlices(model), .sumSlice, input = input,
    negative.value.handling = slot(param, "negative.value.handling"))
  slc <- sapply(x, "[[", "sum")
  mis <- sapply(x, "[[", "mis")

  ## Look for and apply slice-level transformation functions
  tfs <- txpTransFuncs(model)
  if (any(!sapply(tfs, is.null))) {
    for (i in 1:ncol(slc)) {
      if (is.null(tfs[[i]])) next
      slc[ , i] <- tfs[[i]](slc[ , i])
    }
  }

  ## Make infinite NaN
  slc[is.infinite(slc)] <- NaN

  ## Scale slice scores from 0 to 1
  slc <- apply(slc, 2, .z2o)

  ## Make NA 0
  slc[is.na(slc)] <- 0

  ## Calculate ToxPi score
  wts <- txpWeights(model, adjusted = TRUE)
  score <- rowSums(slc*rep(wts, each = NROW(slc)), na.rm = TRUE)

  ## Calculate ToxPi ranks
  rnks <- rank(-score, ties.method = rank.ties.method)

  ## Assign IDs
  ids <- if (!is.null(id.var)) as.character(input[[id.var]]) else NULL

  TxpResult(txpScores = score,
            txpSliceScores = slc,
            txpRanks = rnks,
            txpMissing = mis,
            txpModel = model,
            txpIDs = ids,
            txpResultParam = param)

}

##----------------------------------------------------------------------------##

#' @importFrom BiocGenerics updateObject
#' @importFrom BiocGenerics sort
#' @importFrom BiocGenerics duplicated
#' @importFrom BiocGenerics as.data.frame

#' @name txpGenerics
#' @title toxpiR package generics
#' @description toxpiR package generics; see class man pages for associated
#' methods
#' @param x toxpiR S4 object
#' @param value Replacement value
#' @param ... Included for extendability; not currently used
#'
#' @return See specific methods for details.

NULL

#' @rdname txpGenerics
setGeneric("txpValueNames", function(x, ...) standardGeneric("txpValueNames"))

#' @rdname txpGenerics
setGeneric("txpValueNames<-",
           function(x, ..., value) standardGeneric("txpValueNames<-"))

#' @rdname txpGenerics
setGeneric("txpTransFuncs", function(x, ...) standardGeneric("txpTransFuncs"))

#' @rdname txpGenerics
setGeneric("txpTransFuncs<-",
           function(x, ..., value) standardGeneric("txpTransFuncs<-"))

#' @rdname txpGenerics
setGeneric("txpSlices", function(x, ...) standardGeneric("txpSlices"))

#' @rdname txpGenerics
setGeneric("txpSlices<-",
           function(x, ..., value) standardGeneric("txpSlices<-"))

#' @rdname txpGenerics
setGeneric("txpWeights", function(x, ...) standardGeneric("txpWeights"))

#' @rdname txpGenerics
setGeneric("txpWeights<-",
           function(x, ..., value) standardGeneric("txpWeights<-"))

#' @rdname txpCalculateScores
setGeneric("txpCalculateScores",
           function(model, input, ...) standardGeneric("txpCalculateScores"))

#' @rdname txpGenerics
setGeneric("txpScores", function(x, ...) standardGeneric("txpScores"))

#' @rdname txpGenerics
setGeneric("txpSliceScores", function(x, ...) standardGeneric("txpSliceScores"))

#' @rdname txpGenerics
setGeneric("txpModel", function(x, ...) standardGeneric("txpModel"))

#' @rdname txpGenerics
setGeneric("txpIDs", function(x, ...) standardGeneric("txpIDs"))

#' @rdname txpGenerics
setGeneric("txpIDs<-", function(x, ..., value) standardGeneric("txpIDs<-"))

#' @rdname txpGenerics
setGeneric("txpRanks", function(x, ...) standardGeneric("txpRanks"))

#' @rdname txpGenerics
setGeneric("txpMissing", function(x, ...) standardGeneric("txpMissing"))

#' @rdname txpGenerics
setGeneric("txpResultParam", function(x, ...) standardGeneric("txpResultParam"))

##----------------------------------------------------------------------------##
## pieGrob
##----------------------------------------------------------------------------##

#' @name pieGrob
#' @title Create a pie grob
#' @description Create a pie grob
#' @param rads Numeric, radius values for each slice from 0 to 1
#' @param fills Colors to fill the slices
#' @param wts Numeric, the relative portion of each slice
#' @param name,vp,gp Passed to [grid::gTree]
#' 
#' @details 
#' The default coloring can be set with `options("txp.fills")`.
#' 
#' 
#' @examples
#' library(grid)
#' 
#' s <- seq(0.2, 1, by = 0.1)
#' grid.newpage()
#' grid.pieGrob(rads = s)
#' grid.newpage()
#' grid.pieGrob(rads = s, wts = s)
#' 
#' curr_txp_fills <- options()$txp.fills
#' options(txp.fills = 1:8)
#' grid.newpage()
#' grid.pieGrob(rads = s)
#' options(txp.fills = curr_txp_fills)
#' 
#' ## Can edit
#' grid.newpage()
#' grid.pieGrob(rads = s, name = "myPie")
#' grid.ls() ## show the grid elements
#' grid.edit("myPie", fills = 1:9, wts = 9:1)
#' 
#' @return `pieGrob` [grid::grob] object
#' 
#' @import grid 
#' @export

pieGrob <- function(rads, fills = NULL, wts = NULL, 
                    name = NULL, vp = NULL, gp = NULL) {
  pieVp <- makePieViewport()
  gTree(name = name,
        rads = rads,
        fills = fills,
        wts = wts,
        gp = gp,
        vp = vp,
        childrenvp = pieVp,
        children = makePieGrob(rads = rads, 
                               fills = fills, 
                               wts = wts, 
                               vp = pieVp),
        cl = "pieGrob")
}

#' @rdname pieGrob
#' @export

grid.pieGrob <- function(rads, fills = NULL, wts = NULL, 
                         name = NULL, vp = NULL, gp = NULL) {
  g <- pieGrob(rads = rads, 
               fills = fills, 
               wts = wts, 
               name = name,
               vp = vp,
               gp = gp)
  grid.draw(g)
}

#' @export

editDetails.pieGrob <- function(x, specs) {
  if (any(c("rads", "fills", "wts") %in% names(specs))) {
    newRads <- if (is.null(specs$rads)) x$rads else specs$rads
    newFills <- if (is.null(specs$fills)) x$fills else specs$fills
    newWts <- if (is.null(specs$wts)) x$wts else specs$wts
    x <- setChildren(x, 
                     makePieGrob(rads = newRads, 
                                 fills = newFills, 
                                 wts = newWts,
                                 vp = x$childrenvp))
  }
  x
}

makeSliceGrob <- function(rad, th0, th1, fill, name = NULL, vp = NULL) {
  th <- c(seq(th0, th1, by = pi/360), th1)
  x <- c(0, cos(th))*rad
  y <- c(0, sin(th))*rad
  polygonGrob(x = x, 
              y = y, 
              name = name, 
              gp = gpar(fill = fill, col = NA),
              default.units = "native",
              vp = vp)
}

makePieGrob <- function(rads, fills = NULL, wts = NULL, vp = NULL) {
  nSlices <- length(rads)
  if (is.null(wts)) wts <- rep(1, nSlices)
  wts <- wts/sum(wts)
  ths <- cumsum(c(0, 2*pi*wts))
  
  if (is.null(fills)) fills <- getOption("txp.fills", TXP_FILLS)
  if (nSlices > length(fills)) fills <- colorRampPalette(fills)(nSlices)
  if (nSlices < length(fills)) fills <- fills[1:nSlices]
  
  slices <- vector("list", nSlices)
  for (i in seq_along(rads)) {
    slices[[i]] <- makeSliceGrob(rad = rads[i], 
                                 th0 = ths[i], 
                                 th1 = ths[i + 1], 
                                 fill = fills[i], 
                                 name = gPath(sprintf("slice%s", i)),
                                 vp = vp)
  }
  do.call("gList", slices)
}

makePieViewport <- function() {
  vpStack(viewport(layout = grid.layout(nrow = 1, ncol = 1, respect = TRUE)),
          viewport(name = "pievp",
                   layout.pos.row = 1,
                   layout.pos.col = 1,
                   xscale = c(-1, 1),
                   yscale = c(-1, 1)))
}

##----------------------------------------------------------------------------##

##----------------------------------------------------------------------------##
## txpExportGui
##----------------------------------------------------------------------------##

#' @name txpExportGui
#' @title Export comma-separated file intended for ToxPi GUI
#' @description Export comma-separated file intended for ToxPi GUI
#' 
#' @param fileName Character scalar, the path to the output file
#' @inheritParams txpCalculateScores 
#' @inheritParams pieGrob
#' 
#' @details 
#' The GUI differs in two meaninful ways for exporting `toxpiR` models: (1) the
#' GUI only allows for integer weights; (2) the GUI applies transformation 
#' functions differently. 
#' 
#' `txpExporGui` will not work for models with non-integer weights.
#' 
#' The GUI only applies a single transformation function to every input within
#' a slice, and only functions from a pre-determined list; `toxpiR` allows 
#' users to apply any valid function individually to each input, then a second
#' transformation function on the summed slice values. Because of this 
#' complexity, any exported models with slice-level transformation functions
#' will not export at the input level. In other words, the export will have only
#' the final slice scores. Otherwise, all input-level transformations will be 
#' performed, the and the export will contain transformed input-level data with
#' the `linear(x)` GUI transformation.
#' 
#' @importFrom rlang is_scalar_character
#' @importFrom utils write.table
#' @export

txpExportGui <- function(fileName = "txpModel.csv", 
                         input, 
                         model, 
                         id.var = NULL,
                         fills = NULL) {
  
  ## TODO: fileName checks, can it be written? does it already exist? etc.
  
  stopifnot(is_scalar_character(fileName))
  
  ## Test inputs
  .chkModelInput(model = model, input = input)
  
  ## Clean up infinite in input
  input <- .rmInfinite(model, input)
  
  slcWts <- txpWeights(model)
  if (any(slcWts%%1 != 0)) {
    stop("ToxPi GUI only allows integer weights in the model.")
  }
  
  ## Check for slice-level transformations 
  tfs <- txpTransFuncs(model)
  if (any(!sapply(tfs, is.null))) {
    ## Output as completely transformed slice values
    warning("Model contains slice-level transformation; export will not ",
            "contain input-level data. See ?txpExportGui for more ",
            "information.")
    res <- .calculateScores(model = model, input = input)
    mat <- txpSliceScores(res)
    slcVec <- vnmVec <- colnames(mat)
  } else {
    ## Notes: may duplicate inputs because the same input in different slices
    ## can have different transformation functions
    vnmVec <-  txpValueNames(model, simplify = TRUE)
    slcVec <- names(model)
    vnmLst <-  txpValueNames(model)
    itfsLst <- txpTransFuncs(txpSlices(model))
    matLst <- list()
    for (i in seq_along(vnmLst)) {
      mat <- matrix(NA_real_, nrow = NROW(input), ncol = length(vnmLst[[i]]))
      for (j in seq_along(vnmLst[[i]])) {
        if (is.null(itfsLst[[i]][[j]])) {
          mat[ , j]  <- input[ , vnmLst[[i]][[j]]]
        } else {
          mat[ , j] <- itfsLst[[i]][[j]](input[ , vnmLst[[i]][[j]]])
        }
      }
      # Make sure transformed values are positive
      minMat <- min(mat[is.finite(mat)])
      if (minMat < 0) {
        x <- -floor(minMat)
        # If slices contain multiple components and any missing values, then those
        # missing values must be replaced with the added constant to produce the same
        # slice/toxpi scores because slice scores are computed by sum not mean
        # However, if all values are missing in a row, the leave it alone
        if (ncol(mat) > 1 & any(!is.finite(mat))) {
          idxNA <- apply(mat, 1, function(x) all(is.na(x)))
          mat[!idxNA & !is.finite(mat)] <- 0
          warning("Slice \"", slcVec[i], "\" contains both missing and negative values ",
                  "after applying transformations so missing values were replaced with 0 ",
                  "and then all values were increased by x = ", x, ".")
        } else {
          warning("Slice \"", slcVec[i], "\" contains negative values ",
                  "after applying transformations so all values were increased by x = ", x, ".")
        }
        # Shift all values by a constant to make them positive
        mat <- mat + x
      }
      matLst[[i]] <- mat
    }
    mat <- do.call(cbind, matLst)
  }
  
  ## Make infinite NaN
  mat[is.infinite(mat)] <- NaN
  
  ## Determine colors
  nSlices <- length(slcVec)
  if (is.null(fills)) fills <- getOption("txp.fills", TXP_FILLS)
  if (nSlices > length(fills)) fills <- colorRampPalette(fills)(nSlices)
  if (nSlices < length(fills)) fills <- fills[1:nSlices]
  
  fills <- .col2hex(fills)
  fills <- sub("^#", "0x", fills)
  
  ## Rename any duplicated column names
  vnmLst <- txpValueNames(model)
  if (any(duplicated(vnmVec))) {
    dup <- unique(vnmVec[duplicated(vnmVec)])
    for (i in seq_along(vnmLst)) {
      vnmLst[[i]] <- gsub(paste0('^(', paste(dup, collapse = '|'), ')$'), paste0('\\1_slice', i), vnmLst[[i]])
    }
    vnmVec <- unlist(vnmLst)
  }

  ## Prepare the header
  slcMeta <- paste(slcVec, slcWts, fills, "linear(x)", sep = "!")
  slcMeta <- paste("#", slcMeta)
  slcVnmInd <- vector(mode = "list", length = nSlices)
  names(slcVnmInd) <- slcVec
  for (i in slcVec) {
    slcVnmInd[[i]] <- rep('', length(vnmVec))
    slcVnmInd[[i]][vnmVec %in% vnmLst[[i]]] <- 'x' 
  }
  hdr <- cbind(slcMeta, do.call(rbind, slcVnmInd))
  
  ## Default names
  ids <- if (is.null(id.var)) 1:NROW(input) else input[[id.var]]
  
  ## Make final output
  out <- rbind(hdr, c('', vnmVec), cbind(ids, mat)) 
  
  ## Write csv file
  write.table(x = out, 
              file = fileName, 
              quote = FALSE, 
              sep = ",", 
              row.names = FALSE, 
              col.names = FALSE)
  
}

##----------------------------------------------------------------------------##

##----------------------------------------------------------------------------##
## methods-TxpTransFunc
##----------------------------------------------------------------------------##

#' @name TxpTransFunc-class
#' @title Numeric transformation function
#' @description S4 class to store numeric transformation functions
#'
#' @param x function, see details
#'
#' @details
#' \code{TxpTransFunc} inherits from a standard R function, but specifies a
#' single input and a numeric output of the same length.
#'
#' Functions can be passed directly to \code{TxpTransFuncList} list and the
#' functions will be coerced to \code{TxpTransFunc}.
#'
#' We have an imperfect system for dealing with primitive functions (e.g.,
#' [base::sqrt]).
#' To coerce primitives to TxpTransFunc's, we wrap them in another function
#' cal; wrapping the primitives obscures the original function and requires
#' the user to explore the function environment to understand the primitive
#' called.
#' We recommend wrapping primitives in separate functions to make the intent
#' clear, .e.g., `mysqrt <- function(x) sqrt(x)`.
#'
#' @examples
#' f1 <- function(x) "hello"
#' f2 <- function(x) 3
#' f3 <- function(x) x + 5
#' \dontrun{
#' t1 <- TxpTransFunc(x = f1) ## Produces error
#' t2 <- TxpTransFunc(x = f2) ## Produces error
#' }
#' t3 <- TxpTransFunc(x = f3)
#'
#' ## TxpTransFunc objects act as any other function
#' body(t3)
#' formals(t3)
#' t3(1:10)
#'
#' ## Coercion from functions
#' \dontrun{
#' TxpTransFuncList(f1, f2, f3) ## Produces error because f1, f3 not valid
#' }

NULL

##----------------------------------------------------------------------------##
## constructor

#' @rdname TxpTransFunc-class
#' @export

TxpTransFunc <- function(x) {
  if (missing(x)) return(new("TxpTransFunc"))
  if (is.primitive(x)) {
    somePrimitive <- x
    f <- .convertPrimitive(somePrimitive)
  }
  else f <- x
  new2("TxpTransFunc", f)
}

##----------------------------------------------------------------------------##
## validity

.TxpTransFunc.validity <- function(object) {
  msg <- NULL
  suppressWarnings({
    res1 <- try(object(1:5), silent = TRUE)
    res2 <- try(object(1:6), silent = TRUE)
  })
  if (is(res1, "try-error") || is(res2, "try-error")) {
    msg <- c(msg, "TxpTransFunc returned error when given numeric input.")
    return(msg)
  }
  if (length(res1) != 5 || length(res2) != 6) {
    msg <- c(msg, "TxpTransFunc output length must equal input length.")
  }
  if (!class(res1) %in% c("numeric", "integer")) {
    msg <- c(msg, "TxpTransFunc output must be numeric for numeric inputs.")
  }
  if (is.null(msg)) return(TRUE)
  msg
}

setValidity2(Class = "TxpTransFunc", method = .TxpTransFunc.validity)

##----------------------------------------------------------------------------##
## coercion

.TxpTransFunc.coerce.from.function <- function(from) {
  if (is.primitive(from)) {
    somePrimitive <- from
    f <- .convertPrimitive(somePrimitive)
  }
  else f <- from
  TxpTransFunc(f)
}

setAs("function", "TxpTransFunc", .TxpTransFunc.coerce.from.function)

##----------------------------------------------------------------------------##
## concatenation

## Close, but rearranges the elements inappropriately:
# f <- TxpTransFunc(f)
# c(a = f, b = f, f)
## ALSO, causes errors with the 'c' method for TxpTransFuncList that would
## need correction
# .TxpTransFunc.concatenate <- function(x, ...) {
#   lst <- if (missing(x)) list(...) else list(x, ...)
#   do.call("TxpTransFuncList", lst)
# }
# setMethod("c", "TxpTransFunc", .TxpTransFunc.concatenate)

##----------------------------------------------------------------------------##
## utilities
.convertPrimitive <- function(somePrimitive) {
  warning("Using primitive functions obscures behavior; ",
          "see ?TxpTransFunc for more details.")
  f <- function(y) somePrimitive(y)
  f <- .make_function(formals(f), .substitute_q(body(f), environment(f)))
}

##----------------------------------------------------------------------------##

##----------------------------------------------------------------------------##
## TxpTransFunc tests
##----------------------------------------------------------------------------##

##----------------------------------------------------------------------------##
## Initialization

test_that("We can create TxpTransFunc objects", {
  fx <- function(x) x + 1
  expect_s4_class(tf <- TxpTransFunc(fx), "TxpTransFunc")
  expect_s4_class(as(fx, "TxpTransFunc"), "TxpTransFunc")
  expect_condition(body(tf) == "x + 1", regexp = NA)
  expect_equal(formalArgs(tf), "x")
  expect_equal(tf(1:10), 1:10 + 1)
  expect_error(TxpTransFunc(function(x) "hello"))
  expect_error(TxpTransFunc(function(x) x + "a"))
  expect_error(TxpTransFunc(1))
})

test_that("TxpTransFunc can handle primitives", {
  expect_warning(f1 <- TxpTransFunc(sqrt))
  expect_equal(f1(1:10), sqrt(1:10))
  expect_warning(f2 <- as(sqrt, "TxpTransFunc"))
  expect_equal(f2(1:10), sqrt(1:10))
})

##----------------------------------------------------------------------------##
## TxpResult/txpCalculateScores tests
##----------------------------------------------------------------------------##

##----------------------------------------------------------------------------##
## txpCalculateScores

test_that("We can create TxpResult objects through txpCalculateScores", {
  data(txp_example_input, package = "toxpiR")
  data(txp_example_model, package = "toxpiR")
  expect_s4_class(res <- txpCalculateScores(model = txp_example_model, 
                                            input = txp_example_input, 
                                            id.var = "name"),
                  "TxpResult")
  inf_example <- txp_example_input
  inf_example["chem4", "metric1"] <- Inf
  expect_warning(inf_res <- txpCalculateScores(model = txp_example_model, 
                                                input = inf_example, 
                                                id.var = "name"))
  expect_s4_class(inf_res, "TxpResult")
  txpValueNames(txpSlices(txp_example_model)[[2]]) <- "notInput"
  expect_error(txpCalculateScores(model = txp_example_model, 
                                  input = txp_example_input))
  txp_example_input$notInput <- "hello"
  expect_error(txpCalculateScores(model = txp_example_model, 
                                  input = txp_example_input))
})

##----------------------------------------------------------------------------##
## Accessors

test_that("TxpResult accessors return expected slots", {
  data(txp_example_input, package = "toxpiR")
  data(txp_example_model, package = "toxpiR")
  expect_s4_class(res <- txpCalculateScores(model = txp_example_model, 
                                            input = txp_example_input, 
                                            id.var = "name"),
                  "TxpResult")
  expect_s4_class(txpModel(res), "TxpModel")
  expect_type(txpScores(res), "double")
  expect_type(txpIDs(res), "character")
  expect_equal(txpIDs(res), sprintf("chem%02d", 1:10))
  expect_type(txpSliceScores(res), "double")
  expect_true(is.matrix(txpSliceScores(res)))
  expect_equal(dim(txpSliceScores(res)), c(10, 4))
  expect_equal(rowSums(txpSliceScores(res, adjusted = TRUE)), txpScores(res))
  expect_equal(apply(txpSliceScores(res, adjusted = FALSE), 2, max), 
               c(s1 = 1, s2 = 1, s3 = 1, s4 = 1))
  expect_equal(txpRanks(sort(res)), 1:10)
  expect_equal(txpRanks(sort(res, decreasing = FALSE)), 10:1)
  expect_s4_class(txpSlices(res), "TxpSliceList")
  expect_length(txpSlices(res), 4)
  expect_equal(round(txpScores(res), 6),
               c(0.863316, 0.414845, 0.347997, 0.164044, 0.425231, 
                 0.585716, 0.000000, 0.719512, 0.771979, 0.470999))
  expect_equal(txpTransFuncs(res, level = "model"),
               txpTransFuncs(txpModel(res)))
  expect_equal(txpTransFuncs(res, level = "slices"),
               txpTransFuncs(txpSlices(txpModel(res))))
  expect_equal(txpTransFuncs(res, level = "slices", simplify = TRUE),
               txpTransFuncs(txpSlices(txpModel(res)), simplify = TRUE))
  expect_equal(txpValueNames(res), txpValueNames(txpSlices(txpModel(res))))
  expect_equal(txpValueNames(res, simplify = TRUE), 
               txpValueNames(txpSlices(txpModel(res)), simplify = TRUE))
  expect_type(txpMissing(res), "double")
  expect_equal(length(txpMissing(res)), length(txpSlices(res)))
  expect_true(all(txpMissing(res) >=0 & txpMissing(res) <=1))
  expect_equal(txpMissing(res), c(s1 = 0.1,s2 =0.1,s3 =0.125,s4 =0.1))
})

##----------------------------------------------------------------------------##
## Replacement

test_that("We can replace TxpResult names/txpIDs", {
  expect_silent({
    data(txp_example_input, package = "toxpiR")
    data(txp_example_model, package = "toxpiR")
    res <- txpCalculateScores(model = txp_example_model, 
                              input = txp_example_input, 
                              id.var = "name")
    oldNms <- names(res)
    newNms <- as.character(sprintf("new%02d", 1:10))
  })
  expect_named({names(res) <- newNms; res}, newNms)
  expect_named({txpIDs(res) <- oldNms; res}, oldNms)
  expect_named({txpIDs(res)[1] <- "hello"; res[1]}, "hello")
  expect_named({names(res)[8:9] <- newNms[8:9]; res[8:9] }, newNms[8:9])
  expect_error(names(res) <- letters)
})

##----------------------------------------------------------------------------##
## Subsetting

test_that("TxpResult accessors return expected slots", {
  expect_silent({
    data(txp_example_input, package = "toxpiR")
    data(txp_example_model, package = "toxpiR")
    res <- txpCalculateScores(model = txp_example_model, 
                              input = txp_example_input, 
                              id.var = "name")
  })
  expect_s4_class(res[1], "TxpResult")
  expect_length(res[1], 1)
  expect_named(res[1], "chem01")
  expect_s4_class(res[c(rep(TRUE, 4), rep(FALSE, 6))], "TxpResult")
  expect_length(res[c(rep(TRUE, 4), rep(FALSE, 6))], 4)
  expect_named(res[c(rep(TRUE, 4), rep(FALSE, 6))], sprintf("chem%02d", 1:4))
  expect_s4_class(res[c("chem04", "chem08")], "TxpResult")
  expect_length(res[c("chem04", "chem08")], 2)
  expect_named(res[c("chem04", "chem08")], c("chem04", "chem08"))
  expect_error(res[25])
  expect_warning(expect_length(res[c(TRUE, FALSE)], 5))
  expect_length(res["notAName"], 0)
  expect_silent(names(res) <- NULL)
  expect_error(res["hello"])
})

##----------------------------------------------------------------------------##
## Coercion

test_that("We can coerce TxpResult to data.frame", {
  expect_silent({
    data(txp_example_input, package = "toxpiR")
    data(txp_example_model, package = "toxpiR")
    res <- txpCalculateScores(model = txp_example_model, 
                              input = txp_example_input, 
                              id.var = "name")
  })
  expect_s3_class(as.data.frame(res), "data.frame")
  expect_equal(dim(as.data.frame(res)), c(10, 7))
  expect_named(as.data.frame(res), 
               c("id", "score", "rank", sprintf("s%d", 1:4)))
  expect_named(as.data.frame(res, 
                             id.name = "a", 
                             score.name = "b", 
                             rank.name = "c"), 
               c("a", "b", "c", sprintf("s%d", 1:4)))
  txpIDs(res) <- NULL
  expect_warning(woID <- as.data.frame(res))
  expect_s3_class(woID, "data.frame")
  expect_named(woID, c("score", "rank", sprintf("s%d", 1:4)))
})

##----------------------------------------------------------------------------##
## Show

test_that("TxpResult show method displays correct information", {
  expect_silent({
    data(txp_example_input, package = "toxpiR")
    data(txp_example_model, package = "toxpiR")
    res <- txpCalculateScores(model = txp_example_model, 
                              input = txp_example_input, 
                              id.var = "name")
  })
  expect_output(print(res), "TxpResult of length 10")
  expect_output(print(res), "chem01 chem02 ... chem09 chem10")
})

##----------------------------------------------------------------------------##
## Plot -- TxpResult, missing

test_that("We can make and edit ToxPi diagrams", {
  expect_silent({
    data(txp_example_input, package = "toxpiR")
    data(txp_example_model, package = "toxpiR")
    res <- txpCalculateScores(model = txp_example_model, 
                              input = txp_example_input, 
                              id.var = "name")
  })
  expect_silent(plot(res))
  expect_silent(grid.edit("pie-1", fills = NULL))
  grid.edit("pie-10::slice1", gp = gpar(fill = "#7DBC3D"))
  expect_silent(plot(res, package = "gg"))
  expect_silent(plot(res, package = "gg",fills = c("red","blue","green","magenta")))
  expect_silent(plot(res, package = "gg",showScore = FALSE))
  expect_silent(plot(res, package = "gg",ncol = 2))
  expect_silent(plot(res, package = "gg",bgcolor = "white"))
  expect_silent(plot(res, package = "gg",sliceBorderColor = NULL))
  expect_silent(plot(res, package = "gg",sliceValueColor = "#FF00FF",))
  expect_silent(plot(res, package = "gg",sliceLineColor = "#FF00FF"))
  expect_silent(plot(res, package = "gg",showMissing = FALSE))
  expect_silent(plot(res, package = "gg",showCenter = FALSE))
})

##----------------------------------------------------------------------------##
## Plot -- TxpResult, numeric

test_that("We can make ToxPi rank plot ", {
  expect_silent({
    data(txp_example_input, package = "toxpiR")
    data(txp_example_model, package = "toxpiR")
    res <- txpCalculateScores(model = txp_example_model, 
                              input = txp_example_input, 
                              id.var = "name")
  })
  expect_silent(plot(res, txpRanks(res)))
  expect_silent(plot(res, txpRanks(res), labels = 1:10))
})

---
title: "Export ToxPi GUI Files"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Export ToxPi GUI Files}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---
<style type="text/css">
.small-text {
  font-size: 10px;
}
</style>

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

This vignette will show how to save a toxpiR model that will be compatible with the ToxPi Java GUI, which can be downloaded from [here](https://toxpi.github.io/). The toxpiR package and ToxPi Java GUI are not directly compatible and there are several key differences to keep in mind.

## Key differences between Java GUI and toxpiR

### Slice weights

The Java GUI only allows weights that are either integers or a ratio of integers whereas the toxpiR package has no restrictions. The `txpExportGui()` function requires all weights to be integers, so the user may need to change the model weights to acceptable approximations prior to calling the export function.

### Transformation/scaling functions

Currently the Java GUI only allows specific scaling functions and applies them independently to every input within a slice. The toxpiR package allows user-defined transformation functions at the input-level and slice-level. To account for these differences, all input-level transformation functions are applied before the data is exported. If slice-level transformations are applied, then the export function will create a data file that has the final slice scores rather than input-level data.

The Java GUI does not allow negative input values and will treat them a missing data. This causes a problem if negative values exist after applying any user-defined transformations. If negative values occur within a slice, then all values of that slice will be shifted up by a constant so that no negative values remain. If a slice has both negative transformed values and missing values, then missing values are replaced with the added constant. In this last case, the toxpi and slice scores will be computed correctly, however, the Java GUI should not be used to compute bootstrapped confidence intervals because replacing missing data during the export process will cause the resampling step to be incorrect.

### Metrics in multiple slices

The Java GUI does not allow multiple columns to have the same name, unless the data in those columns matches exactly. If a toxpiR model includes an input column in multiple slices, then the name will be appended with the slice index for each occurrence.

## Example use

First create a toxpiR model with accompanying data. Here we'll load the "Format C" data example using the `txpImportGui()` function.

```{r}
library(toxpiR)

# Load example model from "Import ToxPi GUI Files" vignette
data_format_C <- tempfile()
download.file(
  url = "https://raw.githubusercontent.com/ToxPi/ToxPi-example-files/main/format_C.csv",
  destfile = data_format_C,
  quiet = TRUE
)
gui1 <- txpImportGui(data_format_C)
```

Now we can use to export function to create a new data file. Notice the warnings for negative and missing values.

```{r}
# Export back into GUI format
data_exported <- tempfile()
txpExportGui(
  fileName = data_exported,
  input = gui1$input,
  model = gui1$model,
  id.var = 'Name',
  fills = gui1$fills
)
```

### Compare the data files

Take a moment to observe differences between the original data file (`data_format_C`) and the exported version (`data_exported`).

**`data_format_C`**

```{r echo=FALSE}
knitr::kable(read.csv(data_format_C, header = FALSE, stringsAsFactors = FALSE))
```

**`data_exported`**

<div style="white-space: nowrap">
```{r echo=FALSE}
df <- read.csv(data_exported, header = FALSE, stringsAsFactors = FALSE)
df[6:nrow(df), 2:ncol(df)] <- format(as.numeric(as.matrix(df[6:nrow(df), 2:ncol(df)])), digits = 4)
knitr::kable(df)
```
</div>

### Compare results

Although the data files are visually different, they will result in the same toxpi and slice scores.

```{r}
gui2 <- txpImportGui(data_exported)

res1 <- txpCalculateScores(gui1$model, gui1$input)
res2 <- txpCalculateScores(gui2$model, gui2$input)

all.equal(
  txpScores(res1),
  txpScores(res2)
)

all.equal(
  txpSliceScores(res1),
  txpSliceScores(res2)
)
```

---
title: "Import ToxPi GUI Files"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Import ToxPi GUI Files}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

This vignette will show how to load a data file that was saved using the ToxPi Java GUI, which can be downloaded from [here](https://toxpi.github.io/). The ToxPi Java GUI will save data files using file format "C" described in the ToxPi User Manual. This vignette will use the "format_C.csv" file to demonstrate how to import GUI data.

```{r setup}
library(toxpiR)

## Create a tempfile and download 'format_C.csv'
fmtc <- tempfile()
ghuc <- "https://raw.githubusercontent.com"
fmtcUrl <- file.path(ghuc, "ToxPi", "ToxPi-example-files", "main", "format_C.csv")
download.file(url = fmtcUrl, destfile = fmtc, quiet = TRUE)
```

The "format_C.csv" model specification reuses metrics across different slices. 
In general, we do not recommend duplicating inputs across slices, so the user gets a warning when creating a model with duplicate inputs.

```{r}
## Import file into R
gui <- txpImportGui(fmtc)

```

The resulting `list` object contains: `$model`, a `TxpModel` object with the model specifications; `$input`, a `data.frame` containing the data for calculating ToxPi scores; and `$fills`, an array of slice colors for plotting. 

```{r}
gui$model
gui$input
gui$fills
```

We calculate ToxPi scores using the `txpCalculateScores` function, which takes a model and input `data.frame`.
Note that by default the ToxPi GUI does not accept negative values. However, the package keeps them by default. 
To replicate the GUI functionailty, we set `negative.value.handling = "missing"`. 

```{r}
## Calculate ToxPi scores
res <- txpCalculateScores(model = gui$model, input = gui$input, id.var = "Name",negative.value.handling = "missing")

## Overall ToxPi scores
txpScores(res)

## Slice scores
txpSliceScores(res, adjusted = FALSE)
```

A results output similar to that given by the Java GUI can be obtained by combining score components.

```{r}
out <- as.data.frame(res, adjusted = FALSE)
out <- out[order(out$score, decreasing = TRUE), ]
out
```

ToxPi images and overall score rank plot can also be produced.

```{r fig.width = 7}
plot(sort(res), fills = gui$fills)
```

```{r fig.width = 7, fig.height = 4}
plot(res, txpRanks(res))
plot(res, txpRanks(res), labels = 1:10, pch = 16, size = grid::unit(0.75, "char"))
```

The basic clustering methods offered in the Java GUI can also be recreated.

```{r fig.width = 7, fig.height = 5}
## Hierarchical Clustering
hc <- hclust(dist(txpSliceScores(res)), method = 'complete')
plot(hc, hang = -1, labels = txpIDs(res), xlab = 'Name', sub = '')
```

```{r fig.width = 7, fig.height = 5}
## K-Means Clustering, plotted using principal components
nClusters <- 3
km <- kmeans(txpSliceScores(res), nClusters)
pc <- prcomp(txpSliceScores(res))
coord <- predict(pc) * -sum(txpWeights(res))
plot(coord[,1], coord[,2], col = km$cluster, 
     xlab = 'PC1', ylab = 'PC2', pch = 16)
```

---
title: "toxpiR Introduction"
output: 
  rmarkdown::html_vignette:
    fig_caption: yes
vignette: >
  %\VignetteIndexEntry{toxpiR Introduction}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---
<style type="text/css">

h1.title {
  font-size: 28px;
}
h1 { /* Header 1 */
  font-size: 20px;
}
h2 { /* Header 2 */
    font-size: 20px;
}
h3 { /* Header 3 */
  font-size: 18px;
}
h4 { /* Header 4 */
  font-size: 16px;
   color: DarkBlue;
}
code.r{ /* Code block */
    font-size: 12px;
}
</style>


```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.retina = 6
)
```

## Introduction

This document introduces ToxPi and describes how to use the `toxpiR` package to easily import, recombine, analyze, and visualize high dimensional data. The `toxpiR` package is an R implementation of ToxPi that offers new features over what was previously available for data handling, recombination, and customization; provides formally packaged, open-source code for ToxPi; extends the application domain by supporting rapid analysis of massive datasets; and bridges with the stand-alone, [Graphical User Interface (GUI)](https://toxpi.github.io/) Java application and [ArcGIS Toolkit](https://github.com/ToxPi/ToxPi-GIS). 

## What is ToxPi?

The Toxicological Prioritization Index (ToxPi) is a statistical modeling framework that allows transparent integration and visualization of data across disparate sources (i.e. multimodal or multiscale information). This framework aggregates related features into scored ‘slices’, rescales individual slice scores to range 0–1, and then develops an overall score for each sample using a weighted model. The slice weights can be interpreted as the "importance" of categorized features when predicting sample scores and associated ranks. The resulting scores for each sample are visualized as ToxPi profiles (see image below). Slice weights are represented as the arc-width, slice scores are represented as the radius, and the overall sample scores are represented by the combination of slice weights and radii. These visualizations allow for quick comparison of overall feature importance, feature impacts driving a specific sample, relative impact ranking of common features between samples, and overall ranking between samples. As a flexible method capable of integrating data from any source, it has been applied to problems in several different fields. Additional information is linked from [toxpi.github.io](https://toxpi.github.io/).   

<center>
![](embeddedFigures/txp_explain1.png){width=700px}

</center>

## Outline

There are three main steps in using this package: 

- **Loading data**: Users can load comma-separated value (csv) formatted data where data features that will make up the slices are stored as columns and attributes are rows. If you are more famililar with the GUI data format, the package also has a function to load data that is formatted in any one of the GUI layouts where slice information is presented in rows beginning with a hash tag(#). However, new users should use a simple .csv file with the row and column layout.  

- **Creating ToxPi Model**: ToxPi model can be created by specifying information about each slice and its corresponding weights. Each slice can be made up of one or more features from the input data. You should also name the slices and provide their weights in this step.

- **Computing ToxPi Scores**: Scores can be computed per attribute as well as for each slice. The S4 object resulting from this step will combine and store the ToxPi model and scores all in one place. This final object will contain all the information needed for plotting. 

Data contained within different slots of the S4 objects created using this package can be obtained using *accessor functions*. More details on how this can be done is present later in the document. 


## Installation

### **Standard installation**

You can install `toxpiR` from [CRAN](https://cran.r-project.org/web/packages/available_packages_by_name.html) with:

```{r, eval = FALSE}
install.packages("toxpiR")
```

### **Installation from GitHub**

Any new features will be first added to the GitHub version of the package. However, it may be less stable than the release on CRAN. Before attempting a direct installation from GitHub, please make sure you have `devtools` package on your system. 
Note, Windows users will need Rtools to install directly from GitHub. 
 
```{r, eval = FALSE}
remotes::install_github("ToxPi/toxpiR")
```

## **Launching package in R**

```{r packages, warning=FALSE}
library(toxpiR)
```


## Flowchart 
<center>
![](embeddedFigures/toxpiR-flowchart.svg){height=100% width=100%}
</center>

This flowchart details the basic steps necessary to analyze data using `toxpiR`. A separate vignette describes how to load and work with the GUI data file using txpImportGui() function. Usage for all other functions is described below using an example data set. To view a list of all functions available in this package, please run:  
  
> lsf.str("package:toxpiR")   

## Example usage

Data can either be entered as a regular csv file without the header lines specifying slice information, or in one of the GUI input formats. If you are using the GUI format, please refer to the [Import ToxPi GUI files](https://cran.r-project.org/package=toxpiR/vignettes/importFromGui.html) vignette. 

The standard data input is a standard R dataframe with rows as your datapoints and columns as features that may be turned into ToxPi slices. A simple example data set called *txp_example_input* is available within the package. We will use this 10 x 8 toy data to walk through a simple analysis and explain the functions available in this package. These metrics that we have specified can be in completely different scales and data type, but first we will show a simple example of the package usage.


### Load the dataset

```{r}
data(txp_example_input, package = "toxpiR")
head(txp_example_input)
```

### Specify slice information 

The first step is to specify information about each slice before creating a Toxpi model. This can be done using the **TxpSliceList()** and **TxpSlice()** functions. These slices can be made up of one or more features from the input dataset and are specified by listing the features that go into each slice in the **TxpSlice()** function. We can also specify how we want to transform the slices with the  **TxpTransFuncList()** function. This means we can do a different transformation per slice instead of having to transform the entire input dataset. 


#### TxpSlice(txpValueNames, txpTransFuncs)

>Store individual slice objects. 

**Arguments**

> *txpValueNames* : character vector indicating which data to include in the slice. Can just be the name of a single column if that is all the slice contains.

> *txpTransFuncs* : List of transformation functions to use with one function per entry in txpValueNames. Can be created using TxpTransFuncList() method. Defaults to NULL. 

#### TxpSliceList()

>Method to merge all individuals slices together and assign slice names.

**Basic Usage**



```{r}

## Goal - Create two slices with transformation functions 
# Slice 1: Metric 1, No transformation 
# Slice 2: Metric 2 (square) and Metric 3 (no transformation)

slice2.trans <- TxpTransFuncList(func1 = function(x) x^2, func2 = NULL)

f.slices <- TxpSliceList(Slice1 = TxpSlice("metric1"), 
                         Slice2 = TxpSlice(c("metric2", "metric3"), 
                                           txpTransFuncs = slice2.trans ))

```

### Create ToxPi model

The ToxPi model object is core for specifying how the created slices should behave when the scores are calculated. In this step, we need to specify what the slices we created above are and assign those slices weights. We can also assign transformation functions at this step. However, it is not necessary if you have already done so when creating the slices. The model is a core component of the analysis because it is going to specify how our calculations are performed. 

#### TxpModel()

**Arguments**

> *txpSlices* : TxpSliceList() object.

> *txpWeights* : numeric vector specifying weight of each slice. By default, all slices are weighted equally (1). 

> *txpTransFuncs* : TxpTransFuncList object. Defaults to NULL, if a list is provided, its length must be equal to the number of slices present.

> adjusted : Scalar logical to adjust returned weights such that they sum to 1. 


**Usage**

```{r}

## Goal - Create ToxPi model.
# Slice 1, weight = 2
# Slice 2, weight = 1, apply log transform to final value. 

# Object storing list of transformation functions.
final.trans <- TxpTransFuncList(f1 = NULL, f2 = function(x) log10(x)) 

f.model <- TxpModel(txpSlices = f.slices, 
                    txpWeights = c(2,1),
                    txpTransFuncs = final.trans)

```

### Calculate ToxPi scores

Calculate ToxPi Scores for the given ToxPi model and input data. This input data is what we read in previously from either a standard .csv file or a GUI compatible one. This step uses the model we have created to specify how the ToxPi scores should be calculated. The results object that is created also stores the model used, allowing for easy reference when you are dealing with many models and results.

#### txpCalculateScores()

**Arguments**

> *model* : ToxPi model object created using TxpModel() function.

> *input* : dataframe containing input data for ToxPi model. 

> *id.var* : Character scalar, column in 'input' to store in. Deafults to NULL.

> *rank.ties.method* : character string specifying how ties are treated. Available options "average", "first", "last", "random", "max" and "min". Pased to base::rank function.

> *negative.value.handling* : character string specifying how negative values are treated. Available options are "keep" and "missing". Defaults to keep. 

**Usage**


```{r}
f.model <- txp_example_model #Load a more complex 4 slice model
f.results <- txpCalculateScores(model = f.model,
                                input = txp_example_input,
                                id.var = 'name' ) 

txpSliceScores(f.results) #ToxPi scores
txpWeights(f.results) #Print weights
txpMissing(f.results) #Proportion of missing data within each slice
```


### Visualization

Two paths exist for visualization, **ggplot** methods and **grid** methods. The ggplot method uses the `ggplot2` package and allows for highly customizable profiles allowing for several visualization experiences. The grid method uses advanced `grid` graphics and can be implemented using pieGrob objects that are compatible with the grid system. Additional plots can be made such as a rank plot by using the data stored in the results object. The results object also allows for the flexibility to use other R plotting packages if preferred. 

<br>
**Summary**: The ggPlot graphics path will be the best option for users wanting to create ToxPi profiles, with many options for customization. The grid graphics path is for users wanting to modify very specific, basic elements of ToxPi profiles and/or recombine profiles with other graphics. The `ggplot2` package (high-level) is built atop `grid` graphics (low-level), so the two systems interact well.

:::: {style="display: flex; justify-content:space-around;"}

<div >
<h3>ggPlot</h3>
* Examples shown in this vignette produce a "look-and-feel" that is closer to the graphics output of the familiar [ToxPi GUI](https://toxpi.github.io/)
* Aesthetics are easy to customize 
* Includes missing data visuals (center circle à la ToxPi GUI output)
</div>

<div >
<h3>grid</h3>
* Formal, S4-based customization options for visuals using the R `grid` package and the "grob" (grid graphical object) framework
* ToxPi profile graphics created as grobs can be directly inserted into more complex figures as points on a scatterplot, leaves on a cluster dengrogram, overlaid on a map, etc.
* Vignette examples show how to highlight individual profiles and highlight individual slices 
</div>

::::

#### ggplot Method

This method uses the `ggplot2` package to draw highly customizable ToxPi profiles with multiple different aesthetics and information options. The most notable aesthetic addition is the inclusion of information regarding the amount of missing data each slice has, which is represented by the a small circle of interior slices as is shown in profiles generated by the ToxPi GUI. Each interior slice is colored on a grey scale, with black representing all missing data and white representing no missing data. Each new customization option, along with some other notable differences, are described below. 

<br>
**Customization Options (Arguments)**

> *fills* : Vector containing slice colors. Defaults to a color scheme matching the ToxPi GUI.

> *showMissing* : Boolean to show missing data information. Defaults to TRUE. 

> *showCenter* : Boolean to show inner circle. Defaults to TRUE. FALSE overrides showMissing.

> *showScore* : Boolean to show overall profile scores. Defaults to TRUE.

> *ncol* : Integer specifying number of columns in plot. Default is an automatic calculation by ggplot. 

> *bgColor* : Text specifying background color for profiles as either hex codes or R recognized colors. Defaults to "gray80".

> *borderColor* : Text specifying color of max radius ring as either hex codes or R recognized colors. Defaults to "white".  

> *sliceBorderColor* : Text specifying color of slice borders as either hex codes or R recognized colors. Defaults to "white".  

> *sliceValueColor* : Text specifying color of slice scores as either hex codes or R recognized colors. Defaults to NULL.  

> *sliceLineColor* : Text specifying color of slice guidelines as either hex codes or R recognized colors. Defaults to NULL.  

<br>
**Usage**

Depending on export method, plot resolution may end up poor. SVG and PDF images tend to produce better results than PNG. Usage and example plot output are shown below.
```{r, eval=FALSE}
library("ggplot2")

# Default plot
plot(f.results, package = "gg") # Using ggplot package
```
<center>
```{r, fig.show='hold', fig.width=4, fig.height=4, echo=FALSE}
library("ggplot2")

# Default plot
plot(f.results, package = "gg") # Using ggplot package
```
</center>

Sometimes the inner circle of the ToxPi figure can show plotting artifacts. A possible workaround for this is to create a wrapper function around `ggplot2:::coord_munch` as shown below.

```{r, fig.show = "hide"}
## Optional code to make smoother lines

# Plot before updating coord_munch
plot(f.results["chem01"], package = "gg") 

# Save the original version of coord_munch
coord_munch_orig <- ggplot2:::coord_munch

# Make a wrapper function that has a different default for segment_length
if (length(formals(coord_munch_orig)) == 5) {
  coord_munch_new <- function(coord, data, range, segment_length = 1/1000,
                              is_closed = FALSE) {
    coord_munch_orig(coord, data, range, segment_length, is_closed)
  }
} else {
  coord_munch_new <- function(coord, data, range, segment_length = 1/1000) {
    coord_munch_orig(coord, data, range, segment_length)
  }
}

# The environment may need to be set
#environment(coord_munch_new) <- environment(coord_munch_orig)

# Replace ggplot2:::coord_munch with coord_munch_new
assignInNamespace("coord_munch", coord_munch_new, ns = "ggplot2")

# Plot after updating coord_munch
plot(f.results["chem01"], package = "gg") 

# Revert to original coord_munch ater plotting if desired 
#assignInNamespace("coord_munch", coord_munch_orig, ns = "ggplot2")
```

<center>
![](embeddedFigures/coord_munch_orig.png)
![](embeddedFigures/coord_munch_new.png)
</center>

<br>
**Customization examples**

#### Coloration Options

```{r, eval=FALSE}
# Changing the slice colors
colors <- c("orange", "green", "magenta", "lightblue")
plot(f.results["chem02"], package = "gg", fills = colors)
plot(f.results["chem02"], package = "gg", fills = NULL)
```
<center>
```{r, fig.show="hold", echo=FALSE}
# Changing the slice colors
colors <- c("orange", "green", "magenta", "lightblue")
plot(f.results["chem02"], package = "gg", fills = colors)
plot(f.results["chem02"], package = "gg", fills = NULL)
```
</center>

```{r, eval=FALSE}
# Changing the background color
plot(f.results["chem02"], package = "gg", bgColor = "lightskyblue")
plot(f.results["chem02"], package = "gg", bgColor = NULL)
```
<center>
```{r, fig.show="hold", echo=FALSE}
# Changing the background color
plot(f.results["chem02"], package = "gg", bgColor = "lightskyblue")
plot(f.results["chem02"], package = "gg", bgColor = NULL)
```
</center>

```{r, eval=FALSE}
# Changing the max radius ring color
plot(f.results["chem02"], package = "gg", borderColor = "black")
plot(f.results["chem02"], package = "gg", borderColor = NULL)
```
<center>
```{r, fig.show="hold", echo=FALSE}
# Changing the max radius ring color
plot(f.results["chem02"], package = "gg", borderColor = "black")
plot(f.results["chem02"], package = "gg", borderColor = NULL)
```
</center>

```{r, eval=FALSE}
# Changing the slice border color
plot(f.results["chem02"], package = "gg", sliceBorderColor = "magenta")
plot(f.results["chem02"], package = "gg", sliceBorderColor = NULL)
```
<center>
```{r, fig.show="hold", echo=FALSE}
# Changing the slice border color
plot(f.results["chem02"], package = "gg", sliceBorderColor = "magenta")
plot(f.results["chem02"], package = "gg", sliceBorderColor = NULL)
```
</center>

```{r, eval=FALSE}
# Adding slice guidelines 
plot(f.results["chem02"], package = "gg", sliceLineColor = "red")
plot(f.results["chem02"], package = "gg", sliceLineColor = NULL) 
```
<center>
```{r, fig.show="hold", echo=FALSE}
# Adding slice guidelines 
plot(f.results["chem02"], package = "gg", sliceLineColor = "red")
plot(f.results["chem02"], package = "gg", sliceLineColor = NULL) 
```
</center>

```{r, eval=FALSE}
# Adding visible slice scores
plot(f.results["chem02"], package = "gg", sliceValueColor = "brown")
plot(f.results["chem02"], package = "gg", sliceValueColor = NULL)
```
<center>
```{r, fig.show="hold", echo=FALSE}
# Adding visible slice scores
plot(f.results["chem02"], package = "gg", sliceValueColor = "brown")
plot(f.results["chem02"], package = "gg", sliceValueColor = NULL)
```
</center>

#### Binary Options
```{r, eval=FALSE}
# Hiding inner circle
plot(f.results["chem02"], package = "gg", showCenter = TRUE)
plot(f.results["chem02"], package = "gg", showCenter = FALSE)
```
<center>
```{r, fig.show="hold", echo=FALSE}
# Hiding inner circle
plot(f.results["chem02"], package = "gg", showCenter = TRUE)
plot(f.results["chem02"], package = "gg", showCenter = FALSE)
```
</center>

```{r, eval=FALSE}
# Hiding missing data information (pure white inner circle)
plot(f.results["chem02"], package = "gg", showMissing = TRUE)
plot(f.results["chem02"], package = "gg", showMissing = FALSE)
```
<center>
```{r, fig.show="hold", echo=FALSE}
# Hiding missing data information (pure white inner circle)
plot(f.results["chem02"], package = "gg", showMissing = TRUE)
plot(f.results["chem02"], package = "gg", showMissing = FALSE)
```
</center>

```{r, eval=FALSE}
# Hiding the overall profile scores
plot(f.results["chem02"], package = "gg", showScore = TRUE)
plot(f.results["chem02"], package = "gg", showScore = FALSE) 
```
<center>
```{r, fig.show="hold", echo=FALSE}
# Hiding the overall profile scores
plot(f.results["chem02"], package = "gg", showScore = TRUE)
plot(f.results["chem02"], package = "gg", showScore = FALSE) 
```
</center>

#### Format/Theme Options

```{r, fig.width = 7, fig.align='center'}
# Specifying the number of columns in the plot
plot(f.results, package = "gg", ncol = 5) 
```

```{r, eval=FALSE}
# Moving the legend using ggplot built in theme functions
plot(f.results, package = "gg", ncol = 5) + theme(legend.position = "bottom") 
plot(f.results, package = "gg", ncol = 2) + theme(legend.position = "left")
```
<center>
```{r, fig.show="hold", fig.height=5, echo=FALSE}
# Moving the legend using ggplot built in theme functions
plot(f.results, package = "gg", ncol = 5) + theme(legend.position = "bottom") 
plot(f.results, package = "gg", ncol = 2) + theme(legend.position = "left")
```
</center>

```{r, fig.align='center'}
# Removing plot margins
plot(f.results, package = "gg") + theme(plot.margin = margin(0, 0, 0, 0, "cm"))
# Removing spacing between panels
plot(f.results, package = "gg") + theme(panel.spacing = unit(0, "lines"))
# Removing text labels
plot(f.results, package = "gg") + theme(strip.text.x = element_blank())
# A combination of the above for a rank ordered plot
plot(f.results[order(txpRanks(f.results)[1:9])], package = "gg") +
  theme(
    plot.margin = margin(0, 0, 0, 0, "cm"),
    panel.spacing = unit(0, "lines"),
    strip.text.x = element_blank(),
    legend.position = "none"
  )
```

**Missing Data Examples**
```{r, fig.align='center'}
## Creating an example with more variable missing data amounts
f.input <- txp_example_input

# Add more missing data to slice 2 via metric3 in the raw data
f.input[3:10, "metric3"] <- NA

# Modify transformation function for slice 4
txpSlices(f.model)[[4]] <- TxpSlice("metric8", c(fn = \(x) sqrt(x - 30)))

# Calculate new ToxPi results
f.results_missing <- txpCalculateScores(f.model, f.input, id.var = "name")

# View missing data proportions
txpMissing(f.results_missing)

# View new result profile
plot(f.results_missing["chem02"], package = "gg")
```


#### pieGrob Method
This method is the default and uses the grid package with grob objects to draw ToxPi profiles that can be individually customized after plotting. This method does not allow for the wide array of aesthetics as ggplot does, but instead it allows the user to highlight specific profiles or slices that have significant importance in their results. Methods for highlighting information after plotting is shown below.
```{r fig.width=7, fig.height=3}
library(grid) # Load library
plot(f.results) # ToxPi visuals
# grid.ls() #List grid info

# Highlight one figure using its label
grid.edit("pie-1", fills = c("red", "blue", "black", "brown"))

# Or just one slice in a figure
grid.edit("pie-10::slice1", gp = gpar(fill = "#FC0FC0"))
```

#### ggplot vs grid Comparison

```{r, eval=FALSE}
#Single sample
plot(f.results["chem02"])
plot(f.results["chem02"], package = "gg") 
```
<center>
```{r, fig.show="hold", echo=FALSE}
#Single sample
plot(f.results["chem02"])
plot(f.results["chem02"], package = "gg") 
```
</center>

```{r, eval=FALSE}
# Subset plots
plot(f.results[order(txpRanks(f.results))[1:4]]) #Profiles ranked 1-4
plot(f.results[order(txpRanks(f.results))[1:4]], package = "gg") #Profiles ranked 1-4
```
<center>
```{r, fig.show="hold", echo=FALSE}
# Subset plots
plot(f.results[order(txpRanks(f.results))[1:4]]) #Profiles ranked 1-4
plot(f.results[order(txpRanks(f.results))[1:4]], package = "gg") #Profiles ranked 1-4
```
</center>

```{r, eval=FALSE}
## Long sample names for cramped plots

#change the first sample name in f.results
txpIDs(f.results)[1] <- "I am a long sample name"

plot(f.results) #grid plot for all samples
plot(f.results, package = "gg") #ggplot for all samples

txpIDs(f.results)[1] <- "chem01" # Change the sample name back
```
<center>
```{r, fig.show="hold", echo=FALSE}
## Long sample names for cramped plots

#change the first sample name in f.results
txpIDs(f.results)[1] <- "I am a long sample name"

plot(f.results) #grid plot for all samples
plot(f.results, package = "gg") #ggplot for all samples

txpIDs(f.results)[1] <- "chem01" # Change the sample name back
```
</center>

```{r, eval=FALSE}
## Long slice names for cramped plots

#change first slice name in model slot
names(f.results@txpModel)[1] <- "long slice name"
#change first slice name in scores slot
colnames(f.results@txpSliceScores)[1] <- "long slice name"
#change first slice name in missing data slot
names(f.results@txpMissing)[1] <- "long slice name"

#plot results using grid
plot(f.results)
#plot results using ggplot
plot(f.results, package = "gg") + theme(
  legend.position = "bottom",
  legend.title = element_text(size = 10),
  legend.text = element_text(size = 6)
)

#change slice name back
names(f.results@txpModel)[1] <- "s1"
colnames(f.results@txpSliceScores)[1] <- "s1"
names(f.results@txpMissing)[1] <- "s1"
```
<center>
```{r, fig.show="hold", echo=FALSE}
## Long slice names for cramped plots

#change first slice name in model slot
names(f.results@txpModel)[1] <- "long slice name"
#change first slice name in scores slot
colnames(f.results@txpSliceScores)[1] <- "long slice name"
#change first slice name in missing data slot
names(f.results@txpMissing)[1] <- "long slice name"

#plot results using grid
plot(f.results)
#plot results using ggplot
plot(f.results, package = "gg") + theme(
  legend.position = "bottom",
  legend.title = element_text(size = 10),
  legend.text = element_text(size = 6)
)

#change slice name back
names(f.results@txpModel)[1] <- "s1"
colnames(f.results@txpSliceScores)[1] <- "s1"
names(f.results@txpMissing)[1] <- "s1"
```
</center>

#### Statistic Plots
```{r fig.width=5, fig.height=4, fig.align='center'}
# Rank plot
plot(f.results, y = txpRanks(f.results), labels = 1:10)

# Hierarchical Clustering
f.hc <- hclust(dist(txpSliceScores(f.results)))

plot(f.hc, hang = -1, labels = txpIDs(f.results), xlab = '', sub = '')
```


## Citation

To cite `toxpiR` in your work, please use: 
https://cran.r-project.org/package=toxpiR (Manuscript currently in review)

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotting-boxLegendGrob.R
\name{boxLegendGrob}
\alias{boxLegendGrob}
\title{Create a filled-box legend}
\usage{
boxLegendGrob(labels, fills, name = NULL, vp = NULL, gp = NULL)
}
\arguments{
\item{labels}{Character, the legend labels}

\item{fills}{Colors to fill the slices}

\item{name, vp, gp}{Passed to \link[grid:grid.frame]{grid::frameGrob}}
}
\description{
Create a filled-box legend
}
\details{
Not yet exported. Need to break out the creation of viewports and grobs as
done in the exported grobs. This will allow better grobEdit methods, which
also needs to be created for the boxLegendGrob.
Also need to do some input checks.

Also, if \code{grid::legendGrob} gets updated to use the 'has.fill' option
this function should be removed and \code{grid::legendGrob} can be used
instead.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/allClasses.R, R/methods-TxpTransFunc.R
\docType{class}
\name{TxpTransFunc-class}
\alias{TxpTransFunc-class}
\alias{TxpTransFunc}
\title{Numeric transformation function}
\usage{
TxpTransFunc(x)
}
\arguments{
\item{x}{function, see details}
}
\description{
S4 class to store numeric transformation functions
}
\details{
\code{TxpTransFunc} inherits from a standard R function, but specifies a
single input and a numeric output of the same length.

Functions can be passed directly to \code{TxpTransFuncList} list and the
functions will be coerced to \code{TxpTransFunc}.

We have an imperfect system for dealing with primitive functions (e.g.,
\link[base:MathFun]{base::sqrt}).
To coerce primitives to TxpTransFunc's, we wrap them in another function
cal; wrapping the primitives obscures the original function and requires
the user to explore the function environment to understand the primitive
called.
We recommend wrapping primitives in separate functions to make the intent
clear, .e.g., \code{mysqrt <- function(x) sqrt(x)}.
}
\examples{
f1 <- function(x) "hello"
f2 <- function(x) 3
f3 <- function(x) x + 5
\dontrun{
t1 <- TxpTransFunc(x = f1) ## Produces error
t2 <- TxpTransFunc(x = f2) ## Produces error
}
t3 <- TxpTransFunc(x = f3)

## TxpTransFunc objects act as any other function
body(t3)
formals(t3)
t3(1:10)

## Coercion from functions
\dontrun{
TxpTransFuncList(f1, f2, f3) ## Produces error because f1, f3 not valid
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/allClasses.R, R/methods-TxpSlice.R
\docType{class}
\name{TxpSlice-class}
\alias{TxpSlice-class}
\alias{TxpSlice}
\alias{txpValueNames,TxpSlice-method}
\alias{txpValueNames<-,TxpSlice-method}
\alias{txpTransFuncs,TxpSlice-method}
\alias{txpTransFuncs<-,TxpSlice-method}
\alias{length,TxpSlice-method}
\alias{merge,TxpSlice,TxpSlice-method}
\title{ToxPi Slice}
\usage{
TxpSlice(txpValueNames, txpTransFuncs = NULL)

\S4method{txpValueNames}{TxpSlice}(x)

\S4method{txpValueNames}{TxpSlice}(x) <- value

\S4method{txpTransFuncs}{TxpSlice}(x)

\S4method{txpTransFuncs}{TxpSlice}(x) <- value

\S4method{length}{TxpSlice}(x)

\S4method{merge}{TxpSlice,TxpSlice}(x, y)
}
\arguments{
\item{txpValueNames}{Passed to \code{txpValueNames} slot}

\item{txpTransFuncs}{Passed to \code{txpTransFuncs} slot}

\item{x, y}{\code{TxpSlice} object}

\item{value}{Replacement value}
}
\description{
S4 class to store ToxPi slices
}
\details{
If the user supplies \code{txpTransFuncs} a single function/\link{TxpTransFunc} object,
the given function will be recycled for each input with a warning.
}
\section{Functions}{
\itemize{
\item \code{txpValueNames(TxpSlice)}: Return \code{txpValueNames} slot

\item \code{txpTransFuncs(TxpSlice)}: Return \code{txpTransFuncs} slot

\item \code{length(TxpSlice)}: Return number of inputs in slice; shortcut for
\code{length(txpValueNames(x))}

\item \code{merge(x = TxpSlice, y = TxpSlice)}: Merge two \code{TxpSlice} objects into a single
slice

}}
\section{Slots}{

\describe{
\item{\code{txpValueNames}}{\verb{vector(<character>)} specifying the input columns to
include in the slice}

\item{\code{txpTransFuncs}}{\link{TxpTransFuncList} with one function per entry in
\code{txpValueNames} or an object that can be coerced to \code{TxpTransFuncList};
when \code{NULL}, no transformation function applied}
}}

\examples{
## Create TxpSlice object 
# Without transform functions
TxpSlice(txpValueNames = c("sqrData", "expData")) 
# With transform functions
TxpSlice(txpValueNames = c("sqrData", "expData"),
         txpTransFuncs = c(sq = function(x) x^2, log = function(x) log(x)))

# Transformation function recycled with warning when single function given
TxpSlice(txpValueNames = c("sqrData", "expData"), 
         txpTransFuncs = function(x) x^2) 
         

## Access TxpSlice slots
sl <- TxpSlice(txpValueNames = c("sqrData", "expData"),
               txpTransFuncs = c(sq = function(x) x^2, 
                                 log = function(x) log(x)))
txpValueNames(sl)
txpTransFuncs(sl)

## Replacement
txpValueNames(sl)[1] <- "hello"
sl

txpTransFuncs(sl)[[2]](exp(1))
txpTransFuncs(sl)[[2]] <- function(x) sqrt(x)
txpTransFuncs(sl)[[2]](exp(1))

# Note that replacing a single list element does NOT update the name
sl
names(txpTransFuncs(sl))[2] <- "sqrt" 
sl

# Replacing the whole list DOES update the names
txpTransFuncs(sl) <- list(sqrt = function(x) sqrt(x), 
                          log = function(x) log(x))
sl

## length -- returns number of inputs
length(TxpSlice(letters))

## merge
s1 <- TxpSlice("hello")
s2 <- TxpSlice("data")
merge(s1, s2)

# Note, input names still must be unique
\dontrun{merge(s1, s1)} ## produces error
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/allClasses.R, R/methods-TxpSliceList.R
\docType{class}
\name{TxpSliceList-class}
\alias{TxpSliceList-class}
\alias{TxpSliceList}
\alias{txpValueNames,TxpSliceList-method}
\alias{txpTransFuncs,TxpSliceList-method}
\alias{duplicated,TxpSliceList-method}
\alias{as.TxpSliceList}
\title{List of TxpSlice objects}
\usage{
TxpSliceList(...)

\S4method{txpValueNames}{TxpSliceList}(x, simplify = FALSE)

\S4method{txpTransFuncs}{TxpSliceList}(x, simplify = FALSE)

\S4method{duplicated}{TxpSliceList}(x)

as.TxpSliceList(x)
}
\arguments{
\item{...}{\link{TxpSlice} object to create \code{TxpSliceList} object; MUST give
unique names to each slice}

\item{x}{\code{TxpSliceList} object}

\item{simplify}{Scalar logical, when \code{TRUE} the returned \code{list} is simplified
to a \code{vector}/\link{TxpTransFuncList} object}
}
\description{
Extension of \link[S4Vectors:SimpleList-class]{S4Vectors::SimpleList} that requires
uniquely-named elements and holds only \link{TxpSlice} objects.
}
\details{
Note, there is no coercion for \link{TxpSlice} to \code{TxpSliceList} because unique
names are required.
}
\section{Functions}{
\itemize{
\item \code{txpValueNames(TxpSliceList)}: Return \code{list} of \code{txpValueNames} slots for the
contained \link{TxpSlice} objects, or \code{vector} when \code{simplify = TRUE}

\item \code{txpTransFuncs(TxpSliceList)}: Return \code{list} of \code{txpTransFuncs} slots for the
contained \link{TxpSlice} objects, or \link{TxpTransFuncList} when \code{simplify = TRUE}

\item \code{duplicated(TxpSliceList)}: Returns logical vector of \code{length(x)}, where
\code{TRUE} indicates a duplicate slice in the list; see \link[base:duplicated]{base::duplicated}

}}
\examples{
## Create TxpSlice objects
s1 <- TxpSlice("input1", list(linear = function(x) x))
s2 <- TxpSlice(c("input2", "input3"), 
               list(log = function(x) log(x), sqrt = function(x) sqrt(x)))

## Create TxpSliceList
sl <- TxpSliceList(s1 = s1, s2 = s2)

## Accessors
txpValueNames(sl)
txpValueNames(sl, simplify = TRUE)

txpTransFuncs(sl)
txpTransFuncs(sl, simplify = TRUE)

## Coercion
as(list(s1 = TxpSlice("hello"), s2 = TxpSlice("user")), "TxpSliceList")
as.TxpSliceList(c(s1 = TxpSlice("hello"), s2 = TxpSlice("user")))

## Concatenation
c(sl, TxpSliceList(s3 = TxpSlice("input4")))

## Reduce TxpSliceList to single slice
Reduce(merge, sl)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/allClasses.R, R/methods-TxpResult.R
\docType{class}
\name{TxpResult-class}
\alias{TxpResult-class}
\alias{TxpResult}
\alias{txpScores,TxpResult-method}
\alias{txpSliceScores,TxpResult-method}
\alias{txpRanks,TxpResult-method}
\alias{txpMissing,TxpResult-method}
\alias{txpResultParam,TxpResult-method}
\alias{txpModel,TxpResult-method}
\alias{txpIDs,TxpResult-method}
\alias{txpIDs<-,TxpResult-method}
\alias{txpWeights,TxpResult-method}
\alias{txpSlices,TxpResult-method}
\alias{txpTransFuncs,TxpResult-method}
\alias{txpValueNames,TxpResult-method}
\alias{[,TxpResult,logical,missing-method}
\alias{[,TxpResult,integer,missing-method}
\alias{[,TxpResult,numeric,missing-method}
\alias{[,TxpResult,character,missing-method}
\alias{length,TxpResult-method}
\alias{sort,TxpResult-method}
\alias{names,TxpResult-method}
\alias{names<-,TxpResult-method}
\alias{as.data.frame,TxpResult-method}
\title{ToxPi Result}
\usage{
\S4method{txpScores}{TxpResult}(x)

\S4method{txpSliceScores}{TxpResult}(x, adjusted = TRUE)

\S4method{txpRanks}{TxpResult}(x)

\S4method{txpMissing}{TxpResult}(x)

\S4method{txpResultParam}{TxpResult}(x)

\S4method{txpModel}{TxpResult}(x)

\S4method{txpIDs}{TxpResult}(x)

\S4method{txpIDs}{TxpResult}(x) <- value

\S4method{txpWeights}{TxpResult}(x, adjusted = FALSE)

\S4method{txpSlices}{TxpResult}(x)

\S4method{txpTransFuncs}{TxpResult}(x, level, simplify = FALSE)

\S4method{txpValueNames}{TxpResult}(x, simplify = FALSE)

\S4method{[}{TxpResult,logical,missing}(x, i, j, ..., drop = FALSE)

\S4method{[}{TxpResult,integer,missing}(x, i, j, ..., drop = FALSE)

\S4method{[}{TxpResult,numeric,missing}(x, i, j, ..., drop = FALSE)

\S4method{[}{TxpResult,character,missing}(x, i, j, ..., drop = FALSE)

\S4method{length}{TxpResult}(x)

\S4method{sort}{TxpResult}(x, decreasing = TRUE, na.last = TRUE, ...)

\S4method{names}{TxpResult}(x)

\S4method{names}{TxpResult}(x) <- value

\S4method{as.data.frame}{TxpResult}(
  x,
  row.names = NULL,
  optional = FALSE,
  ...,
  id.name = "id",
  score.name = "score",
  rank.name = "rank",
  adjusted = FALSE
)
}
\arguments{
\item{x}{\link{TxpResult} object}

\item{adjusted}{Logical scalar, when \code{TRUE} the weights are adjusted to sum
to 1 or the slice scores are scaled to their respective weight}

\item{value}{Replacement value}

\item{level}{\code{c('model', 'slices')}; indicates whether to retrieve
\code{txpTransFuncs} slot from the model or underlying slices}

\item{simplify}{Logical scalar, flatten \code{txpValueNames} or \code{txpTransFunc}
slots when retrieving slice-level information}

\item{i}{Subsetting index}

\item{j, drop, optional}{Not currently implemented}

\item{...}{Passed to \link[base:data.frame]{base::data.frame} in \code{as.data.frame} or \link[base:sort]{base::sort}
in \code{sort}}

\item{decreasing, na.last}{Passed to \link[base:sort]{base::sort}}

\item{row.names}{Passed to \link[base:data.frame]{base::data.frame}}

\item{id.name, score.name, rank.name}{Character scalar; when coercing to
\link[base:data.frame]{base::data.frame}, the name for the \code{txpIDs}, \code{txpScores}, and \code{txpRanks}
columns, respectively}
}
\description{
S4 class to store ToxPi results
}
\section{Functions}{
\itemize{
\item \code{txpScores(TxpResult)}: Return \code{txpScores} slot

\item \code{txpSliceScores(TxpResult)}: Return \code{txpSliceScores} slot; default
\code{adjusted = TRUE}, i.e. return slice scores adjusted for weight

\item \code{txpRanks(TxpResult)}: Return \code{txpRanks} slot

\item \code{txpMissing(TxpResult)}: Return \code{txpMissing} slot

\item \code{txpResultParam(TxpResult)}: Return \code{txpResultParam} slot

\item \code{txpModel(TxpResult)}: Return \code{txpModel} slot

\item \code{txpIDs(TxpResult)}: Return \code{txpIDs} slot

\item \code{txpWeights(TxpResult)}: Return \code{txpWeights} slot from model -- shortcut
for \code{txpWeights(txpModel(x))}; default \code{adjusted = FALSE}, i.e. return
unadjusted weights

\item \code{txpSlices(TxpResult)}: Return \code{txpSlices} slot from model -- shortcut
for \code{txpSlices(txpModel(x))}

\item \code{txpTransFuncs(TxpResult)}: Return \code{txpTransFuncs} slot from model --
shortcut for \code{txpTransFuncs(txpModel(x))}

\item \code{txpValueNames(TxpResult)}: Return \code{txpValueNames} slot from slices --
shortcut for \code{txpValueNames(txpSlices(txpModel(x)))}

\item \code{length(TxpResult)}: Return the number of observations; shortcut for
\code{length(txpScores(x))}

\item \code{sort(TxpResult)}: Sort the ``TxpResult` object by their ranks

\item \code{names(TxpResult)}: Returns IDs; equal to \code{txpIDs(x)}

\item \code{as.data.frame(TxpResult)}: Coerce TxpResult to \link[base:data.frame]{base::data.frame} object
with IDs, scores, ranks, and slice scores

}}
\section{Slots}{

\describe{
\item{\code{txpScores}}{\verb{vector(<numeric>)} of model scores}

\item{\code{txpSliceScores}}{\verb{matrix(<numeric>)}, sample by slice \code{matrix} with
individual slice scores}

\item{\code{txpRanks}}{\verb{vector(<numeric>)} with rank of scores}

\item{\code{txpMissing}}{\verb{vector(<numeric>)} with data missingness}

\item{\code{txpModel}}{\link{TxpModel} object}

\item{\code{txpIDs}}{\verb{vector(<character>)} of observation IDs}

\item{\code{txpResultParam}}{\link{TxpResultParam} object}
}}

\examples{
## Load example dataset & model; see ?TxpModel for building model objects
data(txp_example_input, package = "toxpiR")
data(txp_example_model, package = "toxpiR")

## Calculate scores for single model; returns TxpResult object
res <- txpCalculateScores(model = txp_example_model, 
                          input = txp_example_input, 
                          id.var = "name")

## Accessors
txpScores(res)

txpSliceScores(res) ## adjusted for weight, by default
apply(txpSliceScores(res), 2, max, na.rm = TRUE)

txpSliceScores(res, adjusted = FALSE) ## each score should have maximum of 1
apply(txpSliceScores(res, adjusted = FALSE), 2, max, na.rm = TRUE)

txpRanks(res)

txpMissing(res)

txpModel(res)
identical(txpModel(res), txp_example_model)

txpIDs(res)
names(res) ## identical to txpIDs(res)
identical(txpIDs(res), names(res))

# Can access TxpModel slots directly
txpWeights(res)
txpWeights(res, adjusted = TRUE)
txpSlices(res)
# When retrieving transform functions, must specify level because both
# models and slices have transform functions
txpTransFuncs(res, level = "model")

# Can access TxpSliceList slots directly
txpValueNames(res)
txpValueNames(res, simplify = TRUE)
txpTransFuncs(res, level = "slices")
txpTransFuncs(res, level = "slices", simplify = TRUE)

## Subsetting
res[1]
res[c("chem01", "chem09")]
res[grepl("4|6", txpIDs(res))]
\dontrun{
res[c(TRUE, FALSE)] ## gets recycled with warning
}

## length -- returns number of observations
length(res)
length(res[1:5])

## sort
names(res)
names(sort(res))

txpScores(res)
txpScores(sort(res))
txpScores(sort(res, decreasing = FALSE))

## as.data.frame
as.data.frame(res)
as.data.frame(res, id.name = "nm", score.name = "scr", rank.name = "rnk")
}
\seealso{
\link{txpCalculateScores}, \link{plot}, \link{TxpResultList}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/toxpiR-package.R
\name{toxpiR-datasets}
\alias{toxpiR-datasets}
\alias{txp_example_input}
\alias{txp_example_model}
\title{toxpiR data objects}
\source{
\url{https://github.com/ToxPi/ToxPi-example-files}
}
\usage{
data(txp_example_input, package = "toxpiR")

data(txp_example_model, package = "toxpiR")
}
\description{
Objects included in the toxpiR package, loaded with
\link[utils:data]{utils::data}
}
\section{txp_example_input}{


Small example input data to be used with \link{txpCalculateScores} in creating
\link{TxpResult} objects. A \link[base:data.frame]{base::data.frame} with 10 rows and 9 variables
\describe{
\item{name}{Observation names}
\item{metric#}{Input data for ToxPi models}
}
}

\section{txp_example_model}{


Example \link{TxpModel} object intended for \code{txp_example_data}; model with 4
slices.
}

\examples{
data(txp_example_input, package = "toxpiR")
data(txp_example_model, package = "toxpiR")
txp_example_input
txp_example_model

## Code to create txp_example_model
tf1 <- TxpTransFuncList(linear = function(x) x)
sl <- TxpSliceList(s1 = TxpSlice(sprintf("metric\%d", 1:2)),
                   s2 = TxpSlice("metric3"),
                   s3 = TxpSlice(sprintf("metric\%d", 4:7),
                                 tf1[rep("linear", 4)]),
                   s4 = TxpSlice("metric8", tf1))
tf2 <- TxpTransFuncList(NULL, linear = function(x) x, NULL, NULL)
TxpModel(txpSlices = sl, txpWeights = c(2, 1, 3, 2), txpTransFuncs = tf2)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/allClasses.R, R/methods-TxpModel.R
\docType{class}
\name{TxpModel-class}
\alias{TxpModel-class}
\alias{TxpModel}
\alias{txpSlices,TxpModel-method}
\alias{TxpModel-txpSlices}
\alias{txpSlices<-,TxpModel-method}
\alias{txpWeights,TxpModel-method}
\alias{txpWeights<-,TxpModel-method}
\alias{txpTransFuncs,TxpModel-method}
\alias{txpTransFuncs<-,TxpModel-method}
\alias{txpValueNames,TxpModel-method}
\alias{names,TxpModel-method}
\alias{names<-,TxpModel-method}
\alias{length,TxpModel-method}
\alias{merge,TxpModel,TxpModel-method}
\title{ToxPi Model}
\usage{
TxpModel(txpSlices, txpWeights = NULL, txpTransFuncs = NULL)

\S4method{txpSlices}{TxpModel}(x)

\S4method{txpSlices}{TxpModel}(x) <- value

\S4method{txpWeights}{TxpModel}(x, adjusted = FALSE)

\S4method{txpWeights}{TxpModel}(x) <- value

\S4method{txpTransFuncs}{TxpModel}(x)

\S4method{txpTransFuncs}{TxpModel}(x) <- value

\S4method{txpValueNames}{TxpModel}(x, simplify = FALSE)

\S4method{names}{TxpModel}(x)

\S4method{names}{TxpModel}(x) <- value

\S4method{length}{TxpModel}(x)

\S4method{merge}{TxpModel,TxpModel}(x, y)
}
\arguments{
\item{txpSlices}{Passed to \code{txpSlices} slot}

\item{txpWeights}{Passed to \code{txpWeights} slot}

\item{txpTransFuncs}{Passed to \code{txpTransFuncs} slot}

\item{x, y}{TxpModel object}

\item{value}{Replacement value}

\item{adjusted}{Scalar logical, should the returned weights be adjusted
such that they sum to 1?}

\item{simplify}{Scalar logical, when \code{TRUE} the returned \code{list} is simplified}
}
\description{
S4 class to store ToxPi models
}
\section{Functions}{
\itemize{
\item \code{txpSlices(TxpModel)}: Return \code{txpSlices} slot

\item \code{txpWeights(TxpModel)}: Return \code{txpWeights} slot

\item \code{txpTransFuncs(TxpModel)}: Return \code{txpTransFuncs} slot

\item \code{txpValueNames(TxpModel)}: Return \code{list} of \code{txpValueNames} slots for the
contained \link{TxpSliceList} object, or \code{vector} when \code{simplify = TRUE}

\item \code{names(TxpModel)}: Return slice names; shortcut for
\code{names(txpSlices(x))}

\item \code{length(TxpModel)}: Return number of slices in model; shortcut for
\code{length(txpSlices(x))}

\item \code{merge(x = TxpModel, y = TxpModel)}: Merge two \code{TxpModel} objects into a single
model

}}
\section{Slots}{

\describe{
\item{\code{txpSlices}}{\link{TxpSliceList} object}

\item{\code{txpWeights}}{numeric vector specifying the relative weight of each slice;
when NULL, defaults to 1 (equal weighting) for each slice}

\item{\code{txpTransFuncs}}{\link{TxpTransFuncList} object (or list of functions
coercible to TxpTransFuncList)}
}}

\examples{
## Create TxpSliceList & TxpTransFuncList objects
s1 <- list(S1 = TxpSlice("inpt1"), S2 = TxpSlice("inpt2"))
tf <- list(NULL, sqrt = function(x) sqrt(x))

## Create TxpModel object
m1 <- TxpModel(txpSlices = s1, txpWeights = 2:1, txpTransFuncs = tf)
m1

## Access TxpModel slots
txpSlices(m1)
txpWeights(m1)
txpWeights(m1, adjusted = TRUE)
txpTransFuncs(m1)

## length
length(m1) ## equal to length(txpSlices(m1))
length(m1) == length(txpSlices(m1))

## names
names(m1) ## equal to names(txpSlices(m1))
all(names(m1) == names(txpSlices(m1)))

## Replacement
m2 <- m1
txpSlices(m2) <- list(S3 = TxpSlice("inpt3"), S4 = TxpSlice("inpt4"))
m2
names(m2)[2] <- "hello"
names(m2)
txpTransFuncs(m2) <- NULL
m2
txpTransFuncs(m2)[[1]] <- function(x) x^2
names(txpTransFuncs(m2))[1] <- "sq"
m2

## merge
m3 <- merge(m1, m2)
m3
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/allGenerics.R, R/methods-TxpModel.R,
%   R/methods-TxpModelList.R, R/txpCalculateScores.R
\name{txpCalculateScores}
\alias{txpCalculateScores}
\alias{txpCalculateScores,TxpModel,data.frame-method}
\alias{txpCalculateScores,TxpModelList,data.frame-method}
\alias{txpCalculateScores,list,data.frame-method}
\title{Calculate ToxPi Scores for the given model and input data}
\usage{
txpCalculateScores(model, input, ...)

\S4method{txpCalculateScores}{TxpModel,data.frame}(
  model,
  input,
  id.var = NULL,
  rank.ties.method = c("average", "first", "last", "random", "max", "min"),
  negative.value.handling = c("keep", "missing")
)

\S4method{txpCalculateScores}{TxpModelList,data.frame}(
  model,
  input,
  id.var = NULL,
  rank.ties.method = c("average", "first", "last", "random", "max", "min"),
  negative.value.handling = c("keep", "missing")
)

\S4method{txpCalculateScores}{list,data.frame}(
  model,
  input,
  id.var = NULL,
  rank.ties.method = c("average", "first", "last", "random", "max", "min"),
  negative.value.handling = c("keep", "missing")
)
}
\arguments{
\item{model}{\link{TxpModel} object or \link{TxpModelList} object}

\item{input}{data.frame object containing the model input data}

\item{...}{Included for extendability; not currently used}

\item{id.var}{Character scalar, column in 'input' to store in}

\item{rank.ties.method}{Passed to \code{rank.ties.method} slot}

\item{negative.value.handling}{Passed to \code{negative.value.handling} slot}
}
\value{
\link{TxpResult} or \link{TxpResultList} object
}
\description{
Calculate ToxPi Scores for the given model and input data
}
\details{
\code{txpCalculateScores} is implemented as an S4 generic function with methods
for \link{TxpModel} and \link{TxpModelList}.

Ranks are calculated such that the highest ToxPi score has a rank of 1.

Missingness is determined after applying input-level transformations but
before applying slice-level transformations.
}
\examples{
## Load example dataset & model; see ?TxpModel for building model objects
data(txp_example_input, package = "toxpiR")
data(txp_example_model, package = "toxpiR")

## Calculate scores for single model; returns TxpResult object
res <- txpCalculateScores(model = txp_example_model, 
                          input = txp_example_input, 
                          id.var = "name")

## Calculate scores for list of models; returns TxpResultList object
txpCalculateScores(model = TxpModelList(m1 = txp_example_model, 
                                        m2 = txp_example_model), 
                   input = txp_example_input, 
                   id.var = "name")
resLst <- txpCalculateScores(model = list(m1 = txp_example_model, 
                                          m2 = txp_example_model), 
                             input = txp_example_input, 
                             id.var = "name")

}
\seealso{
\link{TxpModel}, \link{TxpResult}, \link{TxpResultParam}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/allGenerics.R
\name{txpGenerics}
\alias{txpGenerics}
\alias{txpValueNames}
\alias{txpValueNames<-}
\alias{txpTransFuncs}
\alias{txpTransFuncs<-}
\alias{txpSlices}
\alias{txpSlices<-}
\alias{txpWeights}
\alias{txpWeights<-}
\alias{txpScores}
\alias{txpSliceScores}
\alias{txpModel}
\alias{txpIDs}
\alias{txpIDs<-}
\alias{txpRanks}
\alias{txpMissing}
\alias{txpResultParam}
\title{toxpiR package generics}
\usage{
txpValueNames(x, ...)

txpValueNames(x, ...) <- value

txpTransFuncs(x, ...)

txpTransFuncs(x, ...) <- value

txpSlices(x, ...)

txpSlices(x, ...) <- value

txpWeights(x, ...)

txpWeights(x, ...) <- value

txpScores(x, ...)

txpSliceScores(x, ...)

txpModel(x, ...)

txpIDs(x, ...)

txpIDs(x, ...) <- value

txpRanks(x, ...)

txpMissing(x, ...)

txpResultParam(x, ...)
}
\arguments{
\item{x}{toxpiR S4 object}

\item{...}{Included for extendability; not currently used}

\item{value}{Replacement value}
}
\value{
See specific methods for details.
}
\description{
toxpiR package generics; see class man pages for associated
methods
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/txpExportGui.R
\name{txpExportGui}
\alias{txpExportGui}
\title{Export comma-separated file intended for ToxPi GUI}
\usage{
txpExportGui(
  fileName = "txpModel.csv",
  input,
  model,
  id.var = NULL,
  fills = NULL
)
}
\arguments{
\item{fileName}{Character scalar, the path to the output file}

\item{input}{data.frame object containing the model input data}

\item{model}{\link{TxpModel} object or \link{TxpModelList} object}

\item{id.var}{Character scalar, column in 'input' to store in}

\item{fills}{Colors to fill the slices}
}
\description{
Export comma-separated file intended for ToxPi GUI
}
\details{
The GUI differs in two meaninful ways for exporting \code{toxpiR} models: (1) the
GUI only allows for integer weights; (2) the GUI applies transformation
functions differently.

\code{txpExporGui} will not work for models with non-integer weights.

The GUI only applies a single transformation function to every input within
a slice, and only functions from a pre-determined list; \code{toxpiR} allows
users to apply any valid function individually to each input, then a second
transformation function on the summed slice values. Because of this
complexity, any exported models with slice-level transformation functions
will not export at the input level. In other words, the export will have only
the final slice scores. Otherwise, all input-level transformations will be
performed, the and the export will contain transformed input-level data with
the \code{linear(x)} GUI transformation.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotting-pieGrob.R
\name{pieGrob}
\alias{pieGrob}
\alias{grid.pieGrob}
\title{Create a pie grob}
\usage{
pieGrob(rads, fills = NULL, wts = NULL, name = NULL, vp = NULL, gp = NULL)

grid.pieGrob(rads, fills = NULL, wts = NULL, name = NULL, vp = NULL, gp = NULL)
}
\arguments{
\item{rads}{Numeric, radius values for each slice from 0 to 1}

\item{fills}{Colors to fill the slices}

\item{wts}{Numeric, the relative portion of each slice}

\item{name, vp, gp}{Passed to \link[grid:grid.grob]{grid::gTree}}
}
\value{
\code{pieGrob} \link[grid:grid.grob]{grid::grob} object
}
\description{
Create a pie grob
}
\details{
The default coloring can be set with \code{options("txp.fills")}.
}
\examples{
library(grid)

s <- seq(0.2, 1, by = 0.1)
grid.newpage()
grid.pieGrob(rads = s)
grid.newpage()
grid.pieGrob(rads = s, wts = s)

curr_txp_fills <- options()$txp.fills
options(txp.fills = 1:8)
grid.newpage()
grid.pieGrob(rads = s)
options(txp.fills = curr_txp_fills)

## Can edit
grid.newpage()
grid.pieGrob(rads = s, name = "myPie")
grid.ls() ## show the grid elements
grid.edit("myPie", fills = 1:9, wts = 9:1)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/allClasses.R, R/methods-TxpResultList.R
\docType{class}
\name{TxpResultList-class}
\alias{TxpResultList-class}
\alias{TxpResultList}
\alias{duplicated,TxpResultList-method}
\alias{as.TxpResultList}
\title{List of TxpResult objects}
\usage{
TxpResultList(...)

\S4method{duplicated}{TxpResultList}(x)

as.TxpResultList(x)
}
\arguments{
\item{...}{\link{TxpResult} object to create \code{TxpResultList} object}

\item{x}{\code{TxpResultList} object}
}
\description{
Extension of \link[S4Vectors:SimpleList-class]{S4Vectors::SimpleList} that holds only \link{TxpResult}
objects.
}
\examples{
## Load example dataset & model; see ?TxpModel for building model objects
data(txp_example_input, package = "toxpiR")
data(txp_example_model, package = "toxpiR")

## Calculate scores for list of models; returns TxpResultList object
txpCalculateScores(model = TxpModelList(m1 = txp_example_model, 
                                        m2 = txp_example_model), 
                   input = txp_example_input, 
                   id.var = "name")
resLst <- txpCalculateScores(model = list(m1 = txp_example_model, 
                                          m2 = txp_example_model), 
                             input = txp_example_input, 
                             id.var = "name")

## duplicated
duplicated(resLst)

## Coercion
as(list(resLst[[1]], resLst[[2]]), "TxpResultList")
as.TxpResultList(list(res1 = resLst[[1]], res2 = resLst[[2]]))

as(resLst[[1]], "TxpResultList")
as.TxpResultList(resLst[[1]])
}
\seealso{
\link{TxpResult}, \link{txpCalculateScores}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/toxpiR-package.R
\docType{package}
\name{toxpiR-package}
\alias{toxpiR}
\alias{toxpiR-package}
\title{toxpiR: Create ToxPi Prioritization Models}
\description{
Enables users to build 'ToxPi' prioritization models and provides functionality within the grid framework for plotting ToxPi graphs. 'toxpiR' allows for more customization than the 'ToxPi GUI' (\url{https://toxpi.github.io/}) and integration into existing workflows for greater ease-of-use, reproducibility, and transparency. toxpiR package behaves nearly identically to the GUI; the package documentation includes notes about all differences. The vignettes download example files from \url{https://github.com/ToxPi/ToxPi-example-files}.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/ToxPi/toxpiR}
  \item \url{https://toxpi.github.io/toxpiR/}
  \item Report bugs at \url{https://github.com/ToxPi/toxpiR/issues}
}

}
\author{
\strong{Maintainer}: Jonathon F Fleming \email{jffleming0129@gmail.com} (\href{https://orcid.org/0000-0003-2447-3139}{ORCID})

Authors:
\itemize{
  \item Dayne L Filer \email{dayne.filer@gmail.com} (\href{https://orcid.org/0000-0002-3443-5315}{ORCID}) [funder]
  \item Dillon T Lloyd
  \item Preethi Thunga (\href{https://orcid.org/0000-0001-5447-0129}{ORCID})
  \item Skylar W Marvel
  \item David M Reif \email{reif.david@gmail.com} (\href{https://orcid.org/0000-0001-7815-6767}{ORCID}) [funder]
}

Other contributors:
\itemize{
  \item Alison A Motsinger-Reif (\href{https://orcid.org/0000-0003-1346-2493}{ORCID}) [funder]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods-TxpResult-plot.R
\name{TxpResult-plot}
\alias{TxpResult-plot}
\alias{plot}
\alias{plot,TxpResult,missing-method}
\alias{plot,TxpResult,numeric-method}
\title{Plot TxpResult objects}
\usage{
\S4method{plot}{TxpResult,missing}(
  x,
  package = c("grid", "ggplot2"),
  fills = NULL,
  showScore = TRUE,
  gp = NULL,
  vp = NULL,
  name = NULL,
  newpage = TRUE,
  ...,
  ncol = NULL,
  bgColor = "grey80",
  borderColor = "white",
  sliceBorderColor = "white",
  sliceValueColor = NULL,
  sliceLineColor = NULL,
  showMissing = TRUE,
  showCenter = TRUE
)

\S4method{plot}{TxpResult,numeric}(
  x,
  y,
  labels = NULL,
  newpage = TRUE,
  margins = c(4, 0, 1, 1),
  name = NULL,
  gp = NULL,
  vp = NULL,
  ...
)
}
\arguments{
\item{x}{\link{TxpResult} object}

\item{package}{Character scalar, choice of "grid" or "ggplot2" for plotting
ToxPi profiles}

\item{fills}{Vector of colors to fill slices. Set to NULL to use default}

\item{showScore}{Logical scalar, overall score printed below the name when
\code{TRUE}}

\item{gp, vp, name}{Passed to \link[grid:grid.frame]{grid::frameGrob} when creating the plotting
area}

\item{newpage}{Logical scalar, \link[grid:grid.newpage]{grid::grid.newpage} called prior to plotting
when \code{TRUE}}

\item{...}{Passed to \link{pieGridGrob} when plotting ToxPi and to pointsGrob
when plotting ranks}

\item{ncol}{Number of columns for ggplot2 ToxPi profiles}

\item{bgColor, borderColor, sliceBorderColor, sliceValueColor, sliceLineColor}{Various color options when creating ggplot2 ToxPi profiles. Set to NULL
for no color}

\item{showMissing}{Boolean for coloring data missingness in ggplot2
ToxPi profiles}

\item{showCenter}{Boolean for showing inner circle in ggplot2 ToxPi
profiles. When set to False overrides showMissing}

\item{y}{Rank vector, i.e. \code{txpRanks(x)}}

\item{labels}{Integer vector, indices of \code{x} to label in the rank plot}

\item{margins}{Passed to \link[grid:plotViewport]{grid::plotViewport}; only affects the scatterplot
region margins}
}
\value{
No return value when using grid; called for side effect (i.e.
drawing in current graphics device). Will return ggplot2 object otherwise.
}
\description{
Plot \link{TxpResult} objects
}
\details{
It is strongly recommended to use a specific device (e.g., \link[grDevices:png]{grDevices::png},
\link[grDevices:pdf]{grDevices::pdf}) when creating rank plots.
Using a GUI device will likely lead to inaccurate labeling, and any changes
to the device size WILL lead to inaccurate labeling.

The plotting is built on the \link[grid:grid-package]{grid::grid-package}, and can be adjusted or
edited as such.

If the labels are running of the device, the top or bottom margins can be
increased with the \code{margins} parameter.

ToxPi profiles can also be plotted using the ggplot2 package.
}
\section{Functions}{
\itemize{
\item \code{plot(x = TxpResult, y = missing)}: Plot ToxPi diagrams

\item \code{plot(x = TxpResult, y = numeric)}: Plot ToxPi ranks

}}
\examples{
## Load example dataset & model; see ?TxpModel for building model objects
data(txp_example_input, package = "toxpiR")
data(txp_example_model, package = "toxpiR")

## Calculate scores for single model; returns TxpResult object
res <- txpCalculateScores(model = txp_example_model, 
                          input = txp_example_input, 
                          id.var = "name")

library(grid)
plot(res)
plot(res[order(txpRanks(res))[1:4]])

library(ggplot2)
plot(res, package = "gg")
plot(res[order(txpRanks(res))], package = "gg", ncol = 5) +
  theme(legend.position = "bottom")

plot(res, txpRanks(res))
plot(res, txpRanks(res), pch = 16, size = unit(0.75, "char"))

## Will likely make inaccurate labels within a GUI, e.g. RStudio
## use png, pdf, etc. to get accurate labels
\dontrun{
tmpPdf <- tempfile()
pdf(tmpPdf)
plot(res, txpRanks(res), labels = c(10, 4, 2), pch = 16)
dev.off()
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/allClasses.R, R/methods-TxpResultParam.R
\docType{class}
\name{TxpResultParam-class}
\alias{TxpResultParam-class}
\alias{TxpResultParam}
\title{ToxPi Result Parameters}
\arguments{
\item{rank.ties.method}{Passed to \code{rank.ties.method} slot}

\item{negative.value.handling}{Passed to \code{negative.value.handling} slot}
}
\description{
S4 class to store ToxPi result calculation parameters
}
\details{
If more than one value is passed to \code{TxoResultParam} scalar options, e.g.
\code{rank.ties.method}, only the first value is kept.

The \code{rank.ties.method} slot is passed to \link[base:rank]{base::rank} for calculating the
ranks of observations, with the highest-scoring observation having the rank
of 1.

\code{negative.value.handling} indicates how to handle negative values in the
inputs. The ToxPi algorithm originally intended to accept non-negative
potency values; the GUI, therefore, treats negative values in the input as
missing. By default, \link{txpCalculateScores} keeps negative values
(\code{negative.value.handling = "keep"}). To replicate the GUI behavior, users
can set \code{negative.value.handling = "missing"}.
}
\section{Slots}{

\describe{
\item{\code{rank.ties.method}}{Character scalar, method used to calculate score
ranks passed to \link[base:rank]{base::rank}}

\item{\code{negative.value.handling}}{Character scalar, how negative values are
handled, see details}
}}

\seealso{
\link{txpCalculateScores}, \link{TxpResult}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/txpImportGui.R
\name{txpImportGui}
\alias{txpImportGui}
\title{Import data file generated by ToxPi GUI}
\usage{
txpImportGui(guiDataFile)
}
\arguments{
\item{guiDataFile}{Character scalar, the path to a 'data' export from the
ToxPi GUI}
}
\value{
\code{list} with \verb{$model} containing \link{TxpModel} object; \verb{$input}
containing \code{data.frame} with input data; \verb{$fills} containing a vector
of fill colors.
}
\description{
Import data file generated by ToxPi GUI
}
\details{
This function takes the '_data.csv' files generated by the GUI.
See \url{https://toxpi.github.io/} for more information.

Because of the way toxpiR implements transformation functions, there is not
a way currently to use the GUI 'hitcount' function.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/allClasses.R, R/methods-TxpModelList.R
\docType{class}
\name{TxpModelList-class}
\alias{TxpModelList-class}
\alias{TxpModelList}
\alias{duplicated,TxpModelList-method}
\alias{as.TxpModelList}
\title{List of TxpModel objects}
\usage{
TxpModelList(...)

\S4method{duplicated}{TxpModelList}(x)

as.TxpModelList(x)
}
\arguments{
\item{...}{\link{TxpModel} object to create \code{TxpModelList} object}

\item{x}{\code{TxpModelList} object}
}
\description{
Extension of \link[S4Vectors:SimpleList-class]{S4Vectors::SimpleList} that holds only \link{TxpModel}
objects.
}
\section{Functions}{
\itemize{
\item \code{duplicated(TxpModelList)}: Returns logical vector of \code{length(x)}, where
\code{TRUE} indicates a duplicate model in the list; see \link[base:duplicated]{base::duplicated}

\item \code{as.TxpModelList()}: Coerce list or \link{TxpModel} objects to
TxpModelList

}}
\examples{
## Create some TxpModel objects; see ?TxpModel for more details
s1 <- list(S1 = TxpSlice("inpt1"), S2 = TxpSlice("inpt2"))
tf <- list(NULL, sqrt = function(x) sqrt(x))
m1 <- TxpModel(txpSlices = s1, txpWeights = 2:1, txpTransFuncs = tf)
m2 <- m1
txpSlices(m2) <- list(S3 = TxpSlice("inpt3"), S4 = TxpSlice("inpt4"))
m3 <- merge(m1, m2)

## Build a TxpModelList object
TxpModelList(m1 = m1, m2 = m2, m3 = m3)

## Note: names are printed as '' when all are NULL
TxpModelList(m1, m2, m3)
names(TxpModelList(m1, m2, m3))

## Test for duplicates
duplicated(TxpModelList(m1 = m1, m2 = m2, m3 = m3))
duplicated(TxpModelList(m1 = m1, m2 = m1, m3 = m3))

## Coerce lists/TxpModel objects to TxpModelList
as(list(m1 = m1, m2 = m2, m3 = m3), "TxpModelList")
as.TxpModelList(list(m1 = m1, m2 = m2, m3 = m3))

as(m1, "TxpModelList")
as.TxpModelList(m1)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/allClasses.R, R/methods-TxpTransFuncList.R
\docType{class}
\name{TxpTransFuncList-class}
\alias{TxpTransFuncList-class}
\alias{TxpTransFuncList}
\alias{as.TxpTransFuncList}
\title{List of TxpTransFunc objects}
\usage{
TxpTransFuncList(...)

as.TxpTransFuncList(x)
}
\arguments{
\item{...}{\link{TxpTransFunc} object or function to create \code{TxpTransFuncList}
object}

\item{x}{\code{list}, \code{function}, or \link{TxpTransFunc} object to coerce to
\code{TxpTransFuncList}}
}
\description{
Extension of \link[S4Vectors:SimpleList-class]{S4Vectors::SimpleList} that holds only \code{NULL} or
\link{TxpTransFunc} objects.
}
\details{
When \code{...} includes function objects, \code{TxpTransFuncList} will attempt to
coerce them to \link{TxpTransFunc} and return an error if any of the elements
cannot be coerced to \link{TxpTransFunc}.
}
\examples{
## Create TxpTransFunc objects
tf1 <- TxpTransFunc(function(x) x)
tf2 <- TxpTransFunc(function(x) sqrt(x))

## Create TxpTransFuncList 
tfl <- TxpTransFuncList(linear = tf1, sqrt = tf2, cube = function(x) x^3)
tfl[[3]](3) == 27
tfl[["sqrt"]](4) == 2

## Concatenate
c(tfl, tfl)

## names
names(c(tfl, tfl))

# note: names are printed as '' when missing; NULL is printed when list item
# is NULL
names(TxpTransFuncList(function(x) x, NULL))
TxpTransFuncList(function(x) x, NULL)

## coercion
as(function(x) x, "TxpTransFuncList")
as.TxpTransFuncList(function(x) x)

as(TxpTransFunc(function(x) x), "TxpTransFuncList")
as.TxpTransFuncList(TxpTransFunc(function(x) x))

as(list(function(x) x, sqrt = function(x) sqrt(x)), "TxpTransFuncList")
as.TxpTransFuncList(list(function(x) x, sqrt = function(x) sqrt(x)))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotting-pieGridGrob.R
\name{pieGridGrob}
\alias{pieGridGrob}
\alias{grid.pieGridGrob}
\title{Make grid of pieGrobs}
\usage{
pieGridGrob(
  radMat,
  wts = NULL,
  fills = NULL,
  labels = NULL,
  showRadSum = FALSE,
  ncol = NULL,
  nrow = NULL,
  byrow = TRUE,
  name = NULL,
  gp = NULL,
  vp = NULL
)

grid.pieGridGrob(
  radMat,
  wts = NULL,
  fills = NULL,
  labels = NULL,
  showRadSum = FALSE,
  ncol = NULL,
  nrow = NULL,
  byrow = TRUE,
  name = NULL,
  gp = NULL,
  vp = NULL
)
}
\arguments{
\item{radMat}{\verb{matrix(<numeric>)}, observations by slice radii}

\item{wts}{\verb{vector(<numeric>)}, relative weights of each slice}

\item{fills}{Vector of colors to fill slices}

\item{labels}{\verb{vector(<character>)}, (optional) label for each observation}

\item{showRadSum}{Logical scalar, when \code{TRUE} show the weighted sum of slices
below the label}

\item{nrow, ncol}{Integer scalar, number of rows and columns for the grid}

\item{byrow}{Logical scalar, fill the grid by rows when \code{TRUE}}

\item{name, gp, vp}{Passed to \link[grid:grid.grob]{grid::gTree}}
}
\value{
\code{pieGrob} \link[grid:grid.grob]{grid::grob} object
}
\description{
Make grid of pieGrobs
}
\examples{
\donttest{
library(grid)

s <- seq(0.2, 1, by = 0.1)
smat <- do.call("rbind", replicate(20, s, simplify = FALSE))
grid.newpage()
grid.pieGridGrob(radMat = smat)

rownames(smat) <- sprintf("obs\%02d", 1:20)
grid.newpage()
grid.pieGridGrob(radMat = smat, wts = s)
grid.newpage()
grid.pieGridGrob(radMat = smat, wts = s, showRadSum = TRUE, labels = FALSE)
grid.newpage()
grid.pieGridGrob(radMat = smat, labels = "hello")
grid.newpage()
grid.pieGridGrob(radMat = smat, labels = 1:20)

## Can edit like normal grid objects
grid.newpage()
grid.pieGridGrob(radMat = smat, wts = s, showRadSum = TRUE)
grid.ls() ## shows grid elements
grid.edit("pie-20", fills = 1:9)
grid.edit("pie-19-label", gp = gpar(font = 2, col = "red"))
grid.edit("pie-1", wts = rep(1, 9), rads = rep(1, 9))
for (s in sprintf("pie-\%d-radSum", 2:4)) {
  grid.edit(s, gp = gpar(font = 2, col = "blue"))
}
}

}
